@@ -1,4 +1,4 @@
-/* $Id: screens.c,v 1.8 2001/01/20 13:07:29 amai Exp $ */
+/* $Id: screens.c,v 1.12 2002/03/26 12:00:49 amai Exp $ */
 /*****************************************************************************/
 /**       Copyright 1988 by Evans & Sutherland Computer Corporation,        **/
 /**                          Salt Lake City, Utah                           **/
@@ -38,8 +38,12 @@
 
 #include <stdio.h>
 
-#include <Xm/Xm.h>
+#include <X11/keysym.h>
+
+#include <Xm/XmP.h>
 #include <Xm/MwmUtil.h>
+#include <XmI/XmI.h>
+
 #include "mwm.h"
 
 
@@ -56,14 +60,7 @@ static char stipple_bits[] = {
    0x55, 0x55, 0xee, 0xee, 0x55, 0x55, 0xba, 0xbb};
 #endif /* #if 0 */
 
-#include <X11/keysym.h>
 
-#include <Xm/XmP.h>
-
-
-extern Boolean debugging;
-
-extern XmFontList _XmFontListCreateDefault(Display *);
 /*
  * create the GC's for the menus
  */
@@ -80,8 +77,8 @@ create_menu_gcs(ScreenInfo *scr)
     gcv.function = GXcopy;
     if (scr->components[MWM_MENU].font_list == NULL)
 	scr->components[MWM_MENU].font_list = _XmFontListCreateDefault(dpy);
-    scr->components[MWM_MENU].font =
-	_XmGetFirstFont(scr->components[MWM_MENU].font_list);
+   _XmFontListGetDefaultFont(scr->components[MWM_MENU].font_list,
+	                     &scr->components[MWM_MENU].font);
     gcv.font = scr->components[MWM_MENU].font->fid;
     if (scr->components[MWM_MENU].font)
     {
@@ -184,8 +181,8 @@ create_feedback_gcs(ScreenInfo *scr)
     gcv.plane_mask = AllPlanes;
     if (scr->components[MWM_FEEDBACK].font_list == NULL)
 	scr->components[MWM_FEEDBACK].font_list = _XmFontListCreateDefault(dpy);
-    scr->components[MWM_FEEDBACK].font =
-	_XmGetFirstFont(scr->components[MWM_FEEDBACK].font_list);
+    _XmFontListGetDefaultFont(scr->components[MWM_FEEDBACK].font_list,
+                              &scr->components[MWM_FEEDBACK].font);
     gcv.font = scr->components[MWM_FEEDBACK].font->fid;
     if (scr->components[MWM_FEEDBACK].font)
     {
@@ -275,8 +272,8 @@ create_pager_gcs(ScreenInfo *scr)
     gcv.plane_mask = AllPlanes;
     if (scr->components[MWM_PAGER].font_list == NULL)
 	scr->components[MWM_PAGER].font_list = _XmFontListCreateDefault(dpy);
-    scr->components[MWM_PAGER].font =
-	_XmGetFirstFont(scr->components[MWM_PAGER].font_list);
+    _XmFontListGetDefaultFont(scr->components[MWM_PAGER].font_list,
+	            &scr->components[MWM_PAGER].font);
     gcv.font = scr->components[MWM_PAGER].font->fid;
     if (scr->components[MWM_PAGER].font)
     {
@@ -319,8 +316,8 @@ create_icon_gcs(ScreenInfo *scr)
     gcv.plane_mask = AllPlanes;
     if (scr->components[MWM_ICON].font_list == NULL)
 	scr->components[MWM_ICON].font_list = _XmFontListCreateDefault(dpy);
-    scr->components[MWM_ICON].font =
-	_XmGetFirstFont(scr->components[MWM_ICON].font_list);
+    _XmFontListGetDefaultFont(scr->components[MWM_ICON].font_list,
+	            &scr->components[MWM_ICON].font);
     gcv.font = scr->components[MWM_ICON].font->fid;
     if (scr->components[MWM_ICON].font)
     {
@@ -480,8 +477,8 @@ create_title_gcs(ScreenInfo *scr)
     gcv.plane_mask = AllPlanes;
     if (scr->components[MWM_TITLE_A].font_list == NULL)
 	scr->components[MWM_TITLE_A].font_list = _XmFontListCreateDefault(dpy);
-    scr->components[MWM_TITLE_A].font =
-	_XmGetFirstFont(scr->components[MWM_TITLE_A].font_list);
+    _XmFontListGetDefaultFont(scr->components[MWM_TITLE_A].font_list,
+                    &scr->components[MWM_TITLE_A].font);
     gcv.font = scr->components[MWM_TITLE_A].font->fid;
     if (scr->components[MWM_TITLE_A].font)
     {
@@ -1268,6 +1265,7 @@ SCREEN_Initialize(ScreenInfo *scr)
 
     PROP_SetPriorityColors(scr);
 
+    RES_GetScreenDefaults(scr);
     RES_GetComponentDefaults(scr);
 
     MWM_SetErrorHandler(REDIRECT);
