@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/lesstif/lesstif/include/Motif-1.2/XmI/XmI.h,v 1.62 2002/05/14 15:42:29 amai Exp $
+ * $Header: /cvsroot/lesstif/lesstif/include/Motif-1.2/XmI/XmI.h,v 1.73 2002/11/14 20:30:04 dannybackx Exp $
  * 
  * Copyright (C) 1995 Free Software Foundation, Inc.
  * Copyright  1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002 LessTif Development Team
@@ -46,6 +46,10 @@
 #include <Xm/Text.h>
 #include <XmI/MacrosI.h>
 
+#ifdef	USE_XFT
+#include <X11/Xft/Xft.h>
+#endif
+
 /*
  * extra resources
  */
@@ -86,6 +90,23 @@ typedef struct __XmRenditionRec {
 			underline_type;
 	XmTabList	tab_list;
 	Display		*dpy;
+#ifdef	USE_XFT
+	/* experimental - use of Xft, see http://fontconfig.org */
+	char		*font_style,
+/*			*family,	Use font_name instead. */
+			*font_foundry,
+			*font_encoding;
+	int		font_size,
+			pixel_size,
+			font_slant,
+			font_spacing,
+			font_weight;
+	XftPattern	*pattern;
+	XftFont		*xft_font;
+	XftColor	xft_foreground, xft_background;
+	int		font_average_width,	/* Calculated by LessTif,	*/
+			font_average_height;	/* depend only on xft_font	*/
+#endif
 #endif
 } *_XmRendition;
 
@@ -516,6 +537,16 @@ void _XmInitTraits(void);
 
 #endif /* XmVERSION >= 2 */
 
+#ifdef	USE_XFT
+/*
+ * XftDraw cache functions, implemented in lib/Xm/FontList.c
+ */
+XftDraw * _XmXftDrawCreate(Display *dpy, Window w);
+void _XmXftDrawDestroy(Display *dpy, Window w, XftDraw *d);
+void _XmXftDrawString(Display *dpy, Window wd, XmRendition r, int bpc, Position x, Position y, char *s, int len);
+void _XmXftFontAverageWidth(Widget w, XtPointer f, int *wid, int *ht);
+#endif
+
 /* from Vendor.c */
 extern Boolean _LtCheckClassOfVendorShell(Widget w);
 
