@@ -1,5 +1,5 @@
 /* 
- * $Id: lookup.c,v 1.11 2001/05/17 13:12:56 amai Exp $
+ * $Id: lookup.c,v 1.12 2001/09/06 12:38:41 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  * Copyright (C) 1995-2001 LessTif Development Team
@@ -92,6 +92,7 @@
 
 #include "misc.h"
 #include "lookup.h"
+#include "Mrm.h"
 
 #include <XmI/DebugUtil.h>
 
@@ -101,9 +102,6 @@ enum
     false, true
 };
 
-extern int NumberRegisteredFunctions;
-extern MrmRegisterArg *RegisteredFunctions;
-
 CreateFunctionsType CreateFunctions[] =
 {
     {"list", NULL},
