@@ -1,6 +1,6 @@
 /**
  *
- * $Id: DragBS.c,v 1.39 2001/05/17 13:04:49 amai Exp $
+ * $Id: DragBS.c,v 1.41 2001/11/28 16:48:42 dannybackx Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  * Copyright (C) 1995-2001 LessTif Development Team
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: DragBS.c,v 1.39 2001/05/17 13:04:49 amai Exp $";
+static const char rcsid[] = "$Id: DragBS.c,v 1.41 2001/11/28 16:48:42 dannybackx Exp $";
 
 #include <LTconfig.h>
 
@@ -157,7 +157,7 @@ create_drag_window(Display *display)
     XSetWindowAttributes attr;
 
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:create_drag_window(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:create_drag_window(%d)\n",
 		      __FILE__, __LINE__));
 
     attr.override_redirect = True;
@@ -189,7 +189,7 @@ read_drag_window(Display *display)
     Window win = None;
 
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:read_drag_window(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:read_drag_window(%d)\n",
 		      __FILE__, __LINE__));
 
     old_handler = XSetErrorHandler(DWError);
@@ -206,7 +206,7 @@ read_drag_window(Display *display)
     {
 	win = *((Window *)prop);
     }
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:read_drag_window(%d) - bafter=%i\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:read_drag_window(%d) - bafter=%i\n",
 		      __FILE__, __LINE__, bafter));
     if (prop)
     {
@@ -217,13 +217,13 @@ read_drag_window(Display *display)
 
     if (error_flag)
     {
-	DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:read_drag_window(%d) - None\n",
+	DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:read_drag_window(%d) - None\n",
 		      __FILE__, __LINE__));
 
 	return None;
     }
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:read_drag_window(%d) - %p\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:read_drag_window(%d) - %p\n",
 		      __FILE__, __LINE__, win));
 
     return win;
@@ -238,7 +238,7 @@ write_drag_window(Display *display, Window *win)
 {
     Atom mw;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:write_drag_window(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:write_drag_window(%d)\n",
 		      __FILE__, __LINE__));
 
     mw = XmInternAtom(display, _XA_MOTIF_DRAG_WINDOW, False);
@@ -256,7 +256,7 @@ get_drag_window(Display *display)
 {
     Window win;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:get_drag_window(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:get_drag_window(%d)\n",
 		      __FILE__, __LINE__));
 
     if (displayToDW == None)
@@ -267,13 +267,13 @@ get_drag_window(Display *display)
     if (XFindContext(display, DefaultRootWindow(display),
 		     displayToDW, (XPointer *)&win) == XCSUCCESS)
     {
-	DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:get_drag_window(%d) - %p\n",
+	DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:get_drag_window(%d) - %p\n",
 		      __FILE__, __LINE__, win));
 
 	return win;
     }
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:get_drag_window(%d) - None\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:get_drag_window(%d) - None\n",
 		      __FILE__, __LINE__));
 
     return None;
@@ -286,7 +286,7 @@ get_drag_window(Display *display)
 static void
 set_drag_window(Display *display, Window win)
 {
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:set_drag_window(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:set_drag_window(%d)\n",
 		      __FILE__, __LINE__));
 
     if (displayToDW == None)
@@ -312,12 +312,12 @@ set_drag_window(Display *display, Window win)
 static int
 protect_handler(Display *display, XErrorEvent *event)
 {
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:protect_handler(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:protect_handler(%d)\n",
 		      __FILE__, __LINE__));
 
     if (event->error_code == BadWindow &&
 	event->resourceid == errorWindow &&
-	event->serial > firstProtectRequest)
+	event->serial >= firstProtectRequest)
     {
 	bad_window = True;
 
@@ -339,7 +339,7 @@ protect_handler(Display *display, XErrorEvent *event)
 static void
 begin_protection(Display *display, Window win)
 {
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:begin_protection(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:begin_protection(%d)\n",
 		      __FILE__, __LINE__));
 
     bad_window = False;
@@ -358,7 +358,7 @@ begin_protection(Display *display, Window win)
 static void
 end_protection(Display *display)
 {
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:end_protection(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:end_protection(%d)\n",
 		      __FILE__, __LINE__));
 
     XSync(display, False);
@@ -380,7 +380,7 @@ _XmReadDragBuffer(XmDndBufMgr bmgr, Boolean read_string,
     int rsize, eaten;
     int size;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:_XmReadDragBuffer(%d) - read_string %s size %i\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:_XmReadDragBuffer(%d) - read_string %s size %i\n",
 		      __FILE__, __LINE__,
 		      read_string ? "True" : "False",
 		      bufsize));
@@ -444,7 +444,7 @@ _XmWriteDragBuffer(XmDndBufMgr bmgr, Boolean write_string,
 {
     int increase_needed;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:_XmWriteDragBuffer(%d) write_string %s size %i\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:_XmWriteDragBuffer(%d) write_string %s size %i\n",
 		      __FILE__, __LINE__,
 		      write_string ? "True" : "False",
 		      size));
@@ -521,7 +521,7 @@ read_atom_pairs(Display *display)
     unsigned long nitems, bafter;
     unsigned char *prop = NULL;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:read_atom_pairs(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:read_atom_pairs(%d)\n",
 		      __FILE__, __LINE__));
 
     pa = XmInternAtom(display, _XA_MOTIF_DRAG_ATOM_PAIRS, False);
@@ -536,7 +536,7 @@ read_atom_pairs(Display *display)
 			   (unsigned char **)&pairs) == Success &&
 	nitems >= 8 && pairs != NULL)
     {
-	DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:read_atom_pairs(%d) - got property, bafter=%i\n",
+	DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:read_atom_pairs(%d) - got property, bafter=%i\n",
 		      __FILE__, __LINE__, bafter));
 
 	gotit = True;
@@ -562,7 +562,7 @@ read_atom_pairs(Display *display)
 	    XFree((char *)prop);
 	}
 
-	DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:read_atom_pairs(%d) - did not get property\n",
+	DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:read_atom_pairs(%d) - did not get property\n",
 		      __FILE__, __LINE__));
 
 	return False;
@@ -580,7 +580,7 @@ read_atom_pairs(Display *display)
 	SWAP4BYTES(pairs->data_size);
     }
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:read_atom_pairs(%d) - %i pairs\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:read_atom_pairs(%d) - %i pairs\n",
 		      __FILE__, __LINE__,
 		      pairs->num_pairs));
 
@@ -606,7 +606,7 @@ read_atom_pairs(Display *display)
 
 	    _XmReadDragBuffer(&bmgr, True, buf, pair.namelen);
 
-	    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:read_atom_pairs(%d) - intern %s\n",
+	    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:read_atom_pairs(%d) - intern %s\n",
 		      __FILE__, __LINE__,
 		      buf));
 
@@ -638,7 +638,7 @@ write_atom_pairs(Display *display)
     int i;
     Window win;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:write_atom_pairs(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:write_atom_pairs(%d)\n",
 		      __FILE__, __LINE__));
 
     pair_rec.byte_order = _XmByteOrder();
@@ -715,7 +715,7 @@ get_atoms_table(Display *display)
 {
     XmDndAtomsTable tbl = NULL;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:get_atoms_table(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:get_atoms_table(%d)\n",
 		      __FILE__, __LINE__));
 
     if (displayToAtoms == (XContext)0)
@@ -741,7 +741,7 @@ set_atoms_table(Display *display, XmDndAtomsTable table)
 {
     XmDndAtomsTable atoms;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:set_atoms_table(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:set_atoms_table(%d)\n",
 		      __FILE__, __LINE__));
 
     if (displayToAtoms == (XContext)0)
@@ -769,7 +769,7 @@ create_default_atoms_table(Display *display)
 {
     XmDndAtomsTable tbl;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:create_default_atoms_table(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:create_default_atoms_table(%d)\n",
 		      __FILE__, __LINE__));
 
     tbl = (XmDndAtomsTable) XtMalloc(sizeof(XmDndAtomsTableRec));
@@ -799,7 +799,7 @@ read_atoms_table(Display *display, XmDndAtomsTable tbl)
     XmDndAtomsTableEntryRec atom_ent;
     XmDndBufMgrRec bmgr;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:read_atoms_table(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:read_atoms_table(%d)\n",
 		      __FILE__, __LINE__));
 
     da = XmInternAtom(display, _XA_MOTIF_DRAG_ATOMS, False);
@@ -814,7 +814,7 @@ read_atoms_table(Display *display, XmDndAtomsTable tbl)
 			   (unsigned char **)&atoms) == Success &&
 	nitems >= 8)
     {
-        DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:read_atom_pairs(%d) - got property, bafter=%i\n",
+        DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:read_atom_pairs(%d) - got property, bafter=%i\n",
 		      __FILE__, __LINE__, bafter));    
 	got_it = True;
     }
@@ -906,7 +906,7 @@ write_atoms_table(Display *display, XmDndAtomsTable tbl)
     int i;
     Window win;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:write_atoms_table(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:write_atoms_table(%d)\n",
 		      __FILE__, __LINE__));
 
     if (tbl == NULL)
@@ -970,7 +970,7 @@ get_targets_table(Display *display)
 {
     XmDndTargetsTable targets = NULL;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:get_targets_table(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:get_targets_table(%d)\n",
 		      __FILE__, __LINE__));
 
     if (displayToTargets == (XContext)0)
@@ -981,13 +981,13 @@ get_targets_table(Display *display)
     if (XFindContext(display, DefaultRootWindow(display),
 		     displayToTargets, (XPointer *)&targets) != XCSUCCESS)
     {
-	DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:get_targets_table(%d) - NULL\n",
+	DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:get_targets_table(%d) - NULL\n",
 		      __FILE__, __LINE__));
 
 	return NULL;
     }
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:get_targets_table(%d) - %p\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:get_targets_table(%d) - %p\n",
 		      __FILE__, __LINE__, targets));
 
     return targets;
@@ -1002,7 +1002,7 @@ set_targets_table(Display *display, XmDndTargetsTable table)
 {
     XmDndTargetsTable targets;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:set_targets_table(%d) - %p\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:set_targets_table(%d) - %p\n",
 		      __FILE__, __LINE__, table));
 
     if (displayToTargets == (XContext)0)
@@ -1036,7 +1036,7 @@ create_default_targets_table(Display *display)
 {
     XmDndTargetsTable tbl;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:create_default_targets_table(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:create_default_targets_table(%d)\n",
 		      __FILE__, __LINE__));
 
     tbl = (XmDndTargetsTable)XtMalloc(sizeof(XmDndTargetsTableRec));
@@ -1070,7 +1070,7 @@ read_targets_table(Display *display, XmDndTargetsTable tbl)
     XmDndBufMgrRec bmgr;
     CARD16 nents;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:read_targets_table(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:read_targets_table(%d)\n",
 		      __FILE__, __LINE__));
 
     ta = XmInternAtom(display, _XA_MOTIF_DRAG_TARGETS, False);
@@ -1085,7 +1085,7 @@ read_targets_table(Display *display, XmDndTargetsTable tbl)
 			   (unsigned char **)&targets) == Success &&
 	nitems >= 8)
     {
-        DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:read_targets_table(%d) bafter=%i\n",
+        DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:read_targets_table(%d) bafter=%i\n",
 		      __FILE__, __LINE__, bafter));
 
 	got_it = True;
@@ -1106,7 +1106,7 @@ read_targets_table(Display *display, XmDndTargetsTable tbl)
 	    XFree((XPointer)targets);
 	}
 
-	DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:read_targets_table(%d) - False\n",
+	DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:read_targets_table(%d) - False\n",
 		      __FILE__, __LINE__));
 
 	return False;
@@ -1178,7 +1178,7 @@ read_targets_table(Display *display, XmDndTargetsTable tbl)
 	}
     }
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:read_targets_table(%d) - True %p\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:read_targets_table(%d) - True %p\n",
 		      __FILE__, __LINE__, tbl));
 
     if (targets)
@@ -1200,7 +1200,7 @@ write_targets_table(Display *display, XmDndTargetsTable tbl)
     Window win;
     CARD16 nents;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:write_targets_table(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:write_targets_table(%d)\n",
 		      __FILE__, __LINE__));
 
     if (tbl == NULL)
@@ -1270,7 +1270,7 @@ _XmInitAtomPairs(Display *display)
     char *dstr;
     Display *d;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:_XmInitAtomPairs(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:_XmInitAtomPairs(%d)\n",
 		      __FILE__, __LINE__));
 
     if ((win = read_drag_window(display)) == None)
@@ -1331,7 +1331,7 @@ _XmInitTargetsTable(Display *display)
     XmDndAtomsTable atoms;
     XmDndTargetsTable targets;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:_XmInitTargetsTable(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:_XmInitTargetsTable(%d)\n",
 		      __FILE__, __LINE__));
 
     if ((win = read_drag_window(display)) == None)
@@ -1415,7 +1415,7 @@ _XmIndexToTargets(Widget shell, Cardinal t_index, Atom **targetsRtn)
     Display *dpy = XtDisplay(shell);
     XmDndTargetsTable targets;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:_XmIndexToTargets(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:_XmIndexToTargets(%d)\n",
 		      __FILE__, __LINE__));
 
     if ((targets = get_targets_table(dpy)) == NULL)
@@ -1459,7 +1459,7 @@ acompare(XmConst void *a, XmConst void *b)
 {
     Atom *pa = (Atom *)a, *pb = (Atom *)b;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:acompare(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:acompare(%d)\n",
 		      __FILE__, __LINE__));
 
     return *pa - *pb;
@@ -1477,7 +1477,7 @@ _XmTargetsToIndex(Widget shell, Atom *tlist, Cardinal numTargets)
     Cardinal i;
     Atom *atoms;
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, shell,
+    DEBUGOUT(_LtDebug(__FILE__, shell,
 		      "%s:_XmTargetsToIndex(%d) - %p 0x%x targets %p\n", 
 		      __FILE__, __LINE__,
 		      tlist,
@@ -1498,7 +1498,7 @@ _XmTargetsToIndex(Widget shell, Atom *tlist, Cardinal numTargets)
 
 	targets = get_targets_table(dpy);
     }
-    DEBUGOUT(_LtDebugDebug(__FILE__, shell,
+    DEBUGOUT(_LtDebug(__FILE__, shell,
 		      "%s:_XmTargetsToIndex(%d) - targets->num_entries %i\n", 
 		      __FILE__, __LINE__,
 		      targets->num_entries
@@ -1604,9 +1604,9 @@ _XmAllocMotifAtom(Widget shell, Time time)
     int i;
     char buf[32];
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, shell, "%s:_XmAllocMotifAtom(%d)\n",
+    DEBUGOUT(_LtDebug(__FILE__, shell, "%s:_XmAllocMotifAtom(%d)\n",
 		      __FILE__, __LINE__));
-    DEBUGOUT(_LtDebugDebug("DRAGSOURCE", shell, "%s:_XmAllocMotifAtom(%d)\n",
+    DEBUGOUT(_LtDebug("DRAGSOURCE", shell, "%s:_XmAllocMotifAtom(%d)\n",
 		      __FILE__, __LINE__));
 
     if ((atoms = get_atoms_table(dpy)) == NULL)
@@ -1670,7 +1670,7 @@ _XmAllocMotifAtom(Widget shell, Time time)
     {
     char *AtomName = XGetAtomName(dpy, alloc);
 
-	DEBUGOUT(_LtDebugDebug0("DRAGSOURCE", shell, "\t%s - 0x%x\n", AtomName, time));
+	DEBUGOUT(_LtDebug0("DRAGSOURCE", shell, "\t%s - 0x%x\n", AtomName, time));
     	XFree(AtomName);
     }
 
@@ -1688,7 +1688,7 @@ _XmFreeMotifAtom(Widget shell, Atom atom)
     Display *dpy = XtDisplay(shell);
     int i;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:_XmFreeMotifAtom(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:_XmFreeMotifAtom(%d)\n",
 		      __FILE__, __LINE__));
 
     if (atom == None)
@@ -1746,7 +1746,7 @@ _XmDestroyMotifWindow(Display *dpy)
     Atom dw;
     Window win;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:_XmDestroyMotifWindow(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:_XmDestroyMotifWindow(%d)\n",
 		      __FILE__, __LINE__));
 
     win = read_drag_window(dpy);
@@ -1776,7 +1776,7 @@ _XmGetDragProxyWindow(Display *display)
     unsigned long nitems, bafter;
     unsigned char *prop = NULL;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:_XmGetDragProxyWindow(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:_XmGetDragProxyWindow(%d)\n",
 		      __FILE__, __LINE__));
 
     if ((win = read_drag_window(display)) == None)
@@ -1786,21 +1786,23 @@ _XmGetDragProxyWindow(Display *display)
 
     pw = XmInternAtom(display, _XA_MOTIF_DRAG_PROXY_WINDOW, False);
 
+    begin_protection(display, win);
     if (XGetWindowProperty(display, win, pw, 0L, PROP_LENGTH,
 			   False, AnyPropertyType,
 			   &atype, &aformat, &nitems, &bafter, &prop)
 	== Success && atype == XA_WINDOW && aformat == 32 && nitems == 1)
     {
-        DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:_XmGetDragProxyWindow(%d) - bafter=%i\n",
+        DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:_XmGetDragProxyWindow(%d) - bafter=%i\n",
 		      __FILE__, __LINE__, bafter));
 	win = *((Window *)prop);
     }
+    end_protection(display);
     if (prop)
     {
 	XFree((XPointer)prop);
     }
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:_XmGetDragProxyWindow(%d) - %p\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:_XmGetDragProxyWindow(%d) - %p\n",
 		      __FILE__, __LINE__, win));
 
     return win;
@@ -1825,9 +1827,9 @@ _XmGetDragReceiverInfo(Display *display, Window win, XmDragReceiverInfo ri)
     unsigned long bafter, length;
     Window root;
 
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:_XmGetDragReceiverInfo(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:_XmGetDragReceiverInfo(%d)\n",
 		      __FILE__, __LINE__));
-    DEBUGOUT(_LtDebugDebug0("DRAGSOURCE", NULL, "%s:_XmGetDragReceiverInfo(%d) - %p\n",
+    DEBUGOUT(_LtDebug0("DRAGSOURCE", NULL, "%s:_XmGetDragReceiverInfo(%d) - %p\n",
 		      __FILE__, __LINE__, win));
 
     dri = XmInternAtom(display, _XA_MOTIF_DRAG_RECEIVER_INFO, False);
@@ -1836,7 +1838,7 @@ _XmGetDragReceiverInfo(Display *display, Window win, XmDragReceiverInfo ri)
 			   dri, &type, &format, &length, &bafter,
 			   (unsigned char **)&receiver) != Success)
     {
-	DEBUGOUT(_LtDebugDebug0("DRAGSOURCE", NULL, "%s:_XmGetDragReceiverInfo(%d) - getting prop failed\n",
+	DEBUGOUT(_LtDebug0("DRAGSOURCE", NULL, "%s:_XmGetDragReceiverInfo(%d) - getting prop failed\n",
 		      __FILE__, __LINE__));
 	return False;
     }
@@ -1845,7 +1847,7 @@ _XmGetDragReceiverInfo(Display *display, Window win, XmDragReceiverInfo ri)
     {
 	ri->dragProtocolStyle = XmDRAG_NONE;
 	XFree((char *)receiver);
-	DEBUGOUT(_LtDebugDebug0("DRAGSOURCE", NULL, "%s:_XmGetDragReceiverInfo(%d) - None available\n",
+	DEBUGOUT(_LtDebug0("DRAGSOURCE", NULL, "%s:_XmGetDragReceiverInfo(%d) - None available\n",
 		      __FILE__, __LINE__));
 	return False;
     }
@@ -1890,7 +1892,7 @@ _XmGetDragReceiverInfo(Display *display, Window win, XmDragReceiverInfo ri)
 extern void
 _XmSetDragReceiverInfo(Widget w, Widget widget)
 {
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:_XmSetDragReceiverInfo(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:_XmSetDragReceiverInfo(%d)\n",
 		      __FILE__, __LINE__));
 
 }
@@ -1899,7 +1901,7 @@ _XmSetDragReceiverInfo(Widget w, Widget widget)
 extern void
 _XmClearDragReceiverInfo(Widget shell)
 {
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:_XmClearDragReceiverInfo(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:_XmClearDragReceiverInfo(%d)\n",
 		      __FILE__, __LINE__));
 
 }
@@ -1910,7 +1912,7 @@ _XmClearDragReceiverInfo(Widget shell)
 extern void
 _XmFreeDragReceiverInfo(XmShellDropSiteInfo di)
 {
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:_XmFreeDragReceiverInfo(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:_XmFreeDragReceiverInfo(%d)\n",
 		      __FILE__, __LINE__));
 
     if (di)
@@ -1931,7 +1933,7 @@ _XmByteOrder(void)
     static unsigned char byte_order = 0;
 
     /*
-    DEBUGOUT(_LtDebugDebug0(__FILE__, NULL, "%s:_XmByteOrder(%d)\n",
+    DEBUGOUT(_LtDebug0(__FILE__, NULL, "%s:_XmByteOrder(%d)\n",
 		      __FILE__, __LINE__));
     */
 
