@@ -1,9 +1,9 @@
 /**
  *
- * $Id: Form.c,v 1.175 2001/12/04 22:35:18 dannybackx Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm/Form.c,v 1.180 2002/08/01 20:32:41 dannybackx Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
- * Copyright (C) 1996-2000 LessTif Development Team
+ * Copyright  1996, 1997, 1998, 1999, 2000, 2001, 2002 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Form.c,v 1.175 2001/12/04 22:35:18 dannybackx Exp $";
+static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm/Form.c,v 1.180 2002/08/01 20:32:41 dannybackx Exp $";
 
 #include <LTconfig.h>
 #include <XmI/XmI.h>
@@ -894,32 +894,23 @@ change_managed(Widget w)
 static int
 _XmGetOffset(Widget w, int i)
 {
-XmFormConstraints con = (XmFormConstraints)CoreConstraints(w);
-int offset;
+	XmFormConstraints con = (XmFormConstraints)CoreConstraints(w);
+	int offset;
 
-    if (FCP_Atta(con, i).offset == XmINVALID_DIMENSION)
-    {
-	if (FCP_Atta(con, i).type != XmATTACH_POSITION)
-	{
-	    if (i == TOP || i == BOTTOM)
-	    {
-		offset = Form_VertSpacing(XtParent(w));
-	    }
-	    else
-	    {
-		offset = Form_HorizSpacing(XtParent(w));
-	    }
+	if (FCP_Atta(con, i).offset == XmINVALID_DIMENSION) {
+		if (FCP_Atta(con, i).type != XmATTACH_POSITION) {
+			if (i == TOP || i == BOTTOM) {
+				offset = Form_VertSpacing(XtParent(w));
+			} else {
+				offset = Form_HorizSpacing(XtParent(w));
+			}
+		} else {
+			offset = 0;
+		}
+	} else {
+		offset = FCP_Atta(con, i).offset;
 	}
-	else
-	{
-	    offset = 0;
-	}
-    }
-    else
-    {
-	offset = FCP_Atta(con, i).offset;
-    }
-    return(offset);
+	return(offset);
 }
 
 
@@ -1465,16 +1456,40 @@ _XmFormFindPreferred(Widget f, Widget cw, XtWidgetGeometry *cg)
 	    }
 	    else
 	    {
+		/*
+		 * I'm not sure why this test was here, it breaks test60
+		 * because we forget to look at the current x/y for
+		 * XmATTACH_SELF cases.
+		 * Danny 16/6/2002
+		 *
 		p.x = XtWidth(f) != 0 ? 0 : XtX(child);
 		p.y = XtHeight(f) != 0 ? 0 : XtY(child);
+		 */
+		p.x = XtX(child);
+		p.y = XtY(child);
 		p.width = XtWidth(child);
 		p.height = XtHeight(child);
 		p.border_width = XtBorderWidth(child);
 		p.request_mode = CWWidth | CWHeight | CWBorderWidth;
 	    }
 
-	    SETX(p.x - 0 * p.border_width);
-	    SETY(p.y - 0 * p.border_width);
+	    if (FCP_Atta(con, LEFT).type == XmATTACH_SELF) {
+		p.request_mode |= CWX;
+	    } else {
+		SETX(p.x - 0 * p.border_width);
+	    }
+	    if (FCP_Atta(con, TOP).type == XmATTACH_SELF) {
+		p.request_mode |= CWY;
+	    } else if (FCP_Atta(con, TOP).type == XmATTACH_WIDGET &&
+		!XtIsManaged(FCP_Atta(con, TOP).w)) {
+		/* Danny experimental bug #531123 */
+		SETY(XtY(FCP_Atta(con, TOP).w));
+		DEBUGOUT(_LtDebug2(__FILE__, f, child, "Yow(%s) %d\n",
+			XtName(FCP_Atta(con, TOP).w),
+			XtY(FCP_Atta(con, TOP).w)));
+	    } else {
+		SETY(p.y - 0 * p.border_width);
+	    }
 
 	    /* rws 21 Aug 1997
 	       I took out the realized test to fix xdir->Connect->anon.
@@ -1614,15 +1629,14 @@ _XmFormPrintAttachmentReport(Widget f)
 static int
 XmFormPath(Widget f, Widget c, int dir, XmFormAttachment atta)
 {
-    XmFormConstraints con;
-    int x;
+	XmFormConstraints con;
+	int x;
+
+	if (c == NULL || !XtIsManaged(c)) {
+		DEBUGOUT(_LtDebug2(__FILE__, f, c, "XmFormPath -> 0\n"));
+		return 0;
+	}
 
-    if (c == NULL || !XtIsManaged(c))
-    {
-	x = 0;
-    }
-    else
-    {
 	con = (XmFormConstraints)CoreConstraints(c);
 
 	atta[dir].type = FCP_Atta(con, dir).type;
@@ -1675,7 +1689,6 @@ XmFormPath(Widget f, Widget c, int dir, XmFormAttachment atta)
 	    atta[dir].tempValue -= _XmGetOffset(c, dir);
 	    break;
 	case XmATTACH_OPPOSITE_WIDGET:
-#if 1
 	    if (FCP_Atta(con, dir).type == FCP_Atta(con, OPPOSITE_DIRECTION(dir)).type)
 	    {
 		/* 48 */
@@ -1694,10 +1707,7 @@ XmFormPath(Widget f, Widget c, int dir, XmFormAttachment atta)
 		    x = FCP_W(con1);
 		    FCP_W(con) = x;
 		}
-	    }
-	    else
-#endif
-	    {
+	    } else {
 		XmFormConstraints con1;
 
 		con1 = (XmFormConstraints)CoreConstraints(FCP_Atta(con, dir).w);
@@ -1724,9 +1734,9 @@ XmFormPath(Widget f, Widget c, int dir, XmFormAttachment atta)
 	{
 	    x += XmFormPath(f, FCP_Atta(con, dir).w, dir, atta);
 	}
-    }
 
-    return (x);
+	DEBUGOUT(_LtDebug2(__FILE__, f, c, "XmFormPath-> %d\n", x));
+	return x;
 }
 
 
@@ -1751,9 +1761,10 @@ _XmFormAllPaths(Widget f, Dimension *ww, Dimension *hh)
 	    atta[BOTTOM].tempValue = 0;
 	    y = XmFormPath(f, MGR_Children(f)[i], TOP, atta);
 	    z = XmFormPath(f, MGR_Children(f)[i], BOTTOM, atta);
-	    /*
-	    printf("%s %i %i %i\n", XtName(child), y, z, FCP_H(con));
-	    */
+
+		DEBUGOUT(_LtDebug2(__FILE__, f, child,
+			"_XmFormAllPaths %d + %d - %d -> %d\n",
+			y, z, FCP_H(con), y+z-FCP_H(con)));
 	    y += z;
 	    y -= FCP_H(con);
 
@@ -1761,9 +1772,11 @@ _XmFormAllPaths(Widget f, Dimension *ww, Dimension *hh)
 		atta[BOTTOM].type == XmATTACH_POSITION)
 	    {
 		/* test 2 test30 nedit */
-		if (atta[BOTTOM].percent != atta[TOP].percent)
+		if (atta[BOTTOM].percent != atta[TOP].percent) {
 			y = (y * Form_FractionBase(f) / (atta[BOTTOM].percent -
 						 atta[TOP].percent));
+	DEBUGOUT(_LtDebug(__FILE__, f, "_XmFormAllPaths %d y = %d\n", __LINE__, y));
+		}
 		atta[TOP].tempValue = 0;
 	    }
 	    else if (atta[BOTTOM].type == XmATTACH_POSITION)
@@ -1772,30 +1785,49 @@ _XmFormAllPaths(Widget f, Dimension *ww, Dimension *hh)
 
 		/* test18 test19 test27 */
 		/* Hack divide by 0 */
-		if (atta[BOTTOM].percent == 0)
+		if (atta[BOTTOM].percent == 0) {
 			y1 = (int)(((10 * (y) * Form_FractionBase(f)) + 5) / 10);
-		else
+	DEBUGOUT(_LtDebug(__FILE__, f, "_XmFormAllPaths %d y = %d\n", __LINE__, y1));
+		} else {
 			y1 = (int)(((10 * (y) * Form_FractionBase(f) / atta[BOTTOM].percent) + 5) / 10);
+	DEBUGOUT(_LtDebug(__FILE__, f, "_XmFormAllPaths %d y = %d\n", __LINE__, y1));
+		}
 		/* Hack divide by 0 */
-		if (Form_FractionBase(f) == 0)
+		if (Form_FractionBase(f) == 0) {
 			atta[TOP].tempValue = 100;	/* Does this make sense ? */
-		else
+		} else {
 			atta[TOP].tempValue = -y + (y1 * atta[BOTTOM].percent) / Form_FractionBase(f);
+		}
 		y = y1;
 	    }
 	    else if (atta[TOP].type == XmATTACH_POSITION)
 	    {
 		/* test28 test29 */
 		/* Hack divide by 0 */
-		if (Form_FractionBase(f) != atta[TOP].percent)
+		if (Form_FractionBase(f) != atta[TOP].percent) {
 			y = (int)(((10 * (y) * Form_FractionBase(f) / (Form_FractionBase(f) - atta[TOP].percent)) + 5) / 10);
+		}
 		/* Hack divide by 0 */
-		if (Form_FractionBase(f) != 0)
+		if (Form_FractionBase(f) != 0) {
 			atta[TOP].tempValue = (y * atta[TOP].percent) / Form_FractionBase(f);
+		}
+	    }
+	    else if (atta[TOP].type == XmATTACH_WIDGET)
+	    {
+	    	/* FIX ME */
+	    }
+	    else if (atta[TOP].type == XmATTACH_OPPOSITE_WIDGET)
+	    {
+	    	/* FIX ME */
+	    }
+	    else if (atta[TOP].type == XmATTACH_FORM)
+	    {
+	    	/* FIX ME */
 	    }
 	    else
 	    {
 		y += atta[TOP].tempValue;
+	DEBUGOUT(_LtDebug(__FILE__, f, "_XmFormAllPaths %d y = %d\n", __LINE__, y));
 	    }
 	    DEBUGOUT(_LtDebug2(__FILE__, (Widget)f, child,
 			       "_XmFormAllPaths H %5i %i %i\n", y, atta[TOP].tempValue, atta[BOTTOM].tempValue));
@@ -2122,10 +2154,11 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	XmFormConstraints con = (XmFormConstraints)CoreConstraints(child);
 
 	DEBUGOUT(_LtDebug0(__FILE__, (Widget)f,
-			   "Child #%d (%s)\tx %d y %d w %d h %d\n",
+			   "Child #%d (%s)\tx %d y %d w %d h %d %s\n",
 			   i, XtName(child),
 			   FCP_X(con), FCP_Y(con),
-			   FCP_W(con), FCP_H(con)));
+			   FCP_W(con), FCP_H(con),
+			   XtIsManaged(child) ? "" : "Not Managed"));
     }
 #endif
 
@@ -2354,7 +2387,7 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		 *   right) offset.
 		 */
 		/* Xinvest */
-
+		SETY(XtY(child));
 		FCP_Atta(con, TOP).percent = (FCP_Y(con) +
 					    _XmGetOffset(child, TOP)) *
 		    Form_FractionBase(f) / fh;
@@ -2530,6 +2563,7 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 		_XmWarning(child, "bottom ATTACH_SELF %s %d"
 			   " no test coverage", __FILE__, __LINE__);
 #endif
+		SETH(XtHeight(child) - XtY(child));	/* FIX ME */
 		FCP_Atta(con, BOTTOM).percent = (FCP_Y(con) + FCP_H(con) +
 					       _XmGetOffset(child, BOTTOM)) *
 		    Form_FractionBase(f) / fh;
@@ -2573,6 +2607,10 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 	    case XmATTACH_POSITION:
 		/* 13 31 7 8 */
 		{
+			/* Roundoff stuff complicates this,
+			 * it's really :
+			 * pos = percent * fw / ffb + offset
+			 */
 		    int pos = (((10 * FCP_Atta(con, LEFT).percent * fw /
 				 Form_FractionBase(f)) +
 				(10 * _XmGetOffset(child, LEFT))) + 5) / 10;
@@ -2675,14 +2713,21 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 
 		/* LEFT */
 	    case XmATTACH_SELF:
-		/* Xinvest */
-
-		FCP_Atta(con, LEFT).percent = (FCP_X(con) +
-					     _XmGetOffset(child, LEFT)) *
-		    Form_FractionBase(f) / fh;
-		FCP_Atta(con, LEFT).type = XmATTACH_POSITION;
-		FCP_Atta(con, LEFT).offset = 0;
+		/* 60, Xinvest */
+		SETX(XtX(child));
 		changed = True;
+		FCP_Atta(con, LEFT).percent = Form_FractionBase(f)
+			* (FCP_X(con) + _XmGetOffset(child, LEFT))
+			/ fw;
+		DEBUGOUT(_LtDebug2(__FILE__, f, child,
+			"LEFT SELF Yow %d  * (x %d + %d) / %d => %d\n",
+			Form_FractionBase(f),
+			FCP_X(con), _XmGetOffset(child, LEFT),
+			fw,
+			FCP_Atta(con, LEFT).percent));
+		/* We're using this value, so reset it only at the end. */
+		FCP_Atta(con, LEFT).offset = 0;
+		FCP_Atta(con, LEFT).type = XmATTACH_POSITION;
 		break;
 
 		/* Now for the illegal cases */
@@ -2841,15 +2886,20 @@ _XmFormLayout(Widget f, Widget cw, XtWidgetGeometry *cg,
 
 		/* RIGHT */
 	    case XmATTACH_SELF:
-		_XmWarning(child, "right ATTACH_SELF %s %d"
-			   " no test coverage", __FILE__, __LINE__);
-
-		FCP_Atta(con, RIGHT).percent = (FCP_X(con) + FCP_W(con) +
-					      _XmGetOffset(child, RIGHT)) *
-		    Form_FractionBase(f) / fh;
-		FCP_Atta(con, RIGHT).type = XmATTACH_POSITION;
-		FCP_Atta(con, RIGHT).offset = 0;
+		SETW(XtWidth(child));
+		FCP_Atta(con, RIGHT).percent = Form_FractionBase(f)
+			* (FCP_X(con) + FCP_W(con) + _XmGetOffset(child, RIGHT))
+			/ fw;
 		changed = True;
+		DEBUGOUT(_LtDebug2(__FILE__, f, child,
+			"RIGHT SELF Yow ffb %d * (x %d + w %d + %d) / %d => %d\n",
+			Form_FractionBase(f),
+			FCP_X(con), FCP_W(con),
+			_XmGetOffset(child, RIGHT), fw,
+			FCP_Atta(con, RIGHT).percent));
+		/* We're using this value so only reset it at the end */
+		FCP_Atta(con, RIGHT).offset = 0;
+		FCP_Atta(con, RIGHT).type = XmATTACH_POSITION;
 		break;
 
 		/* Now for the illegal cases */
