@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/lesstif/lesstif/lib/Xm/LabelG.c,v 1.114 2002/04/25 17:40:11 dannybackx Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm/LabelG.c,v 1.116 2002/11/13 20:35:03 dannybackx Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  * Copyright  1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002 LessTif Development Team
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm/LabelG.c,v 1.114 2002/04/25 17:40:11 dannybackx Exp $";
+static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm/LabelG.c,v 1.116 2002/11/13 20:35:03 dannybackx Exp $";
 
 #include <LTconfig.h>
 
@@ -283,11 +283,19 @@ static XtResource resources[] =
 	XmRImmediate, (XtPointer)NULL
     },
 #endif
+#if (XmVERSION > 1)
     {
 	XmNfontList, XmCFontList, XmRFontList,
 	sizeof(XmFontList), Offset(font),
 	XmRCallProc, (XtPointer)_XmLabelGSetRenderTable
     },
+#else
+    {
+	XmNfontList, XmCFontList, XmRFontList,
+	sizeof(XmFontList), Offset(font),
+	XmRImmediate, (XtPointer)0
+    },
+#endif
 #if (XmVERSION > 1)
     {
 	XmNrenderTable, XmCRenderTable, XmRRenderTable,
@@ -2388,16 +2396,6 @@ _XmLabelGSetRenderTable(Widget w, int o, XrmValue *v)
 	DEBUGOUT(_LtDebug(__FILE__, w, "_XmLabelGSetRenderTable (%d) Font was %p\n",
 		lw->label.check_set_render_table, lw->label.font));
 
-	/* HACK
-	 * this should be 0 or 1 (but it isn't).
-	 * Assume that another value is bogus, replace it by 0.
-	 */
-	if ( ! ((lw->label.check_set_render_table == 0) ||
-			(lw->label.check_set_render_table == 1))) {
-		lw->label.check_set_render_table = 0;
-		DEBUGOUT(_LtDebug(__FILE__, w, "_XmLabelGSetRenderTable restore bogus\n"));
-		_XmWarning(w, "_XmLabelGSetRenderTable restore bogus\n");
-	}
 	if (lw->label.check_set_render_table) {
 		/*
 		 * This is the second time we're getting called.
