@@ -1,9 +1,9 @@
 /**
  * 
- * $Id: MenuShell.c,v 1.86 2001/05/14 19:46:09 amai Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm/MenuShell.c,v 1.89 2002/04/12 18:13:27 dannybackx Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
- * Copyright (C) 1996-2001 LessTif Development Team
+ * Copyright  1996, 1997, 1998, 1999, 2000, 2001, 2002 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -23,12 +23,10 @@
  *
  **/
 
-static const char rcsid[] = "$Id: MenuShell.c,v 1.86 2001/05/14 19:46:09 amai Exp $";
+static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm/MenuShell.c,v 1.89 2002/04/12 18:13:27 dannybackx Exp $";
 
 #include <LTconfig.h>
 
-#include <assert.h>
-
 #include <XmI/XmI.h>
 #include <Xm/XmP.h>
 #include <Xm/MenuShellP.h>
@@ -40,6 +38,7 @@ static const char rcsid[] = "$Id: MenuShell.c,v 1.86 2001/05/14 19:46:09 amai Ex
 
 #include <XmI/DebugUtil.h>
 
+/* ? */
 #undef USE_FOCUS
 
 /* Forward Declarations */
@@ -221,7 +220,7 @@ static void
 initialize(Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebug(__FILE__, new_w,
 		      "%s:initialize(%d) - %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
@@ -287,7 +286,7 @@ initialize(Widget request, Widget new_w,
 
     if (XtWidth(new_w) == 0 || XtHeight(new_w) == 0)
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
+	DEBUGOUT(_LtDebug(__FILE__, new_w,
 			  "Initialize: dimensions %d %d changed to 1x1",
 			  XtWidth(new_w), XtHeight(new_w)));
 
@@ -305,7 +304,7 @@ initialize(Widget request, Widget new_w,
 static void
 destroy(Widget w)
 {
-    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Destroy\n"));
+    DEBUGOUT(_LtDebug(__FILE__, w, "Destroy\n"));
     _XmDestroyFocusData(MS_FocusData(w));
 
     XmFontListFree(MS_DefaultFontList(w));
@@ -316,7 +315,7 @@ destroy(Widget w)
 static void
 resize(Widget w)
 {
-    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Resize -- (%d, %d)\n",
+    DEBUGOUT(_LtDebug(__FILE__, w, "Resize -- (%d, %d)\n",
 		      XtWidth(w), XtHeight(w)));
 }
 
@@ -350,7 +349,7 @@ realize(Widget w,
     (*superclass->core_class.realize) (w, value_mask, attributes);
 #undef superclass
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Realize (size %dx%d)\n",
+    DEBUGOUT(_LtDebug(__FILE__, w, "Realize (size %dx%d)\n",
 		      XtWidth(w), XtHeight(w)));
 }
 
@@ -358,7 +357,7 @@ realize(Widget w,
 static void
 expose(Widget w, XEvent *event, Region region)
 {
-    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Expose\n"));
+    DEBUGOUT(_LtDebug(__FILE__, w, "Expose\n"));
 }
 #endif
 
@@ -366,27 +365,25 @@ static Boolean
 set_values(Widget current, Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "SetValues\n"));
+	DEBUGOUT(_LtDebug(__FILE__, new_w, "SetValues\n"));
+	DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
-    if (MS_DefaultFontList(new_w) != MS_DefaultFontList(current))
-    {
-	XmFontListFree(MS_DefaultFontList(current));
-	MS_DefaultFontList(new_w) = XmFontListCopy(MS_DefaultFontList(new_w));
-    }
+	if (MS_DefaultFontList(new_w) != MS_DefaultFontList(current)) {
+		XmFontListFree(MS_DefaultFontList(current));
+		MS_DefaultFontList(new_w) = XmFontListCopy(MS_DefaultFontList(new_w));
+	}
 
-    if (MS_ButtonFontList(new_w) != MS_ButtonFontList(current))
-    {
-	XmFontListFree(MS_ButtonFontList(current));
-	MS_ButtonFontList(new_w) = XmFontListCopy(MS_ButtonFontList(new_w));
-    }
+	if (MS_ButtonFontList(new_w) != MS_ButtonFontList(current)) {
+		XmFontListFree(MS_ButtonFontList(current));
+		MS_ButtonFontList(new_w) = XmFontListCopy(MS_ButtonFontList(new_w));
+	}
 
-    if (MS_LabelFontList(new_w) != MS_LabelFontList(current))
-    {
-	XmFontListFree(MS_LabelFontList(current));
-	MS_LabelFontList(new_w) = XmFontListCopy(MS_LabelFontList(new_w));
-    }
+	if (MS_LabelFontList(new_w) != MS_LabelFontList(current)) {
+		XmFontListFree(MS_LabelFontList(current));
+		MS_LabelFontList(new_w) = XmFontListCopy(MS_LabelFontList(new_w));
+	}
 
-    return True;
+	return True;
 }
 
 static XtGeometryResult
@@ -398,7 +395,7 @@ geometry_manager(Widget w,
     XtWidgetGeometry wants;
     Widget ms = XtParent(w);
 
-    DEBUGOUT(_LtDebugDebug2(__FILE__, ms, w,
+    DEBUGOUT(_LtDebug2(__FILE__, ms, w,
 		       "geometry_manager: request %s, allow_shell_resize %s\n",
 		       _LtDebugWidgetGeometry2String(request),
 		       ((XmMenuShellWidget)ms)->shell.allow_shell_resize
@@ -410,7 +407,7 @@ geometry_manager(Widget w,
 	return XtGeometryYes;
     }
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, ms,
+    DEBUGOUT(_LtDebug(__FILE__, ms,
 		      "geometry_manager: %s\n",
 		      _LtDebugWidgetGeometry2String(request)));
 
@@ -420,7 +417,7 @@ geometry_manager(Widget w,
 
     if (res == XtGeometryNo)
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, w,
+	DEBUGOUT(_LtDebug(__FILE__, w,
 			  "XtGeometryNo returned... THIS SHOULD NOT HAPPEN\n"));
     }
     *reply = wants;
@@ -434,7 +431,7 @@ geometry_manager(Widget w,
 	XtHeight(w) = wants.height;
     }
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w,
+    DEBUGOUT(_LtDebug(__FILE__, w,
 		      "geometry_manager: size %dx%d => Yes\n",
 		      reply->width, reply->height));
 
@@ -448,14 +445,14 @@ change_managed(Widget w)
     Widget child;
     Cardinal i;
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w,
+    DEBUGOUT(_LtDebug(__FILE__, w,
 		      "ChangeManaged: trying to find child to manage\n"));
 
     /* FIX ME: isn't having no managed children perfectly valid ? */
     child = NULL;
     for (i = 0; i < MGR_NumChildren(w); i++)
     {
-	DEBUGOUT(_LtDebugDebug2(__FILE__, w,
+	DEBUGOUT(_LtDebug2(__FILE__, w,
 			   MGR_Children(w)[i], "ChangeManaged [%d] %s\n",
 			   i,
 			   XtIsManaged(MGR_Children(w)[i])
@@ -516,7 +513,7 @@ change_managed(Widget w)
 
     if (!child)
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, w,
+	DEBUGOUT(_LtDebug(__FILE__, w,
 			  "change_managed: no managed children so we must"
 			  " be popping down\n"));
 
@@ -524,13 +521,13 @@ change_managed(Widget w)
 	{
 	    if (MS_PrivateShell(w))
 	    {
-		DEBUGOUT(_LtDebugDebug(__FILE__, w, "  PrivateShell\n"));
+		DEBUGOUT(_LtDebug(__FILE__, w, "  PrivateShell\n"));
 
 		_XmXtMenuPopdown(w, NULL, NULL, NULL);
 	    }
 	    else
 	    {
-		DEBUGOUT(_LtDebugDebug(__FILE__, w, "  public shell\n"));
+		DEBUGOUT(_LtDebug(__FILE__, w, "  public shell\n"));
 		_XmRemoveGrab(w);
 
 		XtUnmapWidget(w);
@@ -558,7 +555,7 @@ change_managed(Widget w)
     /* FIX ME: should look at the flags */
     _XmResizeObject(w, geo.width, geo.height, 0);
 
-    DEBUGOUT(_LtDebugDebug2(__FILE__, w, child,
+    DEBUGOUT(_LtDebug2(__FILE__, w, child,
 		       "ChangeManaged width %d height %d\n",
 		       XtWidth(w), XtHeight(w)));
 
@@ -572,7 +569,7 @@ change_managed(Widget w)
     {
 	XmMenuState state = _XmGetMenuState(child);
 
-	DEBUGOUT(_LtDebugDebug2(__FILE__, w, child, "Popping up\n"));
+	DEBUGOUT(_LtDebug2(__FILE__, w, child, "Popping up\n"));
 
 	_XmPostPopupMenu(child,
 			 (XEvent *)&state->RC_ButtonEventStatus.event);
@@ -586,23 +583,23 @@ change_managed(Widget w)
 	 */
 	if (RC_Type(child) == XmMENU_PULLDOWN)
 	{
-	    DEBUGOUT(_LtDebugDebug(__FILE__, child, "Pulldown posting\n"));
+	    DEBUGOUT(_LtDebug(__FILE__, child, "Pulldown posting\n"));
 
 	    if (RC_CascadeBtn(child))
 	    {
-	        DEBUGOUT(_LtDebugDebug(__FILE__, child, "Child is cascade\n"));
+	        DEBUGOUT(_LtDebug(__FILE__, child, "Child is cascade\n"));
 		if (XtIsManaged(child))
 		{
-	            DEBUGOUT(_LtDebugDebug(__FILE__, w, "Popping up\n"));
+	            DEBUGOUT(_LtDebug(__FILE__, w, "Popping up\n"));
 
 		    if (MS_PrivateShell(w))
 		    {
-			DEBUGOUT(_LtDebugDebug(__FILE__, w, "Popping up private shell\n"));
+			DEBUGOUT(_LtDebug(__FILE__, w, "Popping up private shell\n"));
 			MSClass_PopupSharedMenuPane(w)(w, child, NULL);
 		    }
 		    else
 		    {
-			DEBUGOUT(_LtDebugDebug(__FILE__, w, "Popping up public shell\n"));
+			DEBUGOUT(_LtDebug(__FILE__, w, "Popping up public shell\n"));
 		    	XtManageChild(child);
 			XMapRaised(XtDisplay(w), XtWindow(w));
 			XtCallCallbacks(w, XmNpopupCallback, NULL);
@@ -616,16 +613,16 @@ change_managed(Widget w)
 		}
 		else
 		{
-	            DEBUGOUT(_LtDebugDebug(__FILE__, w, "Popping down\n"));
+	            DEBUGOUT(_LtDebug(__FILE__, w, "Popping down\n"));
 
 		    if (MS_PrivateShell(w))
 		    {
-			DEBUGOUT(_LtDebugDebug(__FILE__, w, "  PrivateShell\n"));
+			DEBUGOUT(_LtDebug(__FILE__, w, "  PrivateShell\n"));
 			_XmXtMenuPopdown(w, NULL, NULL, NULL);
 		    }
 		    else
 		    {
-			DEBUGOUT(_LtDebugDebug(__FILE__, w, "  public shell\n"));
+			DEBUGOUT(_LtDebug(__FILE__, w, "  public shell\n"));
 			if (RC_Type(child) == XmMENU_POPUP || RC_Type(child) == XmMENU_OPTION)
 			{
 			    _XmRemoveGrab(w);
@@ -639,13 +636,13 @@ change_managed(Widget w)
 	    }
 	    else
 	    {
-		DEBUGOUT(_LtDebugDebug(__FILE__, w, "Not cascade\n"));
+		DEBUGOUT(_LtDebug(__FILE__, w, "Not cascade\n"));
 		/* Not attached to a cascade button. */
 		if (XtIsManaged(child))
 		{
 		    /* Probably the user managed the pulldown after creation. */
 		    /* FIX ME: Motif 2.0 does issue a warning here. */
-		    DEBUGOUT(_LtDebugDebug(__FILE__, child,
+		    DEBUGOUT(_LtDebug(__FILE__, child,
 				      "Unmanaging in change_managed...\n"));
 		    XtUnmanageChild(child);
 		}
@@ -698,13 +695,13 @@ MenuShellPopdownDone(Widget w, XEvent *event,
     Cardinal i;
     XmMenuState state = _XmGetMenuState(w);
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w, "MenuShellPopdownDone()\n"));
+    DEBUGOUT(_LtDebug(__FILE__, w, "MenuShellPopdownDone()\n"));
 
     /* make sure the trigger event here hasn't already been processed */
     if (_XmMenuGetInPMMode(w) && event && event->type == ButtonRelease &&
 	event->xbutton.time <= state->RC_ButtonEventStatus.time)
     {
-      DEBUGOUT(_LtDebugDebug(__FILE__, w, "MenuShellPopdownDone(): exiting b/c already processed\n"));
+      DEBUGOUT(_LtDebug(__FILE__, w, "MenuShellPopdownDone(): exiting b/c already processed\n"));
 	return;
     }
 
@@ -723,16 +720,16 @@ MenuShellPopdownDone(Widget w, XEvent *event,
 	event->xbutton.time <= state->RC_ButtonEventStatus.time
 				+ XtGetMultiClickTime(XtDisplay(w)))
     {
-      DEBUGOUT(_LtDebugDebug(__FILE__, w, "MenuShellPopdownDone(): exiting b/c user too fast\n"));
+      DEBUGOUT(_LtDebug(__FILE__, w, "MenuShellPopdownDone(): exiting b/c user too fast\n"));
 	return;
     }
 #endif
 
-    /* must be a menu shell here. */
-    assert(XmIsMenuShell(w));
+    /* must be a menu shell here:
+    assert(XmIsMenuShell(w)); */
 
     if (MGR_NumChildren(w) == 0) {
-      DEBUGOUT(_LtDebugDebug(__FILE__, w, "MenuShellPopdownDone(): exiting b/c no children\n"));
+      DEBUGOUT(_LtDebug(__FILE__, w, "MenuShellPopdownDone(): exiting b/c no children\n"));
 	return;
     }
     /* Find a child that is managed */
@@ -748,11 +745,11 @@ MenuShellPopdownDone(Widget w, XEvent *event,
 
     if (!rc)
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, w, "MenuShellPopdownDone - NO RC\n"));
+	DEBUGOUT(_LtDebug(__FILE__, w, "MenuShellPopdownDone - NO RC\n"));
 	return;
     }
 
-    DEBUGOUT(_LtDebugDebug2(__FILE__, w, rc,
+    DEBUGOUT(_LtDebug2(__FILE__, w, rc,
 		       "MenuShellPopdownDone - found RC %s, posted: %s\n",
 			XtName(rc), RC_PopupPosted(rc) ? XtName(RC_PopupPosted(rc)) : "NULL"));
 
@@ -760,7 +757,7 @@ MenuShellPopdownDone(Widget w, XEvent *event,
 
     if (!toplevelrc)
     {
-	DEBUGOUT(_LtDebugDebug2(__FILE__, w, rc, "No toplevelrc\n"));
+	DEBUGOUT(_LtDebug2(__FILE__, w, rc, "No toplevelrc\n"));
 
 #if 0
 	/*
@@ -782,7 +779,7 @@ MenuShellPopdownDone(Widget w, XEvent *event,
 	}
 	else
 	{
-	    DEBUGOUT(_LtDebugDebug2(__FILE__, w, rc,
+	    DEBUGOUT(_LtDebug2(__FILE__, w, rc,
 			       "No posted menu\n"));
 	    return;
 	}
@@ -794,18 +791,18 @@ MenuShellPopdownDone(Widget w, XEvent *event,
 
     if (XmIsMenuShell(menu_shell))
     {
-	assert(menu_shell && XmIsMenuShell(menu_shell));
+	/* assert(menu_shell && XmIsMenuShell(menu_shell)); */
 
 	if (Shell_PoppedUp(menu_shell))
 	{
-	    DEBUGOUT(_LtDebugDebug(__FILE__, w,
+	    DEBUGOUT(_LtDebug(__FILE__, w,
 		      "calling PopdownEveryone on toplevel menushell\n"));
 
 	    MSClass_PopdownEveryone(menu_shell)(menu_shell, event, params, &numparams);
 	}
 	else
 	{
-	    DEBUGOUT(_LtDebugDebug(__FILE__, w,
+	    DEBUGOUT(_LtDebug(__FILE__, w,
 		      "MenuShellPopdownDone - MenuShell is not popped up, must have been an accelerator\n"));
 	    return;
 	}
@@ -826,7 +823,7 @@ MenuShellPopdownDone(Widget w, XEvent *event,
 	}
 	else
 	{
-	    DEBUGOUT(_LtDebugDebug(__FILE__, rc, "Unmanaging...\n"));
+	    DEBUGOUT(_LtDebug(__FILE__, rc, "Unmanaging...\n"));
 	    {
 	    Widget realpar;
 
@@ -838,7 +835,7 @@ MenuShellPopdownDone(Widget w, XEvent *event,
 		{
 		    realpar = XtParent(rc);
 		}
-		DEBUGOUT(_LtDebugDebug("EMACS", realpar, "%s:%s(%d) - UNGRAB %i %i\n", __FILE__, "MenuShellPopdownDone" , __LINE__,
+		DEBUGOUT(_LtDebug("EMACS", realpar, "%s:%s(%d) - UNGRAB %i %i\n", __FILE__, "MenuShellPopdownDone" , __LINE__,
 			      RC_PostButton(rc), RC_PostModifiers(rc)));
 		XtUngrabButton(realpar,
 			       RC_PostButton(rc), RC_PostModifiers(rc));
@@ -846,7 +843,7 @@ MenuShellPopdownDone(Widget w, XEvent *event,
 	    XtUnmanageChild(rc);
 	    if (_XmIsActiveTearOff(rc))
 	    {
-		DEBUGOUT(_LtDebugDebug(__FILE__, rc, "Unmanaging... ActiveTearOff\n"));
+		DEBUGOUT(_LtDebug(__FILE__, rc, "Unmanaging... ActiveTearOff\n"));
 		XtManageChild(rc);
 	    }
 	}
@@ -859,7 +856,7 @@ MenuShellPopdownDone(Widget w, XEvent *event,
     {
 	Widget tmp = RC_LastSelectToplevel(rc);
 
-	DEBUGOUT(_LtDebugDebug(__FILE__, tmp,
+	DEBUGOUT(_LtDebug(__FILE__, tmp,
 			  "  calling menuDisarm on lastSelectTopLevel\n"));
 	RCClass_MenuProcs(XtClass(tmp))(XmMENU_DISARM, tmp, NULL);
     }
@@ -869,7 +866,7 @@ MenuShellPopdownDone(Widget w, XEvent *event,
     {
 	Widget tmp = XtParent(menu_shell);
 
-	DEBUGOUT(_LtDebugDebug(__FILE__, tmp,
+	DEBUGOUT(_LtDebug(__FILE__, tmp,
 			  "  calling menuDisarm on XtParent(menu_shell)\n"));
 
 	RCClass_MenuProcs(XtClass(tmp))(XmMENU_DISARM, tmp, NULL);
@@ -887,21 +884,21 @@ MenuShellPopdownOne(Widget w, XEvent *event,
 {
     Widget rc = NULL;
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w, "MenuShellPopdownOne()\n"));
+    DEBUGOUT(_LtDebug(__FILE__, w, "MenuShellPopdownOne()\n"));
 
-    /* must be a menu shell here. */
-    assert(XmIsMenuShell(w));
+    /* must be a menu shell here: 
+    assert(XmIsMenuShell(w)); */
 
     if (MGR_NumChildren(w) == 0)
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, w, "MenuShellPopdownOne: no children\n"));
+	DEBUGOUT(_LtDebug(__FILE__, w, "MenuShellPopdownOne: no children\n"));
 	return;
     }
 
     if (MS_PrivateShell(w) && XmIsRowColumn(XtParent(w)) &&
 	XmIsMenuBar(XtParent(w)))
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, w, "MenuShellPopdownOne: parent %s\n",
+	DEBUGOUT(_LtDebug(__FILE__, w, "MenuShellPopdownOne: parent %s\n",
 			  XtName(XtParent(w))));
 	rc = RC_PopupPosted(XtParent(w));
     }
@@ -912,16 +909,16 @@ MenuShellPopdownOne(Widget w, XEvent *event,
 
     if (!rc || !XmIsRowColumn(rc))
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, w, "MenuShellPopdownOne: no RC\n"));
+	DEBUGOUT(_LtDebug(__FILE__, w, "MenuShellPopdownOne: no RC\n"));
 	return;
     }
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w, "  Child menu pane is %s\n", XtName(rc)));
+    DEBUGOUT(_LtDebug(__FILE__, w, "  Child menu pane is %s\n", XtName(rc)));
 
     /* unhighlight the cascade button that popped us up. */
     if (RC_CascadeBtn(rc))
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, w, "  Removing myself from the cascade\n"));
+	DEBUGOUT(_LtDebug(__FILE__, w, "  Removing myself from the cascade\n"));
 
 	/* disarm the cascade button that pulled us down. */
 	if (XmIsPrimitive(RC_CascadeBtn(rc)))
@@ -940,17 +937,17 @@ MenuShellPopdownOne(Widget w, XEvent *event,
 	RC_CascadeBtn(rc) = NULL;
     }
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w, "  Popping down\n"));
+    DEBUGOUT(_LtDebug(__FILE__, w, "  Popping down\n"));
 
     if (MS_PrivateShell(w))
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, w, "  PrivateShell\n"));
+	DEBUGOUT(_LtDebug(__FILE__, w, "  PrivateShell\n"));
 
 	_XmXtMenuPopdown(w, NULL, NULL, NULL);
     }
     else
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, w, "  public shell\n"));
+	DEBUGOUT(_LtDebug(__FILE__, w, "  public shell\n"));
 	if (RC_Type(rc) == XmMENU_POPUP || RC_Type(rc) == XmMENU_OPTION)
 	{
 	    _XmRemoveGrab(w);
@@ -971,10 +968,10 @@ MenuShellPopdownEveryone(Widget w, XEvent *event,
     Cardinal i;
     Widget rc;
 
-    /* must be a menu shell here. */
-    assert(XmIsMenuShell(w));
+    /* must be a menu shell here:
+    assert(XmIsMenuShell(w)); */
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Popping down everyone - %i kids\n",
+    DEBUGOUT(_LtDebug(__FILE__, w, "Popping down everyone - %i kids\n",
     		MGR_NumChildren(w)));
 
     if (MGR_NumChildren(w) == 0)
@@ -986,7 +983,7 @@ MenuShellPopdownEveryone(Widget w, XEvent *event,
     {
 	rc = MGR_Children(w)[i];
 
-	DEBUGOUT(_LtDebugDebug2(__FILE__, w, rc,"child %i\n", i));
+	DEBUGOUT(_LtDebug2(__FILE__, w, rc,"child %i\n", i));
 	if (!rc)
 	{
 	    continue;
@@ -1004,18 +1001,18 @@ MenuShellPopdownEveryone(Widget w, XEvent *event,
 
 	    if (XmIsMenuShell(shell))
 	    {
-		DEBUGOUT(_LtDebugDebug(__FILE__, shell,
+		DEBUGOUT(_LtDebug(__FILE__, shell,
 			  "  recursing in PopdownEveryone.\n"));
 		MenuShellPopdownEveryone(shell, event, params, num_params);
 	    }
 	}
-	DEBUGOUT(_LtDebugDebug(__FILE__, w, "  calling popdownOne.\n"));
+	DEBUGOUT(_LtDebug(__FILE__, w, "  calling popdownOne.\n"));
 
 	MenuShellPopdownOne(XtParent(rc), event, params, num_params);
 #if 0
 	_XmCallRowColumnUnmapCallback(rc, event);
 #endif
-	DEBUGOUT(_LtDebugDebug2(__FILE__, w, rc,"child %i parent %s\n", 
+	DEBUGOUT(_LtDebug2(__FILE__, w, rc,"child %i parent %s\n", 
 			i,
 			XtName(XtParent(rc))));
     }
@@ -1034,10 +1031,10 @@ MenuShellPopdownEveryone(Widget w, XEvent *event,
 static void
 MenuShellPopupSharedMenuPane(Widget w, Widget w2, XEvent *event)
 {
-    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Popping up shared pane\n"));
+    DEBUGOUT(_LtDebug(__FILE__, w, "Popping up shared pane\n"));
 
-    /* must be a menu shell here. */
-    assert(XmIsMenuShell(w));
+    /* must be a menu shell here:
+    assert(XmIsMenuShell(w)); */
 
     XtManageChild(w2);
     XRaiseWindow(XtDisplay(w2), XtWindow(w2));
@@ -1054,7 +1051,7 @@ MenuShellPopupSharedMenuPane(Widget w, Widget w2, XEvent *event)
     Shell_PoppedUp(w) = True;
  
 #ifdef USE_FOCUS
-    DEBUGOUT(_LtDebugDebug(__FILE__, w,
+    DEBUGOUT(_LtDebug(__FILE__, w,
 		      "FOCUS SET: %s %s\n", XtName(w), XtName(w2)));
 
     _XmMenuFocus(w2, XmMENU_FOCUS_SET, CurrentTime);
@@ -1100,7 +1097,7 @@ static void
 _XmXtMenuPopdown(Widget widget, XEvent *event,
 		 String *params, Cardinal *num_params)
 {
-    DEBUGOUT(_LtDebugDebug(__FILE__, widget, "_XmXtMenuPopdown()\n"));
+    DEBUGOUT(_LtDebug(__FILE__, widget, "_XmXtMenuPopdown()\n"));
     if (MS_PrivateShell(widget))
     {
         XtUnmapWidget(widget);
@@ -1109,7 +1106,7 @@ _XmXtMenuPopdown(Widget widget, XEvent *event,
 
 #ifdef USE_FOCUS
 #if 0
-	DEBUGOUT(_LtDebugDebug(__FILE__, widget,
+	DEBUGOUT(_LtDebug(__FILE__, widget,
 			  "RESTORE: %s\n", XtName(widget)));
 
         _XmMenuFocus(widget, XmMENU_FOCUS_RESTORE, CurrentTime);
@@ -1125,7 +1122,7 @@ extern void
 _XmEnterRowColumn(Widget widget, XtPointer closure,
 		  XEvent *event, Boolean *cont)
 {
-    DEBUGOUT(_LtDebugDebug(__FILE__, widget,
+    DEBUGOUT(_LtDebug(__FILE__, widget,
 		      "_XmEnterRowColumn(not implemented)\n"));
 }
 
@@ -1134,7 +1131,7 @@ extern void
 _XmClearTraversal(Widget wid, XEvent *event,
 		  String *params, Cardinal *num_params)
 {
-    DEBUGOUT(_LtDebugDebug(__FILE__, wid,
+    DEBUGOUT(_LtDebug(__FILE__, wid,
 		      "_XmClearTraversal(not implemented)\n"));
 
     XAllowEvents(XtDisplay(wid), SyncPointer, CurrentTime);
