@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/lesstif/lesstif/lib/Xm/PanedW.c,v 1.51 2001/05/14 19:46:09 amai Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm/PanedW.c,v 1.54 2002/03/28 23:59:33 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  * Copyright (C) 1995-2001 LessTif Development Team
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: PanedW.c,v 1.51 2001/05/14 19:46:09 amai Exp $";
+static const char rcsid[] = "$Id: PanedW.c,v 1.54 2002/03/28 23:59:33 amai Exp $";
 
 #include <LTconfig.h>
 
@@ -536,12 +536,6 @@ XmPanedWindowClassRec xmPanedWindowClassRec = {
 
 WidgetClass xmPanedWindowWidgetClass = (WidgetClass)&xmPanedWindowClassRec;
 
-/*
- * handy defines
- */
-#define _XmMin(a,b)  ((a) < (b)) ? (a) : (b)
-#define _XmMax(a,b)  ((a) < (b)) ? (b) : (a)
-
 /*
  * default number of slots.  8 seems to be a reasonable number.
  */
@@ -673,7 +667,7 @@ static void
 initialize(Widget request, Widget new_w,
 	   ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebug(__FILE__, new_w,
 		      "%s:initialize(%d) - %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
@@ -705,7 +699,7 @@ static void
 constraint_initialize(Widget request, Widget new_w,
 		      ArgList args, Cardinal *num_args)
 {
-    DEBUGOUT(_LtDebugDebug2(__FILE__, XtParent(new_w), new_w,
+    DEBUGOUT(_LtDebug2(__FILE__, XtParent(new_w), new_w,
 		       "%s:constraint_initialize(%d) - %i args\n"
 		       "\trequest X %5i Y %5i W %5i H %5i\n"
 		       "\t  new_w X %5i Y %5i W %5i H %5i\n"
@@ -752,7 +746,7 @@ set_values(Widget old, Widget request, Widget new_w,
     Boolean redisplay = False;
     Arg argl[3];
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebug(__FILE__, new_w,
 		      "PanedWindow set_values****************\n"));
 
     if (PW_SeparatorOn(new_w) != PW_SeparatorOn(old))
@@ -868,7 +862,7 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
     Boolean redisplay = False;
     int i;
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, new_w, "PanedWindow constraint_set_values\n"));
+    DEBUGOUT(_LtDebug(__FILE__, new_w, "PanedWindow constraint_set_values\n"));
 
     if (!PWC_IsPane(current))
     {
@@ -933,7 +927,7 @@ constraint_set_values(Widget current, Widget request, Widget new_w,
 	redisplay = True;
     }
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, pw,
+    DEBUGOUT(_LtDebug(__FILE__, pw,
 		      "constraint_set_values: after PreferredSize\n"));
     return redisplay;
 }
@@ -944,7 +938,7 @@ resize(Widget w)
 {
     XtWidgetGeometry geo;
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w,
+    DEBUGOUT(_LtDebug(__FILE__, w,
 		      "PanedWindow Resize (%d %d)\n",
 		      XtWidth(w), XtHeight(w)));
 
@@ -965,7 +959,7 @@ realize(Widget w,
     XtWidgetGeometry geo;
     */
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:realize(%d) - %dx%d\n",
+    DEBUGOUT(_LtDebug(__FILE__, w, "%s:realize(%d) - %dx%d\n",
     	__FILE__, __LINE__,
     	XtWidth(w), XtHeight(w)));
 
@@ -1024,7 +1018,7 @@ insert_child(Widget w)
     XmPanedWindowWidget pw = (XmPanedWindowWidget)XtParent(w);
     int i;
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w, "%s:insert_child(%d)\n",
+    DEBUGOUT(_LtDebug(__FILE__, w, "%s:insert_child(%d)\n",
 		      __FILE__, __LINE__));
 
     if (PW_RecursivelyCalled(pw))
@@ -1109,7 +1103,7 @@ static XtGeometryResult
 query_geometry(Widget w, XtWidgetGeometry *proposed, XtWidgetGeometry *desired)
 {
     /* Motif does not have this method! */
-    DEBUGOUT(_LtDebugDebug(__FILE__, w,
+    DEBUGOUT(_LtDebug(__FILE__, w,
 		      "QueryGeometry (request %s)\n",
 		      _LtDebugWidgetGeometry2String(proposed)));
 
@@ -1155,17 +1149,17 @@ geometry_manager(Widget w,
     Widget pw = XtParent(w);
     XmKidGeometry boxes = NULL, sashes = NULL, seps = NULL;
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w, "PanedW: geometry manager\n"));
+    DEBUGOUT(_LtDebug(__FILE__, w, "PanedW: geometry manager\n"));
 
     if (AllPanesRealized(pw))
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, w,
+	DEBUGOUT(_LtDebug(__FILE__, w,
 			  "All panes realized in geom_manager\n"));
     }
 
     if (AllPanesRealized(pw) && !PWC_AllowResize(w) && XtIsManaged(pw))
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, w,
+	DEBUGOUT(_LtDebug(__FILE__, w,
 			  "Not allowing resize from geom_manager\n"));
 
 	return XtGeometryNo;
@@ -1180,7 +1174,7 @@ geometry_manager(Widget w,
 
     want = *request;
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w,
+    DEBUGOUT(_LtDebug(__FILE__, w,
 		      "geom_mgr: child %s wants %d %d\n", XtName(w),
 		      request->width, request->height));
 
@@ -1188,7 +1182,7 @@ geometry_manager(Widget w,
 
     if (_XmMakeGeometryRequest(pw, &geo) == XtGeometryYes)
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, pw,
+	DEBUGOUT(_LtDebug(__FILE__, pw,
 			  "We wanted %d %d and we got it\n",
 			  geo.width, geo.height));
     }
@@ -1196,7 +1190,7 @@ geometry_manager(Widget w,
     {
 	geo.width = XtWidth(pw);
 	geo.height = XtHeight(pw);
-	DEBUGOUT(_LtDebugDebug(__FILE__, pw,
+	DEBUGOUT(_LtDebug(__FILE__, pw,
 			  "We didn't get our request, but %d %d\n",
 			  geo.width, geo.height));
     }
@@ -1232,7 +1226,7 @@ geometry_manager(Widget w,
 	    PWC_DHeight(w) = want.height;
 	}
 
-	DEBUGOUT(_LtDebugDebug(__FILE__, w,
+	DEBUGOUT(_LtDebug(__FILE__, w,
 			  "Child wanted %d and that's ok\n", want.height));
 
 	_XmSetKidGeo(boxes, w);
@@ -1252,13 +1246,13 @@ geometry_manager(Widget w,
 
     if (good == 0)
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, w,
+	DEBUGOUT(_LtDebug(__FILE__, w,
 			  "Child wanted %d (NO)\n", want.height));
 	return XtGeometryNo;
     }
     else
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, w,
+	DEBUGOUT(_LtDebug(__FILE__, w,
 			  "Child wanted %d (ALMOST)\n", want.height));
 	return XtGeometryAlmost;
     }
@@ -1282,11 +1276,11 @@ change_managed(Widget w)
     int i, j;
     XtWidgetGeometry geo;
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w, "PanedWindow change_managed\n"));
+    DEBUGOUT(_LtDebug(__FILE__, w, "PanedWindow change_managed\n"));
 
     if (PW_RecursivelyCalled(w))
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, w,
+	DEBUGOUT(_LtDebug(__FILE__, w,
 			  "ChangeManaged was recursively called\n"));
 	return;
     }
@@ -1347,7 +1341,7 @@ change_managed(Widget w)
 
 		PW_PaneCount(w)++;
 
-		DEBUGOUT(_LtDebugDebug(__FILE__, MGR_Children(w)[i],
+		DEBUGOUT(_LtDebug(__FILE__, MGR_Children(w)[i],
 				  "Setting DHeight in change_managed to %d:"
 				  " was %d\n",
 				  XtHeight(MGR_Children(w)[i]),
@@ -1432,7 +1426,7 @@ change_managed(Widget w)
 	    CoreConstraints(PW_ManagedChildren(w)[PW_NumManagedChildren(w) - 1]);
     }
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Compute size in change_managed\n"));
+    DEBUGOUT(_LtDebug(__FILE__, w, "Compute size in change_managed\n"));
 
     _XmPanedWPreferredSize(w, NULL, NULL, &geo);
 
@@ -1497,7 +1491,7 @@ _XmPanedWPreferredSize(Widget w, Widget rchild,
     int i;
     Dimension curw, curh;
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmPanedWPreferredSize\n"));
+    DEBUGOUT(_LtDebug(__FILE__, w, "_XmPanedWPreferredSize\n"));
 
     /* Find widest child and compute total height of PanedWindow */
     curw = curh = 0;
@@ -1506,7 +1500,7 @@ _XmPanedWPreferredSize(Widget w, Widget rchild,
 
 	child = PW_ManagedChildren(w)[i];
 
-	DEBUGOUT(_LtDebugDebug2(__FILE__, w, child,
+	DEBUGOUT(_LtDebug2(__FILE__, w, child,
 			   "_XmPanedWPreferredSize: child wid %d ht %d "
 			   "pref.ht. %d panemin %d panemax %d\n",
 			   XtWidth(child), XtHeight(child),
@@ -1516,7 +1510,7 @@ _XmPanedWPreferredSize(Widget w, Widget rchild,
 	if (rchild != NULL && child == rchild && childgeom != NULL &&
 	    childgeom->request_mode & CWWidth)
 	{
-	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Using childgeom width: %d\n",
+	    DEBUGOUT(_LtDebug(__FILE__, w, "Using childgeom width: %d\n",
 			      childgeom->width));
 	    curw = curw >= childgeom->width ? curw : childgeom->width;
 	}
@@ -1533,7 +1527,7 @@ _XmPanedWPreferredSize(Widget w, Widget rchild,
 	if (rchild != NULL && child == rchild && childgeom != NULL &&
 	    childgeom->request_mode & CWHeight)
 	{
-	    DEBUGOUT(_LtDebugDebug(__FILE__, w, "Using childgeom height: %d\n",
+	    DEBUGOUT(_LtDebug(__FILE__, w, "Using childgeom height: %d\n",
 			      childgeom->height));
 	    curh += _XmMax(childgeom->height, PWC_PaneMinimum(child));
 	}
@@ -1546,7 +1540,7 @@ _XmPanedWPreferredSize(Widget w, Widget rchild,
     curh += 2 * PW_MarginHeight(w);
     curw += 2 * PW_MarginWidth(w);
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w,
+    DEBUGOUT(_LtDebug(__FILE__, w,
 		      "_XmPanedWPreferredSize: computed %d %d\n",
 		      curw, curh));
 
@@ -1788,14 +1782,14 @@ MoveBorder(Widget pw, Widget sash, int dy)
 	/* Move up? */
 	if (dy < 0)
 	{
-	    DEBUGOUT(_LtDebugDebug(__FILE__, pw,
+	    DEBUGOUT(_LtDebug(__FILE__, pw,
 			      "Move up: %d %d %d %d %d\n",
 			      PWC_DHeight(panea), dy, PWC_PaneMaximum(panea),
 			      PWC_DHeight(paneb), PWC_PaneMinimum(paneb)));
 
 	    if ((XtHeight(paneb) - dy) < PWC_PaneMaximum(paneb))
 	    {
-		DEBUGOUT(_LtDebugDebug(__FILE__, paneb, "Widget can grow\n"));
+		DEBUGOUT(_LtDebug(__FILE__, paneb, "Widget can grow\n"));
 
 		/*
 		 * can panea absorb the change?  If so, then adjust paneb's
@@ -1803,7 +1797,7 @@ MoveBorder(Widget pw, Widget sash, int dy)
 		 */
 		if ((XtHeight(panea) + dy) > PWC_PaneMinimum(panea))
 		{
-		    DEBUGOUT(_LtDebugDebug(__FILE__, panea,
+		    DEBUGOUT(_LtDebug(__FILE__, panea,
 				      "Widget can shrink to compensate\n"));
 
 		    cons = PW_BottomPane(pw);
@@ -1831,7 +1825,7 @@ MoveBorder(Widget pw, Widget sash, int dy)
 		 */
 		else if (XtHeight(panea) > PWC_PaneMinimum(panea))
 		{
-		    DEBUGOUT(_LtDebugDebug(__FILE__, panea,
+		    DEBUGOUT(_LtDebug(__FILE__, panea,
 				      "Widget can partially compensate\n"));
 
 		    dy += XtHeight(panea) - PWC_PaneMinimum(panea);
@@ -1859,7 +1853,7 @@ MoveBorder(Widget pw, Widget sash, int dy)
 		    sash = PWC_Sash(PW_ManagedChildren(pw)[PWC_Position(panea) - 1]);
 		    panea = PWC_Sash(sash);
 
-		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb,
+		    DEBUGOUT(_LtDebug(__FILE__, paneb,
 				      "And we'll try to pass it off\n"));
 
 		    PW_TopPane(pw) = (XmPanedWindowConstraintPtr)
@@ -1870,7 +1864,7 @@ MoveBorder(Widget pw, Widget sash, int dy)
 		 */
 		else
 		{
-		    DEBUGOUT(_LtDebugDebug(__FILE__, pw,
+		    DEBUGOUT(_LtDebug(__FILE__, pw,
 				      "No more panes above\n"));
 		    return;
 		}
@@ -1887,7 +1881,7 @@ MoveBorder(Widget pw, Widget sash, int dy)
 		    sash = PWC_Sash(paneb);
 		    paneb = PW_ManagedChildren(pw)[PWC_Position(PWC_Sash(sash)) + 1];
 
-		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb,
+		    DEBUGOUT(_LtDebug(__FILE__, paneb,
 				      "And we'll try to pass it off\n"));
 
 		    PW_BottomPane(pw) = (XmPanedWindowConstraintPtr)
@@ -1895,7 +1889,7 @@ MoveBorder(Widget pw, Widget sash, int dy)
 		}
 		else
 		{
-		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb, "Can't grow\n"));
+		    DEBUGOUT(_LtDebug(__FILE__, paneb, "Can't grow\n"));
 		    return;
 		}
 	    }
@@ -1903,7 +1897,7 @@ MoveBorder(Widget pw, Widget sash, int dy)
 	/* move down? */
 	else if (dy > 0)
 	{
-	    DEBUGOUT(_LtDebugDebug(__FILE__, pw,
+	    DEBUGOUT(_LtDebug(__FILE__, pw,
 			      "Move down: %d %d %d %d %d\n",
 			      PWC_DHeight(panea), dy, PWC_PaneMaximum(panea),
 			      PWC_DHeight(paneb), PWC_PaneMinimum(paneb)));
@@ -1911,7 +1905,7 @@ MoveBorder(Widget pw, Widget sash, int dy)
 	    /* can panea grow? */
 	    if ((XtHeight(panea) + dy) < PWC_PaneMaximum(panea))
 	    {
-		DEBUGOUT(_LtDebugDebug(__FILE__, panea, "Pane can grow\n"));
+		DEBUGOUT(_LtDebug(__FILE__, panea, "Pane can grow\n"));
 
 		/*
 		 * can paneb absorb the change?  If so, increate panea's
@@ -1919,7 +1913,7 @@ MoveBorder(Widget pw, Widget sash, int dy)
 		 */
 		if ((XtHeight(paneb) - dy) > PWC_PaneMinimum(paneb))
 		{
-		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb,
+		    DEBUGOUT(_LtDebug(__FILE__, paneb,
 				  "And this pane can shrink to compensate\n"));
 
 		    XtHeight(paneb) -= dy;
@@ -1947,7 +1941,7 @@ MoveBorder(Widget pw, Widget sash, int dy)
 		 */
 		else if (XtHeight(paneb) > PWC_PaneMinimum(paneb))
 		{
-		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb,
+		    DEBUGOUT(_LtDebug(__FILE__, paneb,
 				  "And this pane can partially compensate\n"));
 
 		    dy -= XtHeight(paneb) - PWC_PaneMinimum(paneb);
@@ -1976,7 +1970,7 @@ MoveBorder(Widget pw, Widget sash, int dy)
 		    sash = PWC_Sash(paneb);
 		    paneb = PW_ManagedChildren(pw)[PWC_Position(PWC_Sash(sash)) + 1];
 
-		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb,
+		    DEBUGOUT(_LtDebug(__FILE__, paneb,
 				      "And we'll try to pass it off\n"));
 
 		    PW_BottomPane(pw) = (XmPanedWindowConstraintPtr)
@@ -1987,7 +1981,7 @@ MoveBorder(Widget pw, Widget sash, int dy)
 		 */
 		else
 		{
-		    DEBUGOUT(_LtDebugDebug(__FILE__, pw, "No more panes below\n"));
+		    DEBUGOUT(_LtDebug(__FILE__, pw, "No more panes below\n"));
 		    return;
 		}
 	    }
@@ -2003,7 +1997,7 @@ MoveBorder(Widget pw, Widget sash, int dy)
 		    sash = PWC_Sash(PW_ManagedChildren(pw)[PWC_Position(panea) - 1]);
 		    panea = PWC_Sash(sash);
 
-		    DEBUGOUT(_LtDebugDebug(__FILE__, paneb,
+		    DEBUGOUT(_LtDebug(__FILE__, paneb,
 				      "And we'll try to pass it off\n"));
 
 		    PW_TopPane(pw) = (XmPanedWindowConstraintPtr)
@@ -2011,7 +2005,7 @@ MoveBorder(Widget pw, Widget sash, int dy)
 		}
 		else
 		{
-		    DEBUGOUT(_LtDebugDebug(__FILE__, panea, "Pane can't grow\n"));
+		    DEBUGOUT(_LtDebug(__FILE__, panea, "Pane can't grow\n"));
 
 		    return;
 		}
@@ -2020,7 +2014,7 @@ MoveBorder(Widget pw, Widget sash, int dy)
 	/* no motion, bail out */
 	else
 	{
-	    DEBUGOUT(_LtDebugDebug(__FILE__, pw, "No motion\n"));
+	    DEBUGOUT(_LtDebug(__FILE__, pw, "No motion\n"));
 
 	    return;
 	}
@@ -2037,15 +2031,21 @@ SashAction(Widget w, XtPointer client_data, XtPointer call_data)
     XButtonEvent *ev = (XButtonEvent *)event;
     Widget child, separator, sash;
 
+    /* 
+     * toed: PW_TopPane and PW_BottomPane need to be set at 
+     * every call in case MotionBorder changed them during the 
+     * last call 
+     */
+    PW_TopPane(pw) = (XmPanedWindowConstraintPtr)
+	CoreConstraints(PWC_Sash(w));
+    PW_BottomPane(pw) = (XmPanedWindowConstraintPtr)
+	CoreConstraints(PW_ManagedChildren(pw)
+	    [PWC_Position(PWC_Sash(w)) + 1]);
+
     switch (event->type)
     {
     case ButtonPress:
 	PW_StartY(pw) = ev->y;
-	PW_TopPane(pw) = (XmPanedWindowConstraintPtr)
-	    CoreConstraints(PWC_Sash(w));
-	PW_BottomPane(pw) = (XmPanedWindowConstraintPtr)
-	    CoreConstraints(PW_ManagedChildren(pw)
-			    [PWC_Position(PWC_Sash(w)) + 1]);
 	break;
 
     case ButtonRelease:
@@ -2063,7 +2063,7 @@ SashAction(Widget w, XtPointer client_data, XtPointer call_data)
 
 	    }
 
-	    DEBUGOUT(_LtDebugDebug(__FILE__, child,
+	    DEBUGOUT(_LtDebug(__FILE__, child,
 			      "SashAction places child %dx%d, size %d/%d\n",
 			      XtX(child), PWC_DY(child),
 			      XtWidth(child), XtHeight(child)));
@@ -2103,7 +2103,7 @@ SashAction(Widget w, XtPointer client_data, XtPointer call_data)
 		sash = PWC_Sash(child);
 		if (sash)
 		{
-		    DEBUGOUT(_LtDebugDebug(__FILE__, pw,
+		    DEBUGOUT(_LtDebug(__FILE__, pw,
 				      "moving sash to : %d %d w = %d h = %d\n",
 				      XtWidth(pw) - PW_MarginWidth(pw) +
 				      PW_SashIndent(pw) - PW_SashWidth(pw),
