@@ -1,6 +1,6 @@
 /**
  *
- * $Id: ResConvert.c,v 1.85 2002/04/12 23:38:44 amai Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm/ResConvert.c,v 1.115 2002/11/15 17:11:40 dannybackx Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  * Copyright  1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002 LessTif Development Team
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: ResConvert.c,v 1.85 2002/04/12 23:38:44 amai Exp $";
+static const char rcsid[] = "$Id: ResConvert.c,v 1.115 2002/11/15 17:11:40 dannybackx Exp $";
 
 #include <LTconfig.h>
 
@@ -46,6 +46,9 @@ static const char rcsid[] = "$Id: ResConvert.c,v 1.85 2002/04/12 23:38:44 amai E
 
 #include <XmI/DebugUtil.h>
 
+#if	USE_XFT
+#include <X11/Xft/Xft.h>
+#endif
 
 enum
 {
@@ -103,6 +106,14 @@ static Boolean _XmCvtStringToTabList(Display *display, XrmValue *args,
 static Boolean _XmCvtStringToRenderTable(Display *display, XrmValue *args,
 		       Cardinal *num_args, XrmValue *from, XrmValue *to,
 		       XtPointer *converter_data);
+#if	USE_XFT
+static Boolean XmuCvtStringToXftColor(Display *dpy,
+                       XrmValue *args, Cardinal *num_args,
+                       XrmValue *fromVal, XrmValue *toVal,
+                       XtPointer *converter_data);
+static void XmuFreeXftColor(XtAppContext app, XrmValuePtr toVal, XtPointer closure,
+                 XrmValuePtr args, Cardinal *num_args);
+#endif	/* USE_XFT */
 #endif /* XmVERSION > 1 */
 
 
@@ -523,6 +534,12 @@ static XtConvertArgRec parentArgs[] = {
 		sizeof(Widget) }
 };
 
+static XtConvertArgRec selfArgs[] = {
+	{	XtBaseOffset,
+		(XtPointer)XtOffsetOf(WidgetRec, core.self),
+		sizeof(Widget) }
+};
+
 #if XmVERSION > 1
 static char *toggle_mode[] =
 {
@@ -631,6 +648,18 @@ static unsigned char indicator_on_values[] =
     XmINDICATOR_FILL,		/* yes */
 };
 
+
+#ifdef	USE_XFT
+static XtConvertArgRec xftColorConvertArgs[] = {
+  {XtWidgetBaseOffset, (XtPointer)XtOffsetOf(WidgetRec, core.screen),
+	sizeof(Screen *)},
+  {XtWidgetBaseOffset, (XtPointer)XtOffsetOf(WidgetRec, core.colormap),
+	sizeof(Colormap)}
+};
+
+#define	XtRXftColor	"XftColor"
+#endif
+
 #endif /* XmVERSION > 1 */
 
 
@@ -1023,11 +1052,17 @@ _XmRegisterConverters(void)
     XtSetTypeConverter(XtRString,	/* source type */
 		       XmRRenderTable,	/* target type */
 		       _XmCvtStringToRenderTable,	/* converter routine */
-		       NULL,	/* args for converter routine */
-		       0,	/* number of args to converter routine */
+		       selfArgs,	/* args for converter routine */
+		       XtNumber(selfArgs),	/* number of args to converter routine */
 		       XtCacheNone,	/* caching instructions */
 		       NULL);	/* destructor function */
-#endif
+#if	USE_XFT
+    XtSetTypeConverter (XtRString, XtRXftColor,
+			XmuCvtStringToXftColor,
+			xftColorConvertArgs, XtNumber(xftColorConvertArgs),
+			XtCacheByDisplay, XmuFreeXftColor);
+#endif	/* USE_XFT */
+#endif	/* XmVERSION > 1 */
 
 #ifdef NONSTANDARD_CONVERTERS
     _XmRegisterNSEConverters();
@@ -1102,9 +1137,14 @@ _LtWarning(Widget w, const char *fmt, va_list arg_list)
     	                      "\n    Name: %s\n    Class: %s\n    ",
 		              XtName(w), XtClass(w)->core_class.class_name);
 #else
-	charsleft -= sprintf(buf,
-	                     "\n    Name: %s\n    Class: %s\n    ",
+#ifndef VOID_SPRINTF
+	charsleft -=
+#endif
+	sprintf(buf, "\n    Name: %s\n    Class: %s\n    ",
 		             XtName(w), XtClass(w)->core_class.class_name);
+#ifdef VOID_SPRINTF
+	charsleft -= strlen(buf);
+#endif
         /* one might add a catch here ... */
 #endif
 
@@ -1374,7 +1414,8 @@ _XmStringsAreEqual(char *in_str,
  * Fixed this thing to make a COPY of the fontlist it returns.
  * Danny 16/4/1996
  */
-extern XmFontList _XmGetDefaultFontList(Widget w, unsigned char fontListType)
+extern XmFontList
+_XmGetDefaultFontList(Widget w, unsigned char fontListType)
 {
     XmFontList labelFontList = NULL;
     XmFontList buttonFontList = NULL;
@@ -2686,8 +2727,7 @@ get_unit_type(Widget w, Cardinal *size, XrmValue *val)
     ut = _XmGetUnitType(w);
 }
 
-
-#if XmVERSION > 1
+#if (XmVERSION > 1)
 /*
  * XmConvertStringToUnits converts a string specification value and returns
  * the converted value as the return value from the function. This function
@@ -2724,8 +2764,9 @@ XmConvertStringToUnits(Screen *screen, String spec, int orientation,
 	}
     return 0;
 }
+#endif
 
-
+#if (XmVERSION > 1)
 static Boolean
 _XmCvtStringToTabList(Display *display,
 		       XrmValue *args,
@@ -2734,22 +2775,610 @@ _XmCvtStringToTabList(Display *display,
 		       XrmValue *to,
 		       XtPointer *converter_data)
 {
+	int			i, count;
+	struct _XmTabRec	**tabs, *thistab;
+	char			*c, *p, *q;
+	struct _XmTabListRec	**d;
+
+	DEBUGOUT(_LtDebug(__FILE__, NULL, "_XmCvtStringToTabList(%s)\n", from->addr));
+	for (i=0, count=1; from->addr[i]; i++)
+		if (from->addr[i] == ',')
+			count++;
+
+	tabs = (struct _XmTabRec **)XtCalloc(count, sizeof(struct _XmTabRec *));
+	for (i=0; i<count; i++)
+		tabs[i] = (struct _XmTabRec *)XtMalloc(sizeof(struct _XmTabRec));
+
+	c = q = XtNewString(from->addr);
+	i=0;
+	while (q) {
+		char	unit[20];
+
+		thistab = tabs[i];
+		p = strchr(q, ',');
+		if (p) {
+			*p = '\0';
+		}
+		/* Convert this into an entry */
+		while (isspace(*q)) q++;
+		thistab->offset_model = (*q == '+') ? XmRELATIVE : XmABSOLUTE;
+
+		unit[0] = '\0';
+		(void) sscanf(q, "%f%s", &thistab->value, &unit[0]);
+
+		/* FIX ME I haven't a clue */
+		if (strcasecmp("in", unit) == 0) {
+			thistab->units = XmINCHES;
+		} else if (strcasecmp("mm", unit) == 0) {
+			thistab->units = XmMILLIMETERS;
+		} else if (strcasecmp("cm", unit) == 0) {
+			thistab->units = XmCENTIMETERS;
+		} else {
+			thistab->units = XmPIXELS;
+		}
+
+		thistab->alignment = XmALIGNMENT_BEGINNING;
+		thistab->decimal = 0;
+
+		if (p) {
+			q = p+1;
+		} else {
+			break;
+		}
+		i++;
+	}
+	XtFree(c);
+
+	if (to->addr == NULL) {
+		to->size = sizeof(XtPointer);
+		return False;
+	} else {
+		if (to->size < sizeof(XtPointer)) {
+			to->size = sizeof(XtPointer);
+			return False;
+		} else {
+			d = (struct _XmTabListRec **)to->addr;
+			*d = (struct _XmTabListRec *)XtMalloc(sizeof(struct _XmTabListRec));
+			(*d)->tabs = tabs;
+			(*d)->count = count;
+		}
+	}
+
+/*
+	if (to->addr == NULL) {
+		to->addr = (XPointer)&tip;
+		to->size = sizeof tip;
+	} else {
+		if (to->size < sizeof tip) {
+		    XtDisplayStringConversionWarning(display, from->addr,
+					     XmRTopItemPosition);
+		    return False;
+		}
+		*(XmTextPosition *)to->addr = tip;
+		to->size = sizeof tip;
+	}
+*/
+	return True;
+}
+
+#define	offset(x)	XtOffsetOf(struct __XmRenditionRec, x)
+
+static XtResource rt_res[] = {
+#if	USE_XFT
+  { XmNrenditionForeground, XmCRenditionForeground, XtRXftColor,
+    sizeof(XftColor), offset(xft_foreground.pixel),
+    XmRImmediate, (XtPointer)XmUNSPECIFIED_PIXEL },
+  { XmNrenditionBackground, XmCRenditionBackground, XtRXftColor,
+    sizeof(XftColor), offset(xft_background.pixel),
+    XmRImmediate, (XtPointer)XmUNSPECIFIED_PIXEL },
+#else
+  { XmNrenditionForeground, XmCRenditionForeground, XmRPixel,
+    sizeof(Pixel), offset(rendition_foreground),
+    XmRImmediate, (XtPointer)XmUNSPECIFIED_PIXEL },
+  { XmNrenditionBackground, XmCRenditionBackground, XmRPixel,
+    sizeof(Pixel), offset(rendition_background),
+    XmRImmediate, (XtPointer)XmUNSPECIFIED_PIXEL },
+#endif
+  { XmNfont, XmCFont, XmRFontList,
+    sizeof(XmFontList), offset(font),
+    XmRImmediate, (XtPointer)XmAS_IS },
+  { XmNfontName, XmCFontName, XmRString,
+    sizeof(String), offset(font_name),
+    XmRImmediate, (XtPointer)XmAS_IS },
+#if	USE_XFT
+  { XmNfontStyle, XmCFontStyle, XmRString,
+    sizeof(String), offset(font_style),
+    XmRImmediate, (XtPointer)NULL },
+  { XmNfontFoundry, XmCFontFoundry, XmRString,
+    sizeof(String), offset(font_foundry),
+    XmRImmediate, (XtPointer)NULL },
+  { XmNfontEncoding, XmCFontEncoding, XmRString,
+    sizeof(String), offset(font_encoding),
+    XmRImmediate, (XtPointer)NULL },
+  { XmNfontSize, XmCFontSize, XmRInt,
+    sizeof(int), offset(font_size),
+    XmRImmediate, (XtPointer)0 },
+#endif
+  { XmNloadModel, XmCLoadModel, XmRUnsignedChar,
+    sizeof(unsigned char), offset(load_model),
+    XmRImmediate, (XtPointer)XmAS_IS },
+  { XmNstrikethruType, XmCStrikethruType, XmRUnsignedChar,
+    sizeof(unsigned char), offset(strike_thru_type),
+    XmRImmediate, (XtPointer)XmAS_IS },
+  { XmNtabList, XmCTabList, XmRTabList,
+    sizeof(XmTabList), offset(tab_list),
+    XmRImmediate, (XtPointer)XmAS_IS },
+  { XmNunderlineType, XmCUnderlineType, XmRUnsignedChar,
+    sizeof(unsigned char), offset(underline_type),
+    XmRImmediate, (XtPointer)XmAS_IS },
+};
+
+/*
+ * Create a rendition from the resources.
+ * Returns true if found.
+ */
+static Boolean
+_XmRenditionFromResources(Display *dpy,
+			Widget w,
+			String name,
+			String cl,
+			XmStringTag tag,
+			XmRendition *rp)
+{
+	XmRendition	r;
+	XrmName		nn[10], *names = &nn[0];
+	XrmClass	cll[10], *classes = &cll[0];
+	XrmHashTable	*searchlist = 0;
+	XrmDatabase	db = NULL;
+	int		len = 8;	/* Means initially 16 */
+	int		i, n = 0, c = 0;
+	Boolean		found = False;
+	XrmRepresentation	tr;
+	XrmValue		v, to;
+
+	DEBUGOUT(_LtDebug(__FILE__, w,
+		"_XmRenditionFromResources(name %s, class %s, tag %s)\n",
+		name, cl, tag));
+
+	/* Initialise names and classes to create a search list from */
+	names[n++] = XrmStringToQuark(name);
+	names[n++] = XrmStringToQuark(tag);
+	names[n++] = NULLQUARK;
+	classes[c++] = XrmStringToQuark(cl);
+	classes[c++] = XrmStringToQuark(XmCRendition);
+	classes[c++] = NULLQUARK;
+
+	db = XtScreenDatabase(XtScreenOfObject(w));
+
+	/* Set up a search list */
+	do {
+		len *= 2;
+		searchlist = (XrmHashTable *)XtRealloc((char *)searchlist,
+			sizeof(XrmHashTable) * len);
+		for (i=0; i<len; i++)
+			searchlist[i]=0;	/* Not necessary */
+	} while (! XrmQGetSearchList(db, names, classes, searchlist, len));
+
+	/* Create a rendition to fill up */
+	r = XmRenditionCreate(w, tag, NULL, 0);
+
+	/* Find resource values and convert them and store them */
+	for (i=0; i<XtNumber(rt_res); i++) {
+		if (XrmQGetSearchResource(searchlist,
+				XrmStringToQuark(rt_res[i].resource_name),
+				XrmStringToQuark(rt_res[i].resource_class),
+				&tr, &v)) {
+			Boolean	success;
+
+			to.addr = ((char *)r) + rt_res[i].resource_offset;
+			to.size = rt_res[i].resource_size;
+
+			success = XtConvertAndStore(w,
+				XmRString, &v,
+				rt_res[i].resource_type, &to);
+			if (!success) {
+				/* Hack FIX ME This cannot be right */
+				if (strcmp(rt_res[i].resource_type, XmRString) == 0) {
+					/* String to string */
+					char	**d = (char **)to.addr;
+					*d = XtNewString(v.addr);
+					found = True;
+					DEBUGOUT(_LtDebug(__FILE__, w,
+						"Convert from %s to %s value %s, %s\n",
+						XmRString,
+						rt_res[i].resource_type,
+						v.addr,
+						"fixed"));
+					continue;
+				}
+				/* Conversion failure */
+				DEBUGOUT(_LtDebug(__FILE__, w,
+					"Convert from %s to %s value %s, %s\n",
+					XmRString,
+					rt_res[i].resource_type,
+					v.addr,
+					success ? "success" : "failed"));
+				continue;
+			} else {
+				DEBUGOUT(_LtDebug(__FILE__, w,
+					"Convert from %s to %s value %s, %s\n",
+					XmRString,
+					rt_res[i].resource_type,
+					v.addr,
+					success ? "success" : "failed"));
+			}
+			
+			found = True;
+
+			DEBUGOUT(_LtDebug(__FILE__, w,
+				"_XmRenditionFromResources(%s,%s) -> %s\n",
+				rt_res[i].resource_name,
+				rt_res[i].resource_class,
+				(char *)v.addr));
+		}
+	}
+#if 0
+	for (i=0; i<XtNumber(rt_res); i++) {
+		if (XrmGetResource(db, rt_res[i].resource_name, rt_res[i].resource_class, st, &v)) {
+			fprintf(stderr, "Found %s (%s)\n", rt_res[i].resource_name, v);
+		}
+	}
+#endif
+	if (found && rp) {
+		*rp = r;
+		return True;
+	}
 	return False;
 }
 
-
 static Boolean
-_XmCvtStringToRenderTable(Display *display,
+_XmCvtStringToRenderTable(Display *dpy,
                           XrmValue *args,
                           Cardinal *num_args,
                           XrmValue *from,
                           XrmValue *to,
                           XtPointer *converter_data)
 {
+	char		*list, *l;
+	XmRendition	r;
+	XmRenderTable	rt = NULL, ort;
+	int		i, count = 0;
+	Boolean		found;
 
-	return False;
+	Widget	w = * (Widget *) args[0].addr;
+
+	DEBUGOUT(_LtDebug(__FILE__, w, "_XmCvtStringToRenderTable\n"));
+
+	/*
+	 *	*One.renderTable: green, variable
+	 *	*One.renderTable.green.renditionForeground: Green
+	 *	*One.renderTable.green.fontName: AS_IS
+	 *	*One.renderTable.variable.underlineType: SINGLE_LINE
+	 *	*One.renderTable.variable.renditionForeground: Red
+	 *
+	 *	_MOTIF_DEFAULT_LOCALE
+	 *
+	 *	If the search of the render table results in no font
+	 *	or fontset, then if there is a rendition in the
+	 *	render table with a tag of _MOTIF_DEFAULT_LOCALE,
+	 *	and if that rendition specifies a font, then that
+	 *	font will be used. If no font or fontset is specified
+	 *	at this point, the text component will not be rendered
+	 *	and a warning message will be displayed.
+	 */
+	/*
+	 * We've got a comma-separated list in 'from',
+	 * split it up and create renditions for it.
+	 */
+	l = list = XtMalloc(from->size + 1);
+	strncpy(list, from->addr, from->size);
+	list[from->size] = '\0';
+	while (1) {
+		char	*p = strchr(l, ',');
+		int	last = 0;
+
+		if (p)
+			*p = '\0';
+		else
+			last = 1;
+		/* We've got one, cut leading spaces */
+		while (isspace(*l))
+			l++;
+
+		found = _XmRenditionFromResources(dpy, w,
+			XmNrenderTable, XmCRenderTable,
+			l, &r);
+		if (found) {
+			ort = rt;
+			rt = XmRenderTableAddRenditions(ort, &r, 1, XmMERGE_REPLACE);
+			XmRenderTableFree(ort);
+			count++;
+		}
+
+		/* Look for more entries */
+		l = p+1;
+		if (last)
+			break;
+	}
+	XtFree(list);
+
+	/*
+	 * Apart from the explicit list above, also scan for the
+	 * _MOTIF_DEFAULT_LOCALE specifications.
+	 */
+	found = _XmRenditionFromResources(dpy, w,
+			XmNrenderTable, XmCRenderTable,
+			"_MOTIF_DEFAULT_LOCALE", &r);
+	if (found) {
+		ort = rt;
+		rt = XmRenderTableAddRenditions(ort, &r, 1, XmMERGE_REPLACE);
+		XmRenderTableFree(ort);
+		count++;
+	}
+
+	/*
+	 * The test below is a silly hack.
+	 */
+	if (!rt)
+		return False;
+#if 0
+/*
+ * Defining this gets us to have both an X Core font and an Xft font.
+ * Unnecessary, time consuming, ..
+ */
+#define	XFT_FALLBACK
+#endif
+
+#if	defined(XFT_FALLBACK) || !defined(USE_XFT)
+	/*
+	 * Even though we want to implement font rendering through Xft,
+	 * we do (currently still) need the font information to be present
+	 * by loading the font.
+	 * This should eventually only be executed when we're not using Xft.
+	 * FIX ME
+	 */
+	for (i=0; i<rt->count; i++) {
+		if (rt->renditions[i]->font_name
+		    && rt->renditions[i]->font == (XFontSet)XmAS_IS) {
+			XFontStruct	*fs = XLoadQueryFont(XtDisplay(w),
+						rt->renditions[i]->font_name);
+			if (fs) {
+				rt->renditions[i]->font = (XFontSet)fs;
+				rt->renditions[i]->type = XmFONT_IS_FONT;
+			} else {
+				fs = XLoadQueryFont(XtDisplay(w), "fixed");
+				if (fs) {
+					rt->renditions[i]->font = (XFontSet)fs;
+					rt->renditions[i]->type = XmFONT_IS_FONT;
+				}
+			}
+			DEBUGOUT(_LtDebug(__FILE__, w, "StringToFont(%s) -> %p\n",
+				rt->renditions[i]->font_name, fs));
+		}
+	}
+#endif
+
+	/*
+	 * Let's also do this for Xft here,
+	 * even though I'm not at all sure that this is the best place.
+	 */
+#ifdef	USE_XFT
+	for (i=0; i<rt->count; i++) {
+		if (rt->renditions[i]->xft_font == 0 &&
+				(rt->renditions[i]->pattern || rt->renditions[i]->font_name)) {
+			XmRendition	r = rt->renditions[i];
+			XftResult	res;
+			XftPattern	*p;
+
+			r->pattern = XftPatternCreate();
+			if (r->font_name)
+				XftPatternAddString(r->pattern, XFT_FAMILY, r->font_name);
+			if (r->font_foundry)
+				XftPatternAddString(r->pattern, XFT_FOUNDRY, r->font_foundry);
+			if (r->font_encoding)
+				XftPatternAddString(r->pattern, XFT_ENCODING, r->font_encoding);
+			if (r->font_style)
+				XftPatternAddString(r->pattern, XFT_STYLE, r->font_style);
+			if (r->font_size)
+				XftPatternAddInteger(r->pattern, XFT_SIZE, r->font_size);
+			if (r->pixel_size)
+				XftPatternAddInteger(r->pattern, XFT_PIXEL_SIZE, r->pixel_size);
+			if (r->font_slant)
+				XftPatternAddInteger(r->pattern, XFT_SLANT, r->font_slant);
+			if (r->font_weight)
+				XftPatternAddInteger(r->pattern, XFT_WEIGHT, r->font_weight);
+			if (r->font_spacing)
+				XftPatternAddInteger(r->pattern, XFT_SPACING, r->font_spacing);
+
+			p = XftFontMatch(XtDisplay(w), 0, r->pattern, &res);
+			r->xft_font = XftFontOpenPattern(XtDisplay(w), p);
+			_XmXftFontAverageWidth(w,
+					(XtPointer)r->xft_font,
+					&r->font_average_width,
+					&r->font_average_height);
+
+			r->type = XmFONT_IS_XFT;
+			r->font = NULL;		/* HACK */
+
+			DEBUGOUT(_LtDebug(__FILE__, w,
+				"Xft Find Font(fn %s) -> font %p\n",
+				r->font_name, r->xft_font));
+		}
+	}
+#endif
+
+	/*
+	 * Do the dance, as in any Xt converter, to return your info.
+	 */
+	if (to->size < sizeof(&rt)) {
+		to->size = sizeof(&rt);
+		return False;
+	}
+	if (to->addr == NULL)
+		to->addr = (XtPointer)&rt;
+	else
+		*(XmRenderTable *)(to->addr) = rt;
+
+	to->addr = (XtPointer)&rt;
+
+	DEBUGOUT(_LtDebug(__FILE__, w,
+		"_XmCvtStringToRenderTable -> %p, %d elements\n", rt, count));
+
+	return True;
 }
 
+#if	USE_XFT
+/*
+ * The functions XmuCvtStringToXftColor() and XmuFreeXftColor() were
+ * "stolen" from the xclock source (xc/programs/xclock/Clock.c).
+ * The source file carried the following license text at the time :
+ *
+ * Copyright 1987, 1988, 1998  The Open Group
+ *
+ * Permission to use, copy, modify, distribute, and sell this software and its
+ * documentation for any purpose is hereby granted without fee, provided that
+ * the above copyright notice appear in all copies and that both that
+ * copyright notice and this permission notice appear in supporting
+ * documentation.
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
+ * OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+ * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ * 
+ * Except as contained in this notice, the name of The Open Group shall not be
+ * used in advertising or otherwise to promote the sale, use or other dealings
+ * in this Software without prior written authorization from The Open Group.
+ * 
+ * 
+ * Copyright 1987, 1988 by Digital Equipment Corporation, Maynard, Massachusetts.
+ * 
+ *                         All Rights Reserved
+ * 
+ * Permission to use, copy, modify, and distribute this software and its
+ * documentation for any purpose and without fee is hereby granted,
+ * provided that the above copyright notice appear in all copies and that
+ * both that copyright notice and this permission notice appear in
+ * supporting documentation, and that the name of Digital not be
+ * used in advertising or publicity pertaining to distribution of the
+ * software without specific, written prior permission.
+ * 
+ * DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
+ * ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
+ * DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
+ * ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
+ * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
+ * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
+ * SOFTWARE.
+ */ 
+
+#define donestr(type, value, tstr) \
+        {                                                       \
+            if (toVal->addr != NULL) {                          \
+                if (toVal->size < sizeof(type)) {               \
+                    toVal->size = sizeof(type);                 \
+                    XtDisplayStringConversionWarning(dpy,       \
+                        (char*) fromVal->addr, tstr);           \
+                    return False;                               \
+                }                                               \
+                *(type*)(toVal->addr) = (value);                \
+            }                                                   \
+            else {                                              \
+                static type static_val;                         \
+                static_val = (value);                           \
+                toVal->addr = (XPointer)&static_val;            \
+            }                                                   \
+            toVal->size = sizeof(type);                         \
+            return True;                                        \
+        }
+
+static Boolean
+XmuCvtStringToXftColor(Display *dpy,
+                       XrmValue *args, Cardinal *num_args,
+                       XrmValue *fromVal, XrmValue *toVal,
+                       XtPointer *converter_data)
+{
+    char            *spec;
+    XRenderColor    renderColor;
+    XftColor        xftColor;
+    Screen          *screen;
+    Colormap        colormap;
+    
+    if (*num_args != 2)
+    {
+        XtAppErrorMsg (XtDisplayToApplicationContext (dpy),
+                       "cvtStringToXftColor", "wrongParameters",
+                       "XtToolkitError",
+                       "String to render color conversion needs screen and colormap arguments",
+                       (String *) NULL, (Cardinal *)NULL);
+        return False;
+    }
+    
+    screen = *((Screen **) args[0].addr);
+    colormap = *((Colormap *) args[1].addr);
+
+    spec = (char *) fromVal->addr; 
+    if (strcasecmp (spec, XtDefaultForeground) == 0)
+    {
+        renderColor.red = 0;
+        renderColor.green = 0;
+        renderColor.blue = 0;
+        renderColor.alpha = 0xffff;
+    }
+    else if (strcasecmp (spec, XtDefaultBackground) == 0)
+    {
+        renderColor.red = 0xffff;
+        renderColor.green = 0xffff;
+        renderColor.blue = 0xffff;
+        renderColor.alpha = 0xffff;
+    }
+    else if (!XRenderParseColor (dpy, spec, &renderColor))
+        return False;
+    if (!XftColorAllocValue (dpy,
+                             DefaultVisual (dpy,
+                                            XScreenNumberOfScreen (screen)),
+                             colormap,
+                             &renderColor,
+                             &xftColor))
+        return False;
+
+    donestr (XftColor, xftColor, XtRXftColor);
+}
+
+static void
+XmuFreeXftColor (XtAppContext app, XrmValuePtr toVal, XtPointer closure,
+                 XrmValuePtr args, Cardinal *num_args)
+{
+    Screen      *screen;
+    Colormap    colormap;
+    XftColor    *color;
+
+    if (*num_args != 2)
+    {
+        XtAppErrorMsg (app,
+                       "freeXftColor", "wrongParameters",
+                       "XtToolkitError",
+                       "Freeing an XftColor requires screen and colormap arguments",
+                       (String *) NULL, (Cardinal *)NULL);
+        return;
+    }
+
+    screen = *((Screen **) args[0].addr);
+    colormap = *((Colormap *) args[1].addr);
+    color = (XftColor *) toVal->addr;
+    XftColorFree (DisplayOfScreen (screen),
+                  DefaultVisual (DisplayOfScreen (screen),
+                                 XScreenNumberOfScreen (screen)),
+                  colormap, color);
+}
+
+
+#endif
 
 extern unsigned int
 XmCvtXmStringToByteStream(XmString string,
