@@ -1,9 +1,9 @@
 /**
  *
- * $Header: /cvsroot/lesstif/lesstif/lib/Xm/SelectionBox.c,v 1.52 2001/05/22 11:39:30 amai Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm/SelectionBox.c,v 1.62 2002/03/30 10:30:06 dannybackx Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
- * Copyright (C) 1995-2001 LessTif Development Team
+ * Copyright  1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm/SelectionBox.c,v 1.52 2001/05/22 11:39:30 amai Exp $";
+static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm/SelectionBox.c,v 1.62 2002/03/30 10:30:06 dannybackx Exp $";
 
 #include <LTconfig.h>
 
@@ -53,41 +53,28 @@ static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm/SelectionB
 /* Forward Declarations */
 
 static void class_initialize(void);
-
 static void class_part_initialize(WidgetClass w_class);
-
 static void initialize(Widget request, Widget new_w,
 		       ArgList args, Cardinal *num_args);
-
 static void destroy(Widget w);
-
 static Boolean set_values(Widget current, Widget request, Widget new_w,
 			  ArgList args, Cardinal *num_args);
-
 static void insert_child(Widget w);
 static void DeleteChild(Widget w);
-
-
-static void
-_XmDialogTypeDefault(Widget w, int offset, XrmValue *value);
-
+static void _XmDialogTypeDefault(Widget w, int offset, XrmValue *value);
 XmGeoMatrix _XmSelectionBoxGeoMatrixCreate(Widget _w, Widget _from,
 					   XtWidgetGeometry *_pref);
-
 Boolean _XmSelectionBoxNoGeoRequest(XmGeoMatrix _geoSpec);
-
 void _XmSelectionBoxGetListItems(Widget wid, int resource_offset,
 				 XtArgVal *value);
-
 void _XmSelectionBoxGetListItemCount(Widget wid, int resource_offset,
 				     XtArgVal *value);
-
 void _XmSelectionBoxGetTextColumns(Widget wid, int resource_offset,
 				   XtArgVal *value);
-
 void _XmSelectionBoxGetTextString(Widget wid, int resource_offset,
 				  XtArgVal *value);
 
+
 static XtAccelerators text_accelerators_parsed = NULL;
 
 /*
@@ -119,7 +106,7 @@ static XtResource resources[] =
     {
 	XmNtextString, XmCTextString, XmRXmString,
 	sizeof(XmString), Offset(text_string),
-	XmRImmediate, (XtPointer)XmUNSPECIFIED /*NULL*/	/* Used to be -1 (XmUNSPECIFIED) */
+	XmRImmediate, (XtPointer)XmUNSPECIFIED
     },
     {
 	XmNlistItems, XmCItems, XmRXmStringTable,
@@ -299,7 +286,7 @@ static XmManagerClassExtRec _XmSelectionBMClassExtRec = {
     /* record_size               */ sizeof(XmManagerClassExtRec),
     /* traversal_children        */ NULL
 };
-#endif
+#endif /* 0 */
 
 XmSelectionBoxClassRec xmSelectionBoxClassRec = {
     /* Core class part */
@@ -410,6 +397,7 @@ class_part_initialize(WidgetClass widget_class)
     _XmFastSubclassInit(widget_class, XmSELECTION_BOX_BIT);
 }
 
+
 /*
  * Determine if the selection is a match with one of the
  * items in the list
@@ -439,6 +427,7 @@ _XmSelectionBoxMatch(XmSelectionBoxWidget w)
     return r;
 }
 
+
 /*
  * When the user clicks on an item in the list, copy its value to the
  * text field.
@@ -457,26 +446,26 @@ ListSingleSelect(Widget w, XtPointer client, XtPointer call)
 
     if (SB_Text(sb) == NULL)
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, w,
+	DEBUGOUT(_LtDebug(__FILE__, w,
 			  "ListSingleSelect (No TextField)\n"));
 	return;
     }
 
     if (lp == NULL || lp->item == NULL)
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, w,
+	DEBUGOUT(_LtDebug(__FILE__, w,
 			  "ListSingleSelect (NULL)\n"));
 	return;
     }
 
     if (!XmStringGetLtoR(lp->item, XmFONTLIST_DEFAULT_TAG, &s))
     {
-	DEBUGOUT(_LtDebugDebug(__FILE__, w,
+	DEBUGOUT(_LtDebug(__FILE__, w,
 			  "ListSingleSelect (Couldn't convert to string)\n"));
 	return;
     }
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w,
+    DEBUGOUT(_LtDebug(__FILE__, w,
 		      "ListSingleSelect '%s'\n", s));
 
     XmTextFieldSetString(SB_Text(sb), s);
@@ -485,6 +474,7 @@ ListSingleSelect(Widget w, XtPointer client, XtPointer call)
     XtFree(s);
 }
 
+
 /*
  * This routine is called for any button child of selectionbox for the
  * XmNactivateCallback. Make mapping to XmNokCallback etc.
@@ -498,7 +488,7 @@ _XmSbButton(Widget w, XtPointer client, XtPointer call)
     XmSelectionBoxCallbackStruct cbs;
     int au = 0;
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmSbButton\n"));
+    DEBUGOUT(_LtDebug(__FILE__, w, "_XmSbButton\n"));
 
     if (XmIsList(w))
     {
@@ -596,13 +586,13 @@ _XmSbButton(Widget w, XtPointer client, XtPointer call)
 	{
 	    XtUnmanageChild((Widget)sb);
 
-	    DEBUGOUT(_LtDebugDebug2(__FILE__, (Widget)sb, w, "AutoUnmanage\n"));
+	    DEBUGOUT(_LtDebug2(__FILE__, (Widget)sb, w, "AutoUnmanage\n"));
 
 	    /* XtNpopdownCallback */
 	    XtCallCallbackList(s, Shell_PopdownCallback(s), NULL);
 	}
     }
-#endif
+#endif /* 0 */
 }
 
 /*
@@ -623,7 +613,7 @@ initialize(Widget request,
 {
     XmSelectionBoxWidget sb = (XmSelectionBoxWidget)new_w;
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebug(__FILE__, new_w,
 		      "initialize: %i args\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
 		      "\t  new_w X %5i Y %5i W %5i H %5i\n",
@@ -634,9 +624,9 @@ initialize(Widget request,
 		      XtWidth(new_w), XtHeight(new_w)));
     DEBUGOUT(_LtDebugPrintArgList(__FILE__, new_w, args, *num_args, False));
 
-
     SB_List(sb) = NULL;
     SB_ListLabel(sb) = NULL;
+    SB_WorkArea(sb) = NULL;
 
     if (SB_DialogType(sb) != XmDIALOG_PROMPT)
     {
@@ -714,6 +704,35 @@ initialize(Widget request,
 	}
 
     }
+
+    if (SB_SelectionLabelString(sb) == (XmString)XmUNSPECIFIED) {
+       SB_SelectionLabelString(sb) = XmStringCreateSimple("");
+    } else {
+       SB_SelectionLabelString(sb) = XmStringCopy(SB_SelectionLabelString(sb));
+    }
+    if (SB_OkLabelString(sb) == (XmString)XmUNSPECIFIED) {
+       SB_OkLabelString(sb) = XmStringCreateSimple("");
+    } else {
+       SB_OkLabelString(sb) = XmStringCopy(SB_OkLabelString(sb));
+    }
+    if (SB_ApplyLabelString(sb) == (XmString)XmUNSPECIFIED) {
+#if 0
+       SB_ApplyLabelString(sb) = XmStringCreateSimple("");
+#endif
+    } else {
+       SB_ApplyLabelString(sb) = XmStringCopy(SB_ApplyLabelString(sb));
+    }
+    if (SB_CancelLabelString(sb) == (XmString)XmUNSPECIFIED) {
+       SB_CancelLabelString(sb) = XmStringCreateSimple("");
+    } else {
+       SB_CancelLabelString(sb) = XmStringCopy(SB_CancelLabelString(sb));
+    }
+    if (SB_HelpLabelString(sb) == (XmString)XmUNSPECIFIED) {
+       SB_HelpLabelString(sb) = XmStringCreateSimple("");
+    } else {
+       SB_HelpLabelString(sb) = XmStringCopy(SB_HelpLabelString(sb));
+    }
+    
     if (SB_List(sb) && SB_ListItems(sb) && SB_ListItemCount(sb) > 0)
     {
     	XtVaSetValues(SB_List(sb),
@@ -730,16 +749,21 @@ initialize(Widget request,
 static void
 destroy(Widget w)
 {
-    DEBUGOUT(_LtDebugDebug("RWS", w,"%s:destroy(%d)\n",
+    XmSelectionBoxWidget sb = (XmSelectionBoxWidget)w;
+
+    DEBUGOUT(_LtDebug("RWS", w,"%s:destroy(%d)\n",
     	__FILE__, __LINE__
     	));
     /* rws 14 Apr 1998
-       Motif does not have this method.  Then again we all know Motif has
-       memory leaks.
+       Motif does not have this method.  Thats because the strings are
+       shared with the child buttons and the buttons manage the storage.
      */
-    XmStringFree(SB_TextString(w));
-	/* T. Straumann: fixed memory leak */
-	XmStringFree(SB_ApplyLabelString(w));
+    if (SB_SelectionLabelString(sb)) XmStringFree(SB_SelectionLabelString(sb));
+    if (SB_TextString(sb))           XmStringFree(SB_TextString(sb));
+    if (SB_OkLabelString(sb))        XmStringFree(SB_OkLabelString(sb));
+    if (SB_ApplyLabelString(sb))     XmStringFree(SB_ApplyLabelString(sb));
+    if (SB_CancelLabelString(sb))    XmStringFree(SB_CancelLabelString(sb));
+    if (SB_HelpLabelString(sb))      XmStringFree(SB_HelpLabelString(sb));
 }
 
 /*
@@ -759,7 +783,7 @@ set_values(Widget old,
     Arg al[10];
     int ac;
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, new_w,
+    DEBUGOUT(_LtDebug(__FILE__, new_w,
 		      "set_values: %i args\n"
 		      "\t    old X %5i Y %5i W %5i H %5i\n"
 		      "\trequest X %5i Y %5i W %5i H %5i\n"
@@ -1090,7 +1114,7 @@ _XmSelectionBoxGetTextColumns(Widget wid, int resource_offset,
     Arg a;
     int n;
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, wid, "XmSelectionBoxGettTextColumns\n"));
+    DEBUGOUT(_LtDebug(__FILE__, wid, "XmSelectionBoxGettTextColumns\n"));
 
     XtSetArg(a, XmNtextColumns, &n);
     n = -1;
@@ -1106,7 +1130,7 @@ _XmSelectionBoxGetTextString(Widget wid, int resource_offset,
 {
     char *str;
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, wid, "_XmSelectionBoxGetTextString\n"));
+    DEBUGOUT(_LtDebug(__FILE__, wid, "_XmSelectionBoxGetTextString\n"));
 
     str = XmTextFieldGetString(SB_Text(wid));
     *value = (XtArgVal)XmStringCreateSimple(str);
@@ -1120,7 +1144,7 @@ _XmSelectionBoxGetListItems(Widget wid, int resource_offset,
     XmStringTable l;
     Arg a;
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, wid,
+    DEBUGOUT(_LtDebug(__FILE__, wid,
 		      "XmSelectionBoxGetListItems(_, %d, _)\n",
 		      resource_offset));
 
@@ -1147,7 +1171,7 @@ _XmSelectionBoxGetListItemCount(Widget wid, int resource_offset,
     Arg a;
     int n;
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, wid, "XmSelectionBoxGetListItemCount\n"));
+    DEBUGOUT(_LtDebug(__FILE__, wid, "XmSelectionBoxGetListItemCount\n"));
 
     XtSetArg(a, XmNitemCount, &n);
     n = -1;
@@ -1200,7 +1224,7 @@ _XmSelectionBoxUpOrDown(Widget wid, XEvent *event,
     int *position_list, position_count, new_position, old_position;
     /* char *selected_text; */
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, wid, "_XmSelectionBoxUpOrDown %s\n",
+    DEBUGOUT(_LtDebug(__FILE__, wid, "_XmSelectionBoxUpOrDown %s\n",
 		      argv ? *argv: "NULL"));
     if (list != NULL && text != NULL)
     {
@@ -1289,7 +1313,7 @@ _XmSelectionBoxRestore(Widget wid, XEvent *event,
     int item_count, *position_list, position_count;
     char *selected_text;
 
-    DEBUGOUT(_LtDebugDebug(__FILE__, wid, "_XmSelectionBoxRestore\n"));
+    DEBUGOUT(_LtDebug(__FILE__, wid, "_XmSelectionBoxRestore\n"));
 
     if (text != NULL && list != NULL)
     {
@@ -1773,6 +1797,7 @@ XmCreatePromptDialog(Widget parent, char *name,
 }
 
 
+/* This routine is marked as obsolete as of Motif 2.1 */
 Widget
 XmSelectionBoxGetChild(Widget w, unsigned char child)
 {
@@ -1780,17 +1805,15 @@ XmSelectionBoxGetChild(Widget w, unsigned char child)
 
     switch (child)
     {
+
     case XmDIALOG_APPLY_BUTTON:
 	return SB_ApplyButton(sb);
 
     case XmDIALOG_CANCEL_BUTTON:
 	return BB_CancelButton(sb);
 
-    case XmDIALOG_OK_BUTTON:
-	return SB_OkButton(sb);
-
-    case XmDIALOG_FILTER_TEXT:
-	return BB_CancelButton(sb);
+    case XmDIALOG_DEFAULT_BUTTON:
+	return BB_DefaultButton(sb);
 
     case XmDIALOG_HELP_BUTTON:
 	return SB_HelpButton(sb);
@@ -1798,43 +1821,31 @@ XmSelectionBoxGetChild(Widget w, unsigned char child)
     case XmDIALOG_LIST:
 	return SB_List(sb);
 
+    case XmDIALOG_LIST_LABEL:
+	return SB_ListLabel(sb);
+	
+    case XmDIALOG_OK_BUTTON:
+	return SB_OkButton(sb);
+
+    case XmDIALOG_SELECTION_LABEL:
+	return SB_SelectionLabel(sb);
+
     case XmDIALOG_SEPARATOR:
 	return SB_Separator(sb);
 
     case XmDIALOG_TEXT:
 	return SB_Text(sb);
 
-    case XmDIALOG_LIST_LABEL:
-	return SB_ListLabel(sb);
+    case XmDIALOG_WORK_AREA:
+	return SB_WorkArea(sb);
 
-    case XmDIALOG_MESSAGE_LABEL:
-	return NULL;
-
-    case XmDIALOG_SELECTION_LABEL:
-	return SB_SelectionLabel(sb);
-
-    case XmDIALOG_SYMBOL_LABEL:
-	return NULL;
-
-    case XmDIALOG_FILTER_LABEL:
-	return NULL;
-
-    case XmDIALOG_DIR_LIST:
-	return NULL;
-
-    case XmDIALOG_DIR_LIST_LABEL:
-	return NULL;
-
-    case XmDIALOG_DEFAULT_BUTTON:
-	return BB_DefaultButton(sb);
-
-	/* These are illegal */
-    case XmDIALOG_NONE:
+    /* These are illegal */
     default:
 	return NULL;
     }
 }
 
+
 /*      
  * Keep track of button children
  */
@@ -1856,6 +1867,8 @@ insert_child(Widget w)
     }
 #endif
 }
+
+
 static void
 DeleteChild(Widget w)
 {
@@ -1885,12 +1898,13 @@ DeleteChild(Widget w)
 		SB_ListLabel(sb) = NULL;
 }
 
+
 static void
 _XmDialogTypeDefault(Widget w, int offset, XrmValue *value)
 {
     static unsigned char dialog_type;
     
-    DEBUGOUT(_LtDebugDebug(__FILE__, w, "_XmDialogTypeDefault\n"));
+    DEBUGOUT(_LtDebug(__FILE__, w, "_XmDialogTypeDefault\n"));
     
     dialog_type = XmDIALOG_WORK_AREA;
     
