@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/lesstif/lesstif/lib/Xm/Text.c,v 1.161 2002/05/11 17:58:04 dannybackx Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm/Text.c,v 1.164 2003/01/22 20:05:10 dannybackx Exp $
  *
  * Copyright (C) 1997 Free Software Foundation, Inc.
  * Copyright  1997, 1998, 1999, 2000, 2001, 2002 LessTif Development Team
@@ -23,9 +23,9 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Text.c,v 1.161 2002/05/11 17:58:04 dannybackx Exp $";
+static const char rcsid[] = "$Id: Text.c,v 1.164 2003/01/22 20:05:10 dannybackx Exp $";
 
-#undef VERBOSE
+#define VERBOSE
 
 #include <LTconfig.h>
 
@@ -1226,34 +1226,6 @@ RefigureLines(XmTextWidget w)
 #endif
 }
 
-
-#if 0
-/* This is not used anywhere. Remove ? FIX ME */
-static LineNum
-LineTableFindNum(XmTextWidget w, XmTextPosition pos)
-{
-    LineNum i;
-
-    if (pos < Text_TopPos(w))
-    {
-	return NOLINE;
-    }
-
-    for (i = 0; i < Text_LineCount(w); i++)
-    {
-	Line line = &Text_Line(w)[i];
-	Line next = &Text_Line(w)[i + 1];
-
-	if (pos >= line->start && pos < next->start)
-	{
-	    return i;
-	}
-    }
-
-    return NOLINE;
-}
-#endif
-
 /* Line-table functions (for the whole textstring)------------------------ */
 
 /* Running test/Xm/test6 (be sure to give it focus so something interesting
@@ -1287,11 +1259,12 @@ _XmTextUpdateLineTable(Widget w,
 		       XmTextBlock block,
 		       Boolean update)
 {
-    XmTextWidget tw = (XmTextWidget)w;
-    XmTextPosition next, ostart;
-    XmTextLineTable line;
-    unsigned int index;
-    Boolean wrap = _XmTextShouldWordWrap(tw), next_is_virtual;
+	Cardinal	i;
+	XmTextWidget	tw = (XmTextWidget)w;
+	XmTextPosition	next, ostart;
+	XmTextLineTable line;
+	unsigned int	index;
+	Boolean		wrap = _XmTextShouldWordWrap(tw), next_is_virtual;
 
     DEBUGOUT(_LtDebug(__FILE__, w, "_XmTextUpdateLineTable start %d end %d\n",
     		start, end));
@@ -1404,7 +1377,7 @@ _XmTextUpdateLineTable(Widget w,
 	{
 	    if (index >= Text_TableSize(w))
 	    {
-		Cardinal i = Text_TableSize(w);
+		i = Text_TableSize(w);
 
 		Text_TableSize(w) += LINETABLE_INCREMENT;
 		Text_LineTable(w) = (XmTextLineTable)
@@ -1486,6 +1459,10 @@ _XmTextUpdateLineTable(Widget w,
 	    line->start_pos = start;
 	    line->virt_line = False;
 
+	    DEBUGOUT(_LtDebug(__FILE__, (Widget)w,
+		       "_XmTextUpdateLineTable nowrap : line %d start %d virtual %d\n",
+				      index, line->start_pos, line->virt_line));
+
 	    /* Be careful not to do PASTENDPOS + 1 */
 	    start = (*Text_Source(w)->Scan) (Text_Source(w), start,
 					  XmSELECT_LINE, XmsdRight, 1, False);
@@ -1507,6 +1484,10 @@ _XmTextUpdateLineTable(Widget w,
      * All done. Store some stuff in the widget.
      */
     Text_TotalLines(w) = index;
+    for (i=index; i<Text_TableSize(w); i++) {
+	Text_LineTable(w)[i].start_pos = 0;
+	Text_LineTable(w)[i].virt_line = False;
+    }
 }
 
 
@@ -3002,10 +2983,14 @@ XmTextScroll(Widget aw, int n)
     }
     else
     {
+	/* bug #672292 */
+	index = _XmMin(n + top_index, Text_TotalLines(w)-1);
+#if 0
 	index = _XmMin(n + top_index,
 /* SG (15/08/1998)  Text_TotalLines(w) - 1) - Text_LineCount(w) + 1;
    we would rather like to be able to scroll down to the last line */
 		       Text_TotalLines(w) ) - Text_LineCount(w) + 1;
+#endif
     }
     DEBUGOUT(_LtDebug(__FILE__, aw, "XmTextScroll index=%d n=%d\n", index, n));
 
