@@ -1,9 +1,9 @@
 /**
  *
- * $Header: /cvsroot/lesstif/lesstif/lib/Xm/TextIn.c,v 1.75 2001/11/26 19:40:24 dannybackx Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm/TextIn.c,v 1.77 2002/07/28 11:51:39 dannybackx Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
- * Copyright  1995, 1996, 1997, 1998, 1999, 2000, 2001 LessTif Development Team
+ * Copyright  1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm/TextIn.c,v 1.75 2001/11/26 19:40:24 dannybackx Exp $";
+static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm/TextIn.c,v 1.77 2002/07/28 11:51:39 dannybackx Exp $";
 
 #include <LTconfig.h>
 
@@ -704,10 +704,11 @@ _XmTextDelete(XmTextWidget w, XEvent *ev, XmTextPosition start, XmTextPosition e
 	     * source.
 	     * Danny 8/5/1997
 	     */
-/*CP:17 May 1999: this does not seem to be needed
+#if 1
+	    /* CP:17 May 1999: this does not seem to be needed */
 	    _XmTextUpdateLineTable((Widget)w, cbs.startPos,
 				   cbs.endPos, &blockrec, True);
-*/
+#endif
 	    _XmTextSetCursorPosition(aw, cbs.newInsert);
 	}
     }
@@ -1587,60 +1588,43 @@ ExtendSelection(Widget w, XEvent *ev, String *params, Cardinal *num_params)
 
     sel = (*Text_Source(w)->GetSelection) (Text_Source(w), &left, &right);
 
-    if (sel)
-    {
+    if (sel) {
 	DEBUGOUT(_LtDebug(__FILE__, w,
-		 "ExtendSelection (pos %d, origleft %d) existing sel. %d-%d\n",
-			  pos, In_HighlightPivot(i), left, right));
-	if ( pos >= left && pos <= right )
-	     /*CP: no need to do any more processing since it was only a mouse move
-		which does not change the selection positions */
-	    return ;
-    }
-    else
-    {
+		"ExtendSelection (pos %d, origleft %d) existing sel. %d-%d\n",
+		pos, In_HighlightPivot(i), left, right));
+    } else {
 	DEBUGOUT(_LtDebug(__FILE__, w,
-	    "ExtendSelection (pos %d, origleft %d) (x %d y %d)\n",
-	    pos, In_HighlightPivot(i), ev->xbutton.x, ev->xbutton.y));
+		"ExtendSelection (pos %d, origleft %d) (x %d y %d)\n",
+		pos, In_HighlightPivot(i), ev->xbutton.x, ev->xbutton.y));
     }
 
-    if (In_HighlightPivot(i) < 0)
-    {
+    if (In_HighlightPivot(i) < 0) {
 	In_HighlightPivot(i) = pos;
     }
 
     st = ScanType((XmTextWidget)w);
 
-    if(pos >= In_HighlightPivot(i))
-    {
-        if(st == XmSELECT_POSITION)
-        {
-            left = In_HighlightPivot(i);
-            right = pos;
-        }
-        else
-        {
-            right = (*Text_Source(w)->Scan)(Text_Source(w), pos, st,
-                                            XmsdRight, 1, False);
-            left = In_HighlightPivot(i);
-        }
-        cursor_pos = right;
-    }
-    else
-    {
-        if(st == XmSELECT_POSITION)
-        {
-            left = pos;
-            right = In_HighlightPivot(i);
-        }
-        else
-        {
-            left = (*Text_Source(w)->Scan)(Text_Source(w), pos, st,
-                                           XmsdLeft, 1, False);
-            right = (*Text_Source(w)->Scan)(Text_Source(w),In_HighlightPivot(i),
-                                            st, XmsdRight, 1, False);
-        }
-        cursor_pos = left;
+    if (pos >= In_HighlightPivot(i)) {
+	if (st == XmSELECT_POSITION) {
+		left = In_HighlightPivot(i);
+		right = pos;
+	} else {
+		right = (*Text_Source(w)->Scan)(Text_Source(w), pos, st,
+			XmsdRight, 1, False);
+		left = In_HighlightPivot(i);
+	}
+	cursor_pos = right;
+    } else {
+	if (st == XmSELECT_POSITION) {
+		left = pos;
+		right = In_HighlightPivot(i);
+	} else {
+		left = (*Text_Source(w)->Scan)(Text_Source(w), pos, st,
+			XmsdLeft, 1, False);
+		right = (*Text_Source(w)->Scan)(Text_Source(w),
+			In_HighlightPivot(i), st, XmsdRight, 1, False);
+	}
+	cursor_pos = left;
     }
 
     XmTextSetSelection(w,left,right,ev->xbutton.time);
@@ -2853,7 +2837,7 @@ TextFocusIn(Widget w, XEvent *ev, String *params, Cardinal *num_params)
     OutputData o = Text_OutputData(w);
 #endif
 
-    DEBUGOUT(_LtDebug(__FILE__, w, "TextFocusIn"));
+    DEBUGOUT(_LtDebug(__FILE__, w, "TextFocusIn\n"));
 
     if (Text_Editable(w))
     {
