@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/lesstif/lesstif/lib/Xm/TextOut.c,v 1.111 2002/05/12 00:44:13 amai Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm/TextOut.c,v 1.123 2002/11/15 17:00:54 dannybackx Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  * Copyright  1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002 LessTif Development Team
@@ -23,11 +23,22 @@
  *
  **/
 
-static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm/TextOut.c,v 1.111 2002/05/12 00:44:13 amai Exp $";
+static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm/TextOut.c,v 1.123 2002/11/15 17:00:54 dannybackx Exp $";
 
-/* Use this to use XtAllocateGC/XtGetGC, otherwise XCreateGC */
-/* Similar for the free operations */
-#define	USE_XT_GC	1
+/* Do NOT use this (USE_XT_GC) :
+ * it requires setting all the dynamic GC fields before every use of
+ * the GC. As we have too many GC fields that can change, this is
+ * really a bad idea.
+ * Setting this macro leads to use of XtAllocateGC and XtGetGC,
+ * and similar for the free operations.
+ * This path has been explored more than once, by different
+ * developers. Please keep the comments here so it gets
+ * remembered.
+ */
+#undef	USE_XT_GC
+/*
+ * End of the USE_XT_GC blurb.
+ */
 
 #include <LTconfig.h>
 
@@ -66,6 +77,7 @@ static void Draw(XmTextWidget w, LineNum num, XmTextPosition start, XmTextPositi
 #define	Out_FontTextWidth(o,s,l)	(int)_XmOut_FontTextWidth(o, s, l)
 #define	Out_FontMaxWidth(o)	(int)_XmOut_FontMaxWidth(o)
 
+static void _XmTextOutSetRenderTable(Widget w, int o, XrmValue *v);
 	
 /* Resources for the TextInner class */
 static XtResource output_resources[] =
@@ -85,11 +97,30 @@ static XtResource output_resources[] =
 	sizeof(Boolean), Offset(cursor_position_visible),
 	XmRImmediate, (XtPointer)True
     },
+#if (XmVERSION > 1) && defined(USE_XFT)
+    /* Note that the right order is important here ! */
+    {
+	"keep.off", "Keep.off", XmRBoolean,
+	sizeof(Boolean), Offset(check_set_render_table),
+	XmRImmediate, (XtPointer)NULL
+    },
+    {
+	XmNfontList, XmCFontList, XmRFontList,
+	sizeof(XmFontList), Offset(fontlist),
+	XmRCallProc, (XtPointer)_XmTextOutSetRenderTable
+    },
+    {
+	XmNrenderTable, XmCRenderTable, XmRRenderTable,
+	sizeof(XmFontList), Offset(fontlist),
+	XmRCallProc, (XtPointer)_XmTextOutSetRenderTable
+    },
+#else
     {
 	XmNfontList, XmCFontList, XmRFontList,
 	sizeof(XmFontList), Offset(fontlist),
 	XmRImmediate, (XtPointer)XmUNSPECIFIED
     },
+#endif
     {
 	XmNresizeHeight, XmCResizeHeight, XmRBoolean,
 	sizeof(Boolean), Offset(resizeheight),
@@ -141,26 +172,29 @@ extern int
 _XmFontCalculateAverageCharacterWidth(Widget w, XFontStruct *fs)
 {
 	/*CP: The following is more or less what the XFree86 people have suggested. */
-    unsigned long width;
-    Atom atom;
+	unsigned long width;
+	Atom atom;
+
+	if (fs == NULL) {
+		/* Especially important in Xft case */
+		return 0;
+	}
 
 	/* this is copied from a similar function in Screen.c */
-    if ( (atom = XmInternAtom(XtDisplay(w), _XA_AVERAGE_WIDTH, True) )
-	   && XGetFontProperty(fs, atom, &width) && width )
-	{
-	width =  (width + 5) / 10; /*CP: just to round it up slightly */
-	}
-    else if ( ( ! XGetFontProperty( fs, XA_QUAD_WIDTH, &width)) || width == 0)
-	{
+	if ((atom = XmInternAtom(XtDisplay(w), _XA_AVERAGE_WIDTH, True))
+			&& XGetFontProperty(fs, atom, &width) && width) {
+		width = (width + 5) / 10; /*CP: just to round it up slightly */
+	} else if ((! XGetFontProperty(fs, XA_QUAD_WIDTH, &width)) || width == 0) {
 		/*CP: Yes, even if the property is available it could be 0 - as per XF86 people */
 		/*So we will will use the character "x" */
-	if ( fs->per_char &&  fs->min_char_or_byte2 <= 'x' &&
-			fs->max_char_or_byte2 >= 'x') /* i.e. we have data for x */
-	    width = fs->per_char['a' - fs->min_char_or_byte2].width;
-	else
-	    width = fs->max_bounds.width / 2 ;                                          
+		if ( fs->per_char &&  fs->min_char_or_byte2 <= 'x' &&
+				fs->max_char_or_byte2 >= 'x')
+				/* i.e. we have data for x */
+			width = fs->per_char['a' - fs->min_char_or_byte2].width;
+		else
+			width = fs->max_bounds.width / 2 ;                                          
 	}
-    return width;
+	return width;
 }
 
 
@@ -233,78 +267,107 @@ FontInit(XmTextWidget w)
 	}
 	/* (Hopefully) end of the all wrong part */
 
-    /* This should only be the very very last resort. */
-    if (entry == NULL || entry->font == NULL) {
-	Out_FontList(o) = _XmFontListCreateDefault(XtDisplay((Widget)w));
-	entry = Out_FontList(o)->renditions[0];
-    } else {	/* Make copy. Original will be free */
-      /*  amai, 20010313:
-       *  I do not understand why this is necessary, but
-       *  it seems to be ...
-       *  Why does copying the list affect behaviour?!
-       *  Try to omit the copying step and run test/Xm/text/test17!
-       *  The original will _not_ be freed, or at least it won't be
-       *  always freed, see test/Xm/text/test11, the leak check!
-       */
-	Out_FontList(o) = XmFontListAppendEntry(NULL, entry);
-	if (Out_FontListCreated(o))
-		XmFontListFree(oldlist);
-	entry = Out_FontList(o)->renditions[0];
-	Out_FontListCreated(o) = True;
-    }
+#if	USE_XFT
+	Out_FontType(o) = XmFONT_IS_FONT;
+	o->xft_font = NULL;
+	
+	/*
+	 * We need a hack to get Xft fonts accepted.
+	 * The current (initial hack) is to use Xft for fonts if
+	 * the first item in the fontList is an Xft font.
+	 */
+	if (Out_FontList(o) && Out_FontList(o)->renditions[0]
+			&& Out_FontList(o)->renditions[0]->type == XmFONT_IS_XFT) {
+		Out_FontType(o) = XmFONT_IS_XFT;
+		entry = Out_FontList(o)->renditions[0];
+		o->xft_font = entry->xft_font;
+		Out_Font(o) = NULL;
+	}
+#endif
 
-    /* We may get a font or a fontset here */
-    if (entry->type == XmFONT_IS_FONT)
-    {
-	fs = (XFontStruct *)entry->font;
-
-        Out_FontAverageWidth(o) =
-	    (fs->max_bounds.width + fs->min_bounds.width) / 2;
-	Out_UseFontSet(o) = False;
-    }
-    else
-    {				/* XmFONT_IS_FONTSET */
-	XFontStruct **fsl;
-	char **nl;
-	int r, i, max, min;
-
-	r = XFontsOfFontSet((XFontSet)entry->font, &fsl, &nl);
-	DEBUGOUT(_LtDebug(__FILE__, (Widget)w, "XFontsOfFontSet : %d\n", r));
-	for (i = 0; i < r; i++)
+	/* This should only be the very very last resort. */
+#ifdef	USE_XFT
+	if (entry == NULL || (entry->font == NULL && entry->type != XmFONT_IS_XFT))
+#else
+	if (entry == NULL || entry->font == NULL)
+#endif
 	{
-	    DEBUGOUT(_LtDebug0(__FILE__, NULL, "\t%s\n", nl[i]));
+		Out_FontList(o) = _XmFontListCreateDefault(XtDisplay((Widget)w));
+		entry = Out_FontList(o)->renditions[0];
+	} else {	/* Make copy. Original will be free */
+		/*  amai, 20010313:
+		 *  I do not understand why this is necessary, but
+		 *  it seems to be ...
+		 *  Why does copying the list affect behaviour?!
+		 *  Try to omit the copying step and run test/Xm/text/test17!
+		 *  The original will _not_ be freed, or at least it won't be
+		 *  always freed, see test/Xm/text/test11, the leak check!
+		 */
+		Out_FontList(o) = XmFontListAppendEntry(NULL, entry);
+		if (Out_FontListCreated(o))
+			XmFontListFree(oldlist);
+		entry = Out_FontList(o)->renditions[0];
+		Out_FontListCreated(o) = True;
 	}
 
-	/* FIX ME - just pick the first one */
-	if (r == 0)
+	/* We may get a font or a fontset here */
+	if (entry->type == XmFONT_IS_FONT)
 	{
-	    DEBUGOUT(_LtDebug(__FILE__, NULL, "No font in fontset\n"));
+		fs = (XFontStruct *)entry->font;
 
-	    abort();
+		Out_FontAverageWidth(o) =
+			(fs->max_bounds.width + fs->min_bounds.width) / 2;
+		Out_UseFontSet(o) = False;
+		Out_Font(o) = fs;
+#ifdef	USE_XFT
+	} else if (entry->type == XmFONT_IS_XFT) {
+		Out_FontAverageWidth(o) = Out_XftFont(o)->max_advance_width;
+		Out_FontAverageWidth(o) = entry->font_average_width;	/* FIX ME HACK */
+		Out_UseFontSet(o) = False;
+#endif
+	} else {			/* XmFONT_IS_FONTSET */
+		XFontStruct **fsl;
+		char **nl;
+		int r, i, max, min;
+
+		r = XFontsOfFontSet((XFontSet)entry->font, &fsl, &nl);
+		DEBUGOUT(_LtDebug(__FILE__, (Widget)w, "XFontsOfFontSet : %d\n", r));
+		for (i = 0; i < r; i++) {
+			DEBUGOUT(_LtDebug0(__FILE__, NULL, "\t%s\n", nl[i]));
+		}
+
+		/* FIX ME - just pick the first one */
+		if (r == 0) {
+			DEBUGOUT(_LtDebug(__FILE__, NULL, "No font in fontset\n"));
+			abort();
+		}
+
+		fs = fsl[0];
+		/* correct calculation */
+		max = 0; min = INT_MAX;
+		for(i=0; i<r; i++) {
+			if (fsl[i]->max_bounds.width  > max)
+				max = fsl[i]->max_bounds.width;
+			if (fsl[i]->min_bounds.width  < min)
+				min = fsl[i]->min_bounds.width;
+		}
+		Out_FontAverageWidth(o) = (max + min) / 2;
+		Out_Font(o) = fs;
+
+	/*	Out_UseFontSet(o) = True;	*/
 	}
 
-	fs = fsl[0];
-        /* correct calculation */
-        max = 0; min = INT_MAX;
-        for(i=0; i<r; i++){
-            if (fsl[i]->max_bounds.width  > max)
-                    max = fsl[i]->max_bounds.width;
-            if (fsl[i]->min_bounds.width  < min)
-                    min = fsl[i]->min_bounds.width;
-        }
-        Out_FontAverageWidth(o) = (max + min) / 2;
-
-/*	Out_UseFontSet(o) = True;
-*/    }
-
-	Out_Font(o) = fs;
-
 	if ( Out_FontAverageWidth(o) <= 0 ) {
 		Out_FontAverageWidth(o) = _XmFontCalculateAverageCharacterWidth((Widget)w, fs);
 	}
 	Out_TabWidth(o) = 8 * Out_FontAverageWidth(o);
+#if 0
+	/*
+	 * These fields are never used.
+	 */
 	Out_Font_Ascent(o) = Out_FontAscent(o);
 	Out_Font_Descent(o) = Out_FontDescent(o);
+#endif
 	Out_FontHeight(o) = Out_FontAscent(o) + Out_FontDescent(o);
 	Out_CursorHeight(o) = Out_FontHeight(o);
 
@@ -317,12 +380,13 @@ FontInit(XmTextWidget w)
 
 
 /*
- * FIX ME
- *
  * This thing still creates GC's the anti-social way - with XCreateGC.
+ *
  * By using XtGetGC or XtAllocateGC, Xt will allow us to share GC's
  * with other widgets. We'd just have to be sure about what we want to be
  * the properties of the thing.
+ *
+ * Note: the explanation for this is in comments at the top of this file.
  */
 static void
 GCInitialize(XmTextWidget w)
@@ -333,20 +397,34 @@ GCInitialize(XmTextWidget w)
 
     /*
      * DrawGC
-     *      Need to change :        ClipRectangles
+     *      Need to change :	ClipRectangles
+     *				GCFillStyle | GCStipple
      */
-    values.line_style = LineSolid;
-    values.line_width = 0;
-    values.fill_style = FillSolid;
-    values.font = Out_Font(o)->fid;
+	values.line_style = LineSolid;
+	values.line_width = 0;
+	values.fill_style = FillSolid;
+
+	mask = GCLineStyle | GCLineWidth | GCForeground | GCBackground
+		| GCClipXOrigin | GCClipYOrigin;
+	if (Out_FontType(o) == XmFONT_IS_FONT) {
+		values.font = Out_Font(o)->fid;
+		mask |= GCFont;
+	} else
+#ifdef	USE_XFT
+	if (Out_FontType(o) == XmFONT_IS_XFT) {
+		/* FIX ME XmFONT_IS_XFT */
+	}
+#endif
+#ifdef	USE_BIDI
+	else if (Out_FontType(o) == XmFONT_IS_XOC) {
+		/* FIX ME XmFONT_IS_BIDI */
+	}
+#endif
     values.foreground = Prim_Foreground(w);
     values.background = XtBackground(w);
     values.clip_x_origin = 0;
     values.clip_y_origin = 0;
-    mask = GCLineStyle | GCLineWidth | GCFillStyle | GCFont |
-	GCForeground | GCBackground;
-    dynamic = GCClipMask | GCClipXOrigin | GCClipYOrigin |
-	GCForeground | GCBackground;
+    dynamic = 0;
     dontcare = 0;
 
     if (! w->core.sensitive) {
@@ -406,86 +484,92 @@ GCInitialize(XmTextWidget w)
 static void
 GCPixmap(XmTextWidget tw)
 {
-    OutputData o = Text_OutputData(tw);
-    Widget w = (Widget)tw;
-    static char dots[] = {2, 1, 1};
-    GC gc;
-    int width, width_erase, height;
-    Display *display;
-    XGCValues values;
-    XtGCMask mask;
+	OutputData o = Text_OutputData(tw);
+	Widget w = (Widget)tw;
+	static char dots[] = {2, 1, 1};
+	GC gc;
+	int width, width_erase, height;
+	Display *display;
+	XGCValues values;
+	XtGCMask mask;
 
-    values.line_style = LineSolid;
-    values.line_width = 0;
-    values.fill_style = FillSolid;
-    values.foreground = 0;
-    values.background = 0;
-
-    display = XtDisplay(w);
-    width = Out_CursorWidth(o) ;
-    height = Out_CursorHeight(o) = Out_FontHeight(o);
-    width_erase = 2 * Out_Font(o)->max_bounds.width;
-
-    if (Out_CursorIBeam(o))
-    {
-	XFreePixmap(XtDisplay(w), Out_CursorIBeam(o));
-    }
-    Out_CursorIBeam(o) = (Pixmap)NULL;
-
-    if (Out_CursorStipple(o))
-    {
-	XFreePixmap(XtDisplay(w), Out_CursorStipple(o));
-    }
-    Out_CursorStipple(o) = (Pixmap)NULL;
-
-    if (Out_CursorSave(o))
-    {
-	XFreePixmap(XtDisplay(w), Out_CursorSave(o));
-    }
-    Out_CursorSave(o) = (Pixmap)NULL;
-
-    Out_CursorSaveValid(o) = False;
-
-    if (height > 0)
-    {
-	Out_CursorIBeam(o) = XCreatePixmap(display,
-					   RootWindowOfScreen(XtScreen(w)),
-					   width, height, 1);
-	Out_CursorStipple(o) = XCreatePixmap(display,
-					     RootWindowOfScreen(XtScreen(w)),
-					     width, height, 1);
-	Out_CursorSave(o) = XCreatePixmap(display,
-					  RootWindowOfScreen(XtScreen(w)),
-					  width_erase, height, w->core.depth);
 	values.line_style = LineSolid;
 	values.line_width = 0;
 	values.fill_style = FillSolid;
 	values.foreground = 0;
 	values.background = 0;
-	mask = GCLineStyle | GCLineWidth | GCFillStyle |
-	    GCForeground | GCBackground;
-	gc = XCreateGC(display, Out_CursorIBeam(o), mask, &values);
 
-	XFillRectangle(display, Out_CursorIBeam(o), gc, 0, 0, width, height);
-	XFillRectangle(display, Out_CursorStipple(o), gc, 0, 0, width, height);
+	display = XtDisplay(w);
+	width = Out_CursorWidth(o) ;
 
-	XSetForeground(display, gc, 1);
+	if (Out_FontType(o) == XmFONT_IS_FONT) {
+		width_erase = 2 * Out_Font(o)->max_bounds.width;
+		height = Out_CursorHeight(o) = Out_FontHeight(o);
+	} else
+#ifdef	USE_XFT
+	if (Out_FontType(o) == XmFONT_IS_XFT) {
+		width_erase = 2 * ((XftFont *)o->xft_font)->max_advance_width;
+		width_erase = 2 * Out_FontAverageWidth(o);	/* FIX ME HACK */
+		height = Out_CursorHeight(o) = Out_XftFont(o)->height;
+	}
+#endif
 
-	XDrawLine(display, Out_CursorIBeam(o), gc, 2, 1, 2, height - 2);
-	XDrawLine(display, Out_CursorIBeam(o), gc, 0, 0, 4, 0);
-	XDrawLine(display, Out_CursorIBeam(o), gc, 0, height - 1,
-		  4, height - 1);
+	if (Out_CursorIBeam(o)) {
+		XFreePixmap(XtDisplay(w), Out_CursorIBeam(o));
+	}
+	Out_CursorIBeam(o) = (Pixmap)NULL;
 
-	XSetLineAttributes(display, gc, 0, LineOnOffDash, CapRound, JoinRound);
-	XSetDashes(display, gc, 0, &dots[1], (int)dots[0]);
+	if (Out_CursorStipple(o)) {
+		XFreePixmap(XtDisplay(w), Out_CursorStipple(o));
+	}
+	Out_CursorStipple(o) = (Pixmap)NULL;
 
-	XDrawLine(display, Out_CursorStipple(o), gc, 2, 1, 2, height - 2);
-	XDrawLine(display, Out_CursorStipple(o), gc, 1, 0, 3, 0);
-	XDrawLine(display, Out_CursorStipple(o), gc, 1, height - 1,
-		  3, height - 1);
+	if (Out_CursorSave(o)) {
+		XFreePixmap(XtDisplay(w), Out_CursorSave(o));
+	}
+	Out_CursorSave(o) = (Pixmap)NULL;
 
-	XFreeGC(display, gc);
-    }
+	Out_CursorSaveValid(o) = False;
+
+	if (height > 0) {
+		Out_CursorIBeam(o) = XCreatePixmap(display,
+			RootWindowOfScreen(XtScreen(w)),
+			width, height, 1);
+		Out_CursorStipple(o) = XCreatePixmap(display,
+			RootWindowOfScreen(XtScreen(w)),
+			width, height, 1);
+		Out_CursorSave(o) = XCreatePixmap(display,
+			RootWindowOfScreen(XtScreen(w)),
+			width_erase, height, w->core.depth);
+		values.line_style = LineSolid;
+		values.line_width = 0;
+		values.fill_style = FillSolid;
+		values.foreground = 0;
+		values.background = 0;
+		mask = GCLineStyle | GCLineWidth | GCFillStyle |
+			GCForeground | GCBackground;
+		gc = XCreateGC(display, Out_CursorIBeam(o), mask, &values);
+
+		XFillRectangle(display, Out_CursorIBeam(o), gc, 0, 0, width, height);
+		XFillRectangle(display, Out_CursorStipple(o), gc, 0, 0, width, height);
+
+		XSetForeground(display, gc, 1);
+
+		XDrawLine(display, Out_CursorIBeam(o), gc, 2, 1, 2, height - 2);
+		XDrawLine(display, Out_CursorIBeam(o), gc, 0, 0, 4, 0);
+		XDrawLine(display, Out_CursorIBeam(o), gc, 0, height - 1,
+			4, height - 1);
+
+		XSetLineAttributes(display, gc, 0, LineOnOffDash, CapRound, JoinRound);
+		XSetDashes(display, gc, 0, &dots[1], (int)dots[0]);
+
+		XDrawLine(display, Out_CursorStipple(o), gc, 2, 1, 2, height - 2);
+		XDrawLine(display, Out_CursorStipple(o), gc, 1, 0, 3, 0);
+		XDrawLine(display, Out_CursorStipple(o), gc, 1, height - 1,
+			3, height - 1);
+
+		XFreeGC(display, gc);
+	}
 }
 
 
@@ -516,58 +600,76 @@ GCClip(XmTextWidget w)
 static void
 SizeRecalc(XmTextWidget w)
 {
-    OutputData o = Text_OutputData(w);
-    int previousColumns = Out_Columns(o);
+	OutputData o = Text_OutputData(w);
+	int previousColumns = Out_Columns(o);
 
-    Out_XDraw(o) = Out_LeftMargin(o);
-
-    /* Y offset for a row R is calculated as Out_YDraw(o) - R*fontHeight,
-     * and I add FontAscent here because y positions for fonts are
-     * measured from the baseline of the font. */
-    Out_YDraw(o) = Out_TopMargin(o) + Out_FontAscent(o);
+	Out_XDraw(o) = Out_LeftMargin(o);
 
+#ifdef	USE_XFT
+	/* HACK */
+	if (Out_XftFont(o) && Out_XftFont(o)->height < 0)
+		Out_XftFont(o)->height = Out_XftFont(o)->ascent + Out_XftFont(o)->descent;
+	if (Out_XftFont(o)) {
+			Out_Columns(o) = (XtWidth(w) - (Out_LeftMargin(o) + Out_RightMargin(o)))
+				/ Out_XftFont(o)->max_advance_width;
+			Out_Columns(o) = (XtWidth(w) - (Out_LeftMargin(o) + Out_RightMargin(o)))
+				/ Out_FontAverageWidth(o);	/* FIX ME HACK */
+			DEBUGOUT(_LtDebug(__FILE__, (Widget)w,
+				"SizeRecalc: xft cols %d = (wid %d - lm %d - rm %d) / fw %d, "
+				"ht %d = %d + %d\n",
+				Out_Columns(o), XtWidth(w), Out_LeftMargin(o), Out_RightMargin(o),
+				Out_XftFont(o)->max_advance_width,
+				Out_XftFont(o)->height,
+				Out_XftFont(o)->ascent, Out_XftFont(o)->descent));
+	} else 
+#endif
+	{
 #ifdef USE_AVERAGE_WIDTH
-    Out_Columns(o) = (XtWidth(w) - ( Out_LeftMargin(o) + Out_RightMargin(o) ) )
+		Out_Columns(o) = (XtWidth(w) - ( Out_LeftMargin(o) + Out_RightMargin(o) ) )
 			/ Out_FontAverageWidth(o);
+		DEBUGOUT(_LtDebug(__FILE__, (Widget)w,
+			"SizeRecalc: cols %d = (wid %d - lm %d - rm %d) / fw %d\n",
+			Out_Columns(o), XtWidth(w), Out_LeftMargin(o), Out_RightMargin(o),
+			Out_FontAverageWidth(o)));
 #else
-    Out_Columns(o) = (XtWidth(w) - ( Out_LeftMargin(o) + Out_RightMargin(o) ) )
+		Out_Columns(o) = (XtWidth(w) - ( Out_LeftMargin(o) + Out_RightMargin(o) ) )
 			/ Out_FontMaxWidth(o);
+		DEBUGOUT(_LtDebug(__FILE__, (Widget)w,
+			"SizeRecalc: cols %d = (wid %d - lm %d - rm %d) / fw %d\n",
+			Out_Columns(o), XtWidth(w), Out_LeftMargin(o), Out_RightMargin(o),
+			Out_FontMaxWidth(o)));
 #endif
+	}
 
-    if (Text_EditMode(w) == XmSINGLE_LINE_EDIT)
-    {
-	Out_Rows(o) = 1;
-    }
-    else
-    {
-	Out_Rows(o) = (XtHeight(w)
-		- ( Out_TopMargin(o) + Out_BottomMargin(o) ) )
-		/ Out_FontHeight(o);
-	if ( Out_Rows(o) < 1 )
-	{
+	/* Y offset for a row R is calculated as Out_YDraw(o) - R*fontHeight,
+	 * and I add FontAscent here because y positions for fonts are
+	 * measured from the baseline of the font. */
+	Out_YDraw(o) = Out_TopMargin(o) + Out_FontAscent(o);
+
+	if (Text_EditMode(w) == XmSINGLE_LINE_EDIT) {
 		Out_Rows(o) = 1;
+	} else {
+		Out_Rows(o) = (XtHeight(w)
+			- ( Out_TopMargin(o) + Out_BottomMargin(o) ) )
+			/ Out_FontHeight(o);
+		if ( Out_Rows(o) < 1 ) {
+			Out_Rows(o) = 1;
+		}
 	}
-    }
-    if ( XtHeight(w) < ( Out_TopMargin(o) + Out_BottomMargin(o) + Out_FontHeight(o) * Out_Rows(o) ) )
-    {
-	XtHeight(w) = Out_TopMargin(o) + Out_BottomMargin(o) + Out_FontHeight(o) * Out_Rows(o) ;
-    }
-
-    Out_NumberLines(o) = Out_Rows(o);
-    if ( Out_Columns(o) != previousColumns && Out_WordWrap(o) )
-	{
-        _XmTextUpdateLineTable((Widget)w, 0, 0, NULL, False);
+	if (XtHeight(w) < (Out_TopMargin(o) + Out_BottomMargin(o)
+				+ Out_FontHeight(o) * Out_Rows(o))) {
+		XtHeight(w) = Out_TopMargin(o) + Out_BottomMargin(o)
+			+ Out_FontHeight(o) * Out_Rows(o);
 	}
 
+	Out_NumberLines(o) = Out_Rows(o);
+	if (Out_Columns(o) != previousColumns && Out_WordWrap(o)) {
+		_XmTextUpdateLineTable((Widget)w, 0, 0, NULL, False);
+	}
 
-#ifdef	DANNY
-    fprintf(stderr, "XmText SizeRecalc wid %d ht %d -> rows %d cols %d\n",
-		      XtWidth(w), XtHeight(w), Out_Rows(o), Out_Columns(o));
-#endif
-
-    DEBUGOUT(_LtDebug(__FILE__, (Widget)w,
-		      "SizeRecalc wid %d ht %d -> rows %d cols %d\n",
-		      XtWidth(w), XtHeight(w), Out_Rows(o), Out_Columns(o)));
+	DEBUGOUT(_LtDebug(__FILE__, (Widget)w,
+		"SizeRecalc wid %d ht %d -> rows %d cols %d\n",
+		XtWidth(w), XtHeight(w), Out_Rows(o), Out_Columns(o)));
 }
 
 
@@ -796,28 +898,24 @@ _XmTextNextX(XmTextWidget w, int x, char *ptr, int len)
 	unsigned int c;
 
 	c = (unsigned char)*ptr;
-	if (c == '\t')
-	{
+	if (c == '\t') {
 	    x = Out_NextTabStop(o, x);
-	}
-	else if (c == '\0')
-	{
+	} else if (c == '\0') {
 	    x = 0;
-	}
-	else
-	{
-
-	    if (Out_FontList(o)->renditions[0]->type == XmFONT_IS_FONT)
-	    {
+	} else {
+	    if (Out_FontList(o)->renditions[0]->type == XmFONT_IS_FONT) {
                 x += XTextWidth(fs, ptr, len);
-            }
-	    else if (Out_FontList(o)->renditions[0]->type == XmFONT_IS_FONTSET)
-	    {
+            } else if (Out_FontList(o)->renditions[0]->type == XmFONT_IS_FONTSET) {
                 XmbTextExtents((XFontSet)Out_FontList(o)->renditions[0]->font, ptr, len, &ink, &log);
                 x += log.width;
-            }
-	    else
-	    {
+#ifdef	USE_XFT
+            } else if (Out_XftFont(o)) {
+	    	XGlyphInfo	ext;
+		XftTextExtents8(XtDisplay((Widget)w), Out_XftFont(o),
+			ptr, len, &ext);
+		x += ext.width;
+#endif
+	    } else {
 		/* ??? */
 	    }
 
@@ -1010,74 +1108,97 @@ YToLineNum(XmTextWidget w, Position y)
 
 
 static void
-DrawText(XmTextWidget w, int x, int y, char *ptr, int len,
-	 XmHighlightMode highlight)
+DrawText(XmTextWidget w, int x, int y, char *ptr, int len, XmHighlightMode highlight)
 {
-    OutputData o = Text_OutputData(w);
-    int xoff, xstart = x, xend;
+	OutputData o = Text_OutputData(w);
+	int	xoff, xstart = x, xend;
+	Pixel	fg, bg;
 
 #ifdef VERBOSE
-    DEBUGOUT(_LtDebug(__FILE__, (Widget)w,
-		      "DrawText: x %d y %d '%s' len %d hl %d\n",
-		      x, y, (ptr == 0) ? "(NULL)" : ptr, len, highlight));
+	DEBUGOUT(_LtDebug(__FILE__, (Widget)w,
+		"DrawText: x %d y %d '%s' len %d hl %d\n",
+		x, y, (ptr == 0) ? "(NULL)" : ptr, len, highlight));
 #endif
 
-    xoff = Out_LeftMargin(o) - Out_XOffset(o);
+	xoff = Out_LeftMargin(o) - Out_XOffset(o);
 
-#ifdef	DANNY
-    fprintf(stderr, "XmText DrawText(%s dpy %p)  x %d y %d '%s' len %d hl %d, fh %d\n",
-		XtName(w), XtDisplay(w),
+#if 1
+	DEBUGOUT(_LtDebug(__FILE__, (Widget)w,
+		"XmText DrawText x %d y %d '%s' len %d hl %d, fh %d\n",
 		x + xoff, y, (ptr == 0) ? "(NULL)" : ptr, len, highlight,
-		Out_FontHeight(o));
+		Out_FontHeight(o)));
 #endif
 
-    /* Only change the GC if we need to */
-    if (highlight == XmHIGHLIGHT_SELECTED && !Out_DrawGCInverted(o))
-    {
-	XSetForeground(XtDisplay(w), Out_DrawGC(o), XtBackground(w));
-	XSetBackground(XtDisplay(w), Out_DrawGC(o), Prim_Foreground(w));
-	Out_DrawGCInverted(o) = True;
-    }
-    else if (Out_DrawGCInverted(o) && highlight != XmHIGHLIGHT_SELECTED)
-    {
-	XSetForeground(XtDisplay(w), Out_DrawGC(o), Prim_Foreground(w));
-	XSetBackground(XtDisplay(w), Out_DrawGC(o), XtBackground(w));
-	Out_DrawGCInverted(o) = False;
-    }
+	fg = Prim_Foreground(w);
+	bg = XtBackground(w);
 
-    while (len > 0 && *ptr)
-    {
-	char *start;
-	Dimension piecewidth;
-	int count;
-
-	start = ptr;
-	count = 0;
-	while (len > 0 && *ptr && *ptr != '\t' && *ptr != '\n' )
-	{
-	    ptr++, count++, len--;
+	/* Only change the GC if we need to */
+	if (highlight == XmHIGHLIGHT_SELECTED && !Out_DrawGCInverted(o)) {
+		fg = XtBackground(w);
+		bg = Prim_Foreground(w);
+		Out_DrawGCInverted(o) = True;
+	} else if (Out_DrawGCInverted(o) && highlight != XmHIGHLIGHT_SELECTED) {
+		fg = Prim_Foreground(w);
+		bg = XtBackground(w);
+		Out_DrawGCInverted(o) = False;
 	}
 
-	if (count > 0)
-	{
-	    piecewidth = _XmTextNextX(w, 0, start, count);
+	while (len > 0 && *ptr) {
+		char *start;
+		Dimension piecewidth, pieceheight;
+		int count;
 
-	    if (count > 0)
-	    {
-		if (Out_FontList(o)->renditions[0]->type == XmFONT_IS_FONT) {
-			XDrawString(XtDisplay(w), XtWindow(w), Out_DrawGC(o),
-				 x + xoff, y, start, count);
-		} else if (Out_FontList(o)->renditions[0]->type == XmFONT_IS_FONTSET) {
-			XmbDrawString(XtDisplay(w), XtWindow(w),
-				(XFontSet)Out_FontList(o)->renditions[0]->font,
-				Out_DrawGC(o),
-				x + xoff, y,
-				start, count);
-		} else {
-			/* ??? */
+		start = ptr;
+		count = 0;
+		while (len > 0 && *ptr && *ptr != '\t' && *ptr != '\n' ) {
+			ptr++, count++, len--;
 		}
 
-	    }
+		if (count > 0) {
+			piecewidth = _XmTextNextX(w, 0, start, count);
+			pieceheight = 10;	/* FIX ME */
+
+			if (count > 0) {
+#if 1
+				/* This is for bug #558840.
+				 * Can be tested in test/Xm/text/test1 by double-clicking.
+				 */
+				unsigned long	mask = GCForeground | GCBackground;
+				XGCValues	v;
+
+				v.foreground = bg;
+				v.background = fg;
+				XChangeGC(XtDisplay(w), Out_DrawGC(o), mask, &v);
+				XFillRectangle(XtDisplay(w), XtWindow(w), Out_DrawGC(o),
+					x + xoff, y - o->font_ascent,
+					piecewidth, y + o->font_descent);
+
+				v.background = bg;
+				v.foreground = fg;
+				XChangeGC(XtDisplay(w), Out_DrawGC(o), mask, &v);
+#endif
+				if (Out_FontList(o)->renditions[0]->type == XmFONT_IS_FONT) {
+					XDrawString(XtDisplay(w), XtWindow(w), Out_DrawGC(o),
+						 x + xoff, y, start, count);
+				} else if (Out_FontList(o)->renditions[0]->type == XmFONT_IS_FONTSET) {
+					XmbDrawString(XtDisplay(w), XtWindow(w),
+						(XFontSet)Out_FontList(o)->renditions[0]->font,
+						Out_DrawGC(o),
+						x + xoff, y,
+						start, count);
+#ifdef	USE_XFT
+				} else if (Out_XftFont(o)) {
+					_XmXftDrawString(XtDisplay(w), XtWindow(w),
+						Out_FontList(o)->renditions[0],
+						1,
+						x + xoff, y,
+						start, count);
+#endif
+				} else {
+					/* ??? */
+				}
+
+			}
 
 #ifdef VERBOSE
 	    if (_LtDebugInDebug(__FILE__, (Widget)w))
@@ -1091,8 +1212,8 @@ DrawText(XmTextWidget w, int x, int y, char *ptr, int len,
 	    }
 #endif
 
-	    x += piecewidth;
-	}
+			x += piecewidth;
+		}
 
 	if (len > 0 && *ptr == '\t')
 	{
@@ -2166,19 +2287,33 @@ OutputInvalidate(XmTextWidget w, XmTextPosition start, XmTextPosition end,
 static void
 GetPreferredSize(Widget aw, Dimension *width, Dimension *height)
 {
-    XmTextWidget w = (XmTextWidget)aw;
-    OutputData o = Text_OutputData(w);
+	XmTextWidget	w = (XmTextWidget)aw;
+	OutputData	o = Text_OutputData(w);
 
-#ifdef USE_AVERAGE_WIDTH
-    *width = Out_LeftMargin(o) + Out_RightMargin(o) +
-	      Out_Columns(o) * Out_FontAverageWidth(o) ;
-#else
-    *width = Out_LeftMargin(o) + Out_RightMargin(o) +
-	      Out_Columns(o) * Out_FontMaxWidth(o) ;
+	*height = Out_TopMargin(o) + Out_BottomMargin(o) +
+		Out_Rows(o) * Out_FontHeight(o);
+
+#ifdef	USE_XFT
+	if (Out_XftFont(o)) {
+		*width = Out_LeftMargin(o) + Out_RightMargin(o) +
+			Out_Columns(o) * Out_FontAverageWidth(o);
+		DEBUGOUT(_LtDebug(__FILE__, aw, "GetPreferredSize (fw %d )-> wid %d ht %d\n",
+			Out_FontAverageWidth(o), *width, *height));
+	} else
 #endif
-
-    *height = Out_TopMargin(o) + Out_BottomMargin(o) +
-	       Out_Rows(o) * Out_FontHeight(o);
+	{
+#ifdef USE_AVERAGE_WIDTH
+		*width = Out_LeftMargin(o) + Out_RightMargin(o) +
+			Out_Columns(o) * Out_FontAverageWidth(o) ;
+		DEBUGOUT(_LtDebug(__FILE__, aw, "GetPreferredSize (fw %d )-> wid %d ht %d\n",
+				Out_FontAverageWidth(o), *width, *height));
+#else
+		*width = Out_LeftMargin(o) + Out_RightMargin(o) +
+			Out_Columns(o) * Out_FontMaxWidth(o) ;
+		DEBUGOUT(_LtDebug(__FILE__, aw, "GetPreferredSize (fw %d )-> wid %d ht %d\n",
+				Out_FontMaxWidth(o), *width, *height));
+#endif
+	}
 }
 
 
@@ -2292,9 +2427,8 @@ OutputSetValues(Widget old, Widget request, Widget new_w,
 	   But only free it if we created it....
 	   xmgrace Data->Results
 	*/
-	if (Out_FontListCreated(od) && Out_FontList(od))
-	{
-	    XmFontListFree(Out_FontList(od));
+	if (Out_FontListCreated(od) && Out_FontList(od)) {
+		XmFontListFree(Out_FontList(od));
 	}
 
 	FontInit((XmTextWidget)new_w);
@@ -2319,13 +2453,27 @@ OutputSetValues(Widget old, Widget request, Widget new_w,
 	     */
 	    XGCValues	values;
 
-	    values.font = Out_Font(nd)->fid;
-	    XChangeGC(XtDisplay(new_w), Out_DrawGC(nd), GCFont, &values);
+	    if (Out_FontType(nd) == XmFONT_IS_FONT) {
+	    	values.font = Out_Font(nd)->fid;
+		XChangeGC(XtDisplay(new_w), Out_DrawGC(nd), GCFont, &values);
+	    } else
+#ifdef	USE_XFT
+	    if (Out_FontType(nd) == XmFONT_IS_XFT) {
+	    	/* Probably nothing to do here */
+	    }
+#endif
 
-	    /* We may still need to do things here.
-	     * CP Hennessy suggest calling
-		GCClip((XmTextWidget)new_w);
-	     */
+	    {
+		/*
+		 * Need braces here to provide an 'else' statement
+		 * in case that USE_XFT is not defined.
+		 */
+
+		/* We may still need to do things here.
+		 * CP Hennessy suggest calling
+			GCClip((XmTextWidget)new_w);
+		 */
+	    }
 	}
 
 	redraw = True;
@@ -2367,13 +2515,23 @@ OutputSetValues(Widget old, Widget request, Widget new_w,
 	DEBUGOUT(_LtDebug(__FILE__, new_w, "OutputSetValues: Columns change.\n"));
 
 	/* FIX ME ? */
-#ifdef USE_AVERAGE_WIDTH
-	XtWidth(new_w) = Out_Columns(nd) * Out_FontAverageWidth(nd)
-		+ Out_LeftMargin(nd) + Out_RightMargin(nd);
-#else
-	XtWidth(new_w) = Out_Columns(nd) * Out_FontMaxWidth(nd)
-		+ Out_LeftMargin(nd) + Out_RightMargin(nd);
+#ifdef	USE_XFT
+	if (Out_XftFont(nd)) {
+		XtWidth(new_w) = Out_Columns(nd) * Out_XftFont(nd)->max_advance_width
+			+ Out_LeftMargin(nd) + Out_RightMargin(nd);
+		XtWidth(new_w) = Out_Columns(nd) * Out_FontAverageWidth(nd)
+			+ Out_LeftMargin(nd) + Out_RightMargin(nd);	/* FIX ME HACK */
+	} else
 #endif
+	{
+#ifdef USE_AVERAGE_WIDTH
+		XtWidth(new_w) = Out_Columns(nd) * Out_FontAverageWidth(nd)
+			+ Out_LeftMargin(nd) + Out_RightMargin(nd);
+#else
+		XtWidth(new_w) = Out_Columns(nd) * Out_FontMaxWidth(nd)
+			+ Out_LeftMargin(nd) + Out_RightMargin(nd);
+#endif
+	}
 
 	redraw = True;
     }
@@ -2493,6 +2651,9 @@ destroy(Widget aw)
     {
 	XmFontListFree(Out_FontList(o));
     }
+#ifdef	USE_XFT
+	/* FIX ME */
+#endif
 
     XtFree((char *)Text_Output(w));
 }
@@ -2501,15 +2662,14 @@ destroy(Widget aw)
 static void
 OutputExpose(Widget aw, XEvent *event, Region region)
 {
-    XmTextWidget w = (XmTextWidget)aw;
+	XmTextWidget w = (XmTextWidget)aw;
 
-    DEBUGOUT(_LtDebug(__FILE__, aw, "OutputExpose: cursor pos: %d\n",
-		      Text_CursorPos(w)));
+	DEBUGOUT(_LtDebug(__FILE__, aw, "OutputExpose: cursor pos: %d\n",
+		Text_CursorPos(w)));
 
-    if (!XtIsRealized(aw))
-    {
-	return;
-    }
+	if (!XtIsRealized(aw)) {
+		return;
+	}
 
 /*
     XClearArea(XtDisplay(w), XtWindow(w),
@@ -2569,28 +2729,22 @@ OutputRealize(Widget aw, XtValueMask *value_mask, XSetWindowAttributes *attribut
 static void
 resize(Widget aw, Boolean state)
 {
-    XmTextWidget w = (XmTextWidget)aw;
-    OutputData o = Text_OutputData(w);
+	XmTextWidget w = (XmTextWidget)aw;
+	OutputData o = Text_OutputData(w);
 
-    SizeRecalc(w);
+	SizeRecalc(w);
 
-    /* SG (15/08/1998) Update scrollbars immediately on resize
-       to give the correct visual feedback to the user and keep
-       scroll bars in sync with any keyboard actions after this resize */
-
-    _XmChangeVSB(w, Text_TopPos(w));
-    {
-    OutputData o = Text_OutputData(w);
+	/* SG (15/08/1998) Update scrollbars immediately on resize
+	 * to give the correct visual feedback to the user and keep
+	 * scroll bars in sync with any keyboard actions after this resize */
 
+	_XmChangeVSB(w, Text_TopPos(w));
         _XmRedisplayHBar(w, Out_XOffset(o));
-    }
 
-
-    /* Only adjust the GCs if they have already been created! */
-    if (Out_DrawGC(o))
-    {
-	GCClip(w);
-    }
+	/* Only adjust the GCs if they have already been created! */
+	if (Out_DrawGC(o)) {
+		GCClip(w);
+	}
 }
 
 
@@ -2692,6 +2846,17 @@ _XmTextOutputCreate(Widget aw, ArgList args, Cardinal num_args)
     else
     {
 	/* Reverse of this thing also copied into OutputSetValues */
+#ifdef	USE_XFT
+	if (Out_XftFont(o)) {
+		Out_Columns(o) = (XtWidth(w) -
+			(Out_LeftMargin(o) + Out_RightMargin(o)))
+			/ Out_XftFont(o)->max_advance_width;
+		Out_Columns(o) = (XtWidth(w) -
+			(Out_LeftMargin(o) + Out_RightMargin(o)))
+			/ Out_FontAverageWidth(o);	/* FIX ME HACK */
+	} else
+#endif
+	{
 #ifdef USE_AVERAGE_WIDTH
 	Out_Columns(o) = (XtWidth(w) - ( Out_LeftMargin(o) + Out_RightMargin(o) )
 		) / Out_FontAverageWidth(o);
@@ -2699,6 +2864,7 @@ _XmTextOutputCreate(Widget aw, ArgList args, Cardinal num_args)
 	Out_Columns(o) = (XtWidth(w) - ( Out_LeftMargin(o) + Out_RightMargin(o) )
 		) / Out_FontMaxWidth(o);
 #endif
+	}
     }
 
     if (XtHeight(w) == (Dimension)0)
@@ -2733,14 +2899,10 @@ _XmTextOutputCreate(Widget aw, ArgList args, Cardinal num_args)
     Out_ColumnsSet(o) = Out_Columns(o);
     Out_RowsSet(o) = Out_Rows(o);
 
-    if (XtWidth(w) == 0 || XtHeight(w) == 0)
-    {
+    if (XtWidth(w) == 0 || XtHeight(w) == 0) {
 	(*Text_Output(w)->GetPreferredSize) (aw, &width, &height);
 	XtWidth(w) = width;
 	XtHeight(w) = height;
-
-	DEBUGOUT(_LtDebug(__FILE__, aw, "GetPreferredSize -> wid %d ht %d\n",
-			  width, height));
     }
 
     SizeRecalc(w);
@@ -3073,7 +3235,7 @@ _XmTextFindLineEnd(XmTextWidget w, XmTextPosition pos, LineTableExtra *extra)
 	for(i=pos+1; i<end; i++){
 	    oldwidth = width;
 	    width = FontTextWidth(w, pos, i);
-	    if (width > avail){
+	    if (width >= avail){
 		last = i - 1;
 		break;
 	    }
@@ -3130,15 +3292,19 @@ _XmTextGetBaselines(Widget widget, Dimension **baselines, int *line_count)
 extern int
 _XmOut_FontTextWidth(OutputData o, char *s, int l)
 {
+	if (Out_FontList(o)->renditions[0]->type == XmFONT_IS_FONT) {
+		return XTextWidth(Out_Font(o), s, l);
+#ifdef	USE_XFT
+	} else if (Out_XftFont(o)) {
+		XGlyphInfo	ext;
+		XftTextExtents8(Out_FontList(o)->renditions[0]->dpy,
+			Out_XftFont(o), s, l, &ext);
 
-    if (Out_FontList(o)->renditions[0]->type == XmFONT_IS_FONT)
-    {
-	return XTextWidth(Out_Font(o), s, l);
-    }
-    else
-    {
-	return XmbTextEscapement((XFontSet)Out_FontList(o)->renditions[0]->font, s, l);
-    }
+		return ext.width;
+#endif
+	} else {
+		return XmbTextEscapement((XFontSet)Out_FontList(o)->renditions[0]->font, s, l);
+	}
 
 }
 
@@ -3147,24 +3313,48 @@ _XmOut_FontTextWidth(OutputData o, char *s, int l)
 extern int
 _XmOut_FontMaxWidth(OutputData o)
 {
+	if (Out_FontList(o)->renditions[0]->type == XmFONT_IS_FONT) {
+		return Out_Font(o)->max_bounds.width;
+	} else
+#ifdef	USE_XFT
+	if (Out_FontList(o)->renditions[0]->type == XmFONT_IS_XFT) {
+		return Out_FontAverageWidth(o);	/* FIX ME HACK */
+		return Out_XftFont(o)->max_advance_width;
+	} else
+#endif
+	{
+		XFontStruct **fsl;
+		char **nl;
+		int i, num, max;
 
-    if (Out_FontList(o)->renditions[0]->type == XmFONT_IS_FONT)
-    {
-	return Out_Font(o)->max_bounds.width;
-    }
-    else
-    {
-	XFontStruct **fsl;
-	char **nl;
-	int i, num, max;
+		num = XFontsOfFontSet((XFontSet)Out_FontList(o)->renditions[0]->font, &fsl, &nl);
+		max = 0;
+		for(i=0; i<num; i++){
+			if (fsl[i]->max_bounds.width  > max)
+				max = fsl[i]->max_bounds.width;
+		}
+		return max;
+	}
+}
 
-	num = XFontsOfFontSet((XFontSet)Out_FontList(o)->renditions[0]->font, &fsl, &nl);
-	max = 0;
-	for(i=0; i<num; i++){
-	    if (fsl[i]->max_bounds.width  > max)
-		    max = fsl[i]->max_bounds.width;
+static void
+_XmTextOutSetRenderTable(Widget w, int ofs, XrmValue *v)
+{
+	OutputData o = Text_OutputData(w);
+
+	DEBUGOUT(_LtDebug(__FILE__, w, "_XmTextOutSetRenderTable (%d)\n",
+		o->check_set_render_table));
+
+	if (o->check_set_render_table) {
+		/*
+		 * This is the second time we're getting called.
+		 * Therefore the result is undefined.
+		 */
+		v->addr = NULL;	/* FIX ME or should we pass nothing ? */
+	} else {
+		v->addr = (char *)&(o->fontlist);
+		o->check_set_render_table = True;
+		DEBUGOUT(_LtDebug(__FILE__, w, "_XmTextOutSetRenderTable rt %p\n",
+			o->fontlist));
 	}
-	return max;
-    }
-
 }
