@@ -1,9 +1,9 @@
 /**
  *
- * $Header: /cvsroot/lesstif/lesstif/lib/Xm/TextStrSo.c,v 1.58 2001/11/26 19:40:24 dannybackx Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm/TextStrSo.c,v 1.60 2002/07/25 20:11:59 dannybackx Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
- * Copyright  1995, 1996, 1997, 1998, 1999, 2000, 2001 LessTif Development Team
+ * Copyright  1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: TextStrSo.c,v 1.58 2001/11/26 19:40:24 dannybackx Exp $";
+static const char rcsid[] = "$Id: TextStrSo.c,v 1.60 2002/07/25 20:11:59 dannybackx Exp $";
 
 /*
  * Empirical evidence (fooling around with the structures found in Motif
@@ -325,6 +325,10 @@ Replace(XmTextWidget w,
     XmTextPosition start, end, lastPos = 0;
     XmTextVerifyCallbackStruct cbs;
 
+	DEBUGOUT(_LtDebug(__FILE__, (Widget)w,
+		"XmTextStrSrc-Replace(start %d, end %d, call %d)\n",
+		*startret, *endret, call_callback));
+
     if (!call_callback || _XmTextModifyVerify(w, ev, startret, endret, &Text_CursorPos(w), block, NULL, False))
     {
 	d = Text_Source(w)->data;
@@ -460,15 +464,12 @@ Replace(XmTextWidget w,
 	    XtCallCallbacks((Widget)w, XmNvalueChangedCallback, &cbs);
 	}
 
-	for (i = 0; i < d->numwidgets; i++)
-	{
-	    _XmTextUpdateLineTable((Widget)d->widgets[i], start, end, block, True);
+	for (i = 0; i < d->numwidgets; i++) {
+		_XmTextUpdateLineTable((Widget)d->widgets[i], start, end, block, True);
 	}
 
-	for (i = 0; i < d->numwidgets; i++)
-	{
-	    _XmTextInvalidate(d->widgets[i], start, end,
-			      block->length - end + start);
+	for (i = 0; i < d->numwidgets; i++) {
+		_XmTextInvalidate(d->widgets[i], start, end, block->length - end + start);
 	}
 
 #if 1
@@ -916,10 +917,14 @@ SetSelection(XmTextSource source, XmTextPosition left,
     }
 
     /* Widgets */
-    for (i = 0; i < d->numwidgets; i++)
-    {
+    for (i = 0; i < d->numwidgets; i++) {
+	/* Unset the highlight over the complete widget */
+	XmTextSetHighlight((Widget)d->widgets[i],
+		0, XmTextGetLastPosition((Widget)d->widgets[0]),
+		XmHIGHLIGHT_NORMAL);
+	/* Highlight the selected area */
 	XmTextSetHighlight((Widget)d->widgets[i], left, right,
-			   XmHIGHLIGHT_SELECTED);
+		XmHIGHLIGHT_SELECTED);
     }
 }
 
