@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/lesstif/lesstif/lib/Xm/Vendor.c,v 1.92 2002/03/23 18:24:41 dannybackx Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm/Vendor.c,v 1.94 2003/02/05 20:50:56 dannybackx Exp $
  *
  * Copyright (C) 1996 Free Software Foundation, Inc.
  * Copyright  1996, 1997, 1998, 1999, 2000, 2001, 2002 LessTif Development Team
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Vendor.c,v 1.92 2002/03/23 18:24:41 dannybackx Exp $";
+static const char rcsid[] = "$Id: Vendor.c,v 1.94 2003/02/05 20:50:56 dannybackx Exp $";
 
 #include <LTconfig.h>
 
@@ -1652,6 +1652,8 @@ LTRemoveGrab(Widget wid, XmVendorShellExtObject ve,
 
     for (i = Display_NumModals(d); i > 0; i--, ps++)
     {
+	DEBUGOUT(_LtDebug2("GRAB", XtParent(wid), wid, "NEDIT\n"));
+
 /* amai: The famous "NEdit freeze" bug (an out of sync problem between
          the Xt and LT grab list) was cured by changing
            if ((ps->wid == wid) && !CoreBeingDestroyed(wid) && remove_grab_physically)
@@ -1661,15 +1663,95 @@ LTRemoveGrab(Widget wid, XmVendorShellExtObject ve,
 	This 'fix' in turn triggered XtRemoveGrabs with bogus arguments, leading
 	to Xt warning messages. See comments at beginning of LTRemoveGrab()!
 */
+#if 1
+	/* 1.83 version */
+	if ((XtParent(wid) == NULL) ||
+			((ps->wid == wid) && !CoreBeingDestroyed(wid) &&
+			!XtIsTopLevelShell(wid) && remove_grab_physically)) {
+		XtRemoveGrab(wid);
+	}
+#endif
+#if 0
+	/* This has all kinds of experiments on how to remove grabs.
+	 * Basically the code was changed away from the 1.83 version
+	 * because of bug #461658 "Fix for annoying XtRemoveGrab Messages".
+	 * After that, nedit started to experience hangs, as described in
+	 * bug #566315 "NEdit - new window ignores all input",
+	 * and reproduced in test/Xm/menushell/test10 .
+	 * For now (5 february 2003), the 1.83 version - above - still
+	 * appears to give the best results.
+	 */
+#if 1
+	if (XtParent(wid) == NULL ||
+	( /* (ps->wid == wid) &&  */
+	  (XmIsDialogShell(wid) || XmIsMenuShell(wid))
+	 && !CoreBeingDestroyed(wid) && remove_grab_physically))
+#else
+#if 1
 	if ((ps->wid == wid) && !CoreBeingDestroyed(wid) &&
 	     !XtIsTopLevelShell(wid) && remove_grab_physically)
+#else
+	if ((ps->wid == wid) && remove_grab_physically)
+#endif
+#endif
 	{
-	    DEBUGOUT(_LtDebug2("GRAB", XtParent(wid), wid, "%s:XtRemoveGrab(%d) - %s %s\n",
-	                       __FILE__, __LINE__,
-			       XtIsShell(wid) ? "True" : "False",
-                               CoreBeingDestroyed(wid) ? "True" : "False"));
+//	    DEBUGOUT(_LtDebug2("GRAB", XtParent(wid), wid, "%s:XtRemoveGrab(%d) - %s %s\n",
+//				__FILE__, __LINE__,
+//				XtIsShell(wid) ? "True" : "False",
+//				CoreBeingDestroyed(wid) ? "True" : "False"));
+		DEBUGOUT(_LtDebug2("GRAB", XtParent(wid), wid,
+			"XtRemoveGrab [pswid %p wid %p shell %s CoreDB %s Tls %s removephys %s]\n",
+			ps->wid, wid,
+			XtIsShell(wid) ? "True" : "False",
+			CoreBeingDestroyed(wid) ? "True" : "False",
+			XtIsTopLevelShell(wid) ? "True" : "False",
+			remove_grab_physically ? "True" : "False"
+			));
 	    XtRemoveGrab(wid);
 	}
+#if 0
+	/* Danny */
+	else {
+		DEBUGOUT(_LtDebug2("GRAB", XtParent(wid), wid,
+			"NEDIT didn't XtRemoveGrab [pswid %p wid %p shell %s CoreDB %s Tls %s removephys %s]\n",
+			ps->wid, wid,
+			XtIsShell(wid) ? "True" : "False",
+			CoreBeingDestroyed(wid) ? "True" : "False",
+			XtIsTopLevelShell(wid) ? "True" : "False",
+			remove_grab_physically ? "True" : "False"
+			));
+		XtRemoveGrab(wid);
+	}
+#endif
+#if 1
+	/* HACK */
+	else if (ps->wid == 0x8063180 && wid == 0x80a3298 && CoreBeingDestroyed(wid) == False) {
+		DEBUGOUT(_LtDebug2("GRAB", XtParent(wid), wid,
+			"Yow - NEDIT didn't XtRemoveGrab [pswid %p wid %p shell %s CoreDB %s Tls %s removephys %s] %s %s %s %s\n",
+			ps->wid, wid,
+			XtIsShell(wid) ? "True" : "False",
+			CoreBeingDestroyed(wid) ? "True" : "False",
+			XtIsTopLevelShell(wid) ? "True" : "False",
+			remove_grab_physically ? "True" : "False",
+			XtName(XtParent(wid)), XtName(ps->wid), XtName(wid),
+			wid->core.widget_class->core_class.class_name
+			));
+		XtRemoveGrab(wid);
+	} else {
+		DEBUGOUT(_LtDebug2("GRAB", XtParent(wid), wid,
+			"Yow Noppes - NEDIT didn't XtRemoveGrab [pswid %p wid %p shell %s DLS %s CoreBD %s Tls %s removephys %s] %s %s %s %s\n",
+			ps->wid, wid,
+			XtIsShell(wid) ? "True" : "False",
+			XmIsDialogShell(wid) ? "True" : "False",
+			CoreBeingDestroyed(wid) ? "True" : "False",
+			XtIsTopLevelShell(wid) ? "True" : "False",
+			remove_grab_physically ? "True" : "False",
+			XtName(XtParent(wid)), XtName(ps->wid), XtName(wid),
+			wid->core.widget_class->core_class.class_name
+			));
+	}
+#endif
+#endif
     }
 
     ps = pd;
