@@ -1,8 +1,9 @@
 /**
  *
- * $Id: XmStrDefs.c,v 1.4 1999/08/12 12:09:17 danny Exp $
+ * $Id: XmStrDefs.c,v 1.8 2001/05/14 19:46:11 amai Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
+ * Copyright (C) 1995-2001 LessTif Development Team
  *
  * This file is part of the GNU LessTif Library.
  *
@@ -22,19 +23,24 @@
  *
  **/
 
-#include <LTconfig.h>
-#include <XmI/XmI.h>
+static const char rcsid[] = "$Header: /cvsroot/lesstif/lesstif/lib/Xm/XmStrDefs.c,v 1.8 2001/05/14 19:46:11 amai Exp $";
 
-#include <Xm/XmP.h>
-#if XtSpecificationRelease < 6
-#include <X11/IntrinsicP.h>
-#endif
+/*
+ * This file contains the _XmStrings array for Motif 1.2 libXm
+ */
+
+
+#include <LTconfig.h>
+
+/*
+#include <XmI/XmI.h>
+*/
 
 /*
  * The missing commas are intentional.  Let the compiler
  * concatenate the strings
  */
-_XmConst char _XmStrings[] =
+const char _XmStrings[] =
 /* 0     */ "\0"
 /* 1     */ "Accelerator\0"
 /* 13    */ "AcceleratorText\0"
