@@ -1,6 +1,6 @@
 /**
  *
- * $Header: /cvsroot/lesstif/lesstif/lib/Xm/Xmos.c,v 1.46 2002/04/12 23:38:44 amai Exp $
+ * $Header: /cvsroot/lesstif/lesstif/lib/Xm/Xmos.c,v 1.47 2002/07/26 16:15:12 dannybackx Exp $
  *
  * Copyright (C) 1995 Free Software Foundation, Inc.
  * Copyright (C) 1995-2002 LessTif Development Team
@@ -23,7 +23,7 @@
  *
  **/
 
-static const char rcsid[] = "$Id: Xmos.c,v 1.46 2002/04/12 23:38:44 amai Exp $";
+static const char rcsid[] = "$Id: Xmos.c,v 1.47 2002/07/26 16:15:12 dannybackx Exp $";
 
 #include <LTconfig.h>
 
@@ -857,11 +857,15 @@ _XmOSBuildFileList(String dirPath,
     }
 
     i = _Lesstif_glob(buf, flags, NULL, &result);
-    if (i)
+    if (i) {
+	DEBUGOUT(_LtDebug0(__FILE__, NULL, "WARNING: _XmOSBuildFileList() return after glob\n"));
 	return;
+    }
 
     DEBUGOUT(_LtDebug0(__FILE__, NULL,
-	"WARNING: _XmOSBuildFileList: Resetting initial list\n"));
+	"WARNING: _XmOSBuildFileList: Resetting initial list (glob count %d)\n",
+	result.gl_pathc));
+
     *pNumAlloc = *pNumEntries = 0;
     *pEntries = NULL;
 
@@ -885,8 +889,8 @@ _XmOSBuildFileList(String dirPath,
 	    ret = (String *)XtRealloc((char *)ret, max * sizeof(String *));
 	}
 
-	if (typeMask & XmFILE_REGULAR &&
-	    result.gl_pathv[i][strlen(result.gl_pathv[i]) - 1] != '/')
+	if ((typeMask & XmFILE_REGULAR) &&
+	    (result.gl_pathv[i][strlen(result.gl_pathv[i]) - 1] != '/'))
 	{
 	    if ((ret[cnt] = XtNewString(result.gl_pathv[i])) == NULL)
 	    {
@@ -895,8 +899,8 @@ _XmOSBuildFileList(String dirPath,
 	    cnt++;
 	}
 
-	if (typeMask & XmFILE_DIRECTORY &&
-	    result.gl_pathv[i][strlen(result.gl_pathv[i]) - 1] == '/')
+	if ((typeMask & XmFILE_DIRECTORY) &&
+	    (result.gl_pathv[i][strlen(result.gl_pathv[i]) - 1] == '/'))
 	{
 	    if (result.gl_pathv[i][strlen(result.gl_pathv[i]) - 1] == '/')
 	    {
