@@ -1,11 +1,15 @@
+/* $Header: /cvsroot/lesstif/lesstif/test/Xm/label/test1.c,v 1.10 2002/05/01 15:39:21 amai Exp $ */
 /* test for multi font labels */
 
+#include <stdlib.h>
+#include <stdio.h>
+
 #include <Xm/Label.h>
 #include <Xm/LabelP.h>
-#include <stdio.h>
+
 
 #include "../../common/Test.h"
-#include "../../common/DumpLayout.h"
+
 
 int
 main(int argc, char **argv)
